/**
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): Adrien LOUIS
 * --------------------------------------------------------------------------
 * $Id: SourceHelper.java 94 2006-03-26 17:11:05Z alouis $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.util;

import java.io.ByteArrayInputStream;
import java.io.DataInputStream;

import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;

/**
 * Sample class used to manage the mapping between
 * <code>javax.xml.transform.Source</code> from String. the mapping is a
 * quicly "string" -> "<content>string</content>" conversion.
 * 
 * @author Adrien LOUIS - EBM WebSourcing
 */
public final class SourceHelper {

    private SourceHelper() {
        // avoid to be built
    }

    public static Source createSource(String msg) {
        StreamSource source = new StreamSource();

        byte[] msgByte = msg.getBytes();

        ByteArrayInputStream in = new ByteArrayInputStream(msgByte);

        source.setInputStream(in);

        return source;
    }

    public static String createString(Source s) throws Exception {
        if (!(s instanceof StreamSource)) {
            throw new IllegalArgumentException(
                "Source is null or is not a streamsource");
        }
        DataInputStream bio = new DataInputStream(((StreamSource) s)
            .getInputStream());
        byte[] buff = new byte[bio.available()];
        bio.readFully(buff);
        bio.reset();
        bio.close();
        return new String(buff);
    }

}
