/**
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: MX4JRMIAdaptor.java,v 1.2 2005/07/22 10:24:27 ddesjardins Exp $
 * --------------------------------------------------------------------------  
 */
package org.objectweb.petals.kernel.jmx;

import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;

import javax.management.remote.JMXConnectorServer;
import javax.management.remote.JMXConnectorServerFactory;
import javax.management.remote.JMXServiceURL;
import javax.management.remote.rmi.RMIConnectorServer;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.jmx.agent.AdminAttributes;
import org.objectweb.petals.util.InterfaceNamesHelper;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.petals.util.SystemUtil;
import org.objectweb.util.monolog.api.Logger;

import static org.objectweb.petals.util.InterfaceNamesHelper.JMXAGENT_ADMINATT_NAME;
import static org.objectweb.petals.util.InterfaceNamesHelper.LOGGER_NAME;

/**
 * This is the Fractal JMX adaptor for use with MX4J tools.
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class RMIConnector implements BindingController, LifeCycleController {

    /**
     * AdminAttributes
     */
    protected AdminAttributes adminAttributes;

    /**
     * Logger
     */
    protected LoggingUtil log;

    /**
     * Logger
     */
    protected Logger logger;

    /**
     * JMX server
     */
    protected JMXConnectorServer cs;

    /**
     * Fractal Component state.
     */
    private String fcState;

    /**
     * @see BindingController#bindFc(String, Object)
     */
    public void bindFc(final String itfName, final Object itfValue)
        throws NoSuchInterfaceException {
        if (LOGGER_NAME.equals(itfName)) {
            logger = (Logger) itfValue;
            log = new LoggingUtil(logger);
        } else if (JMXAGENT_ADMINATT_NAME.equals(itfName)) {
            adminAttributes = (AdminAttributes) itfValue;
        } else {
            throw new NoSuchInterfaceException(itfName);
        }
    }

    public String getFcState() {
        return fcState == STARTED ? STARTED : STOPPED;
    }

    public String[] listFc() {
        return new String[] {JMXAGENT_ADMINATT_NAME};
    }

    /**
     * @see BindingController#lookupFc(String)
     */
    public Object lookupFc(final String itfName)
        throws NoSuchInterfaceException {
        if (JMXAGENT_ADMINATT_NAME.equals(itfName)) {
            return adminAttributes;
        } else if (InterfaceNamesHelper.LOGGER_NAME.equals(itfName)) {
            return logger;
        } else {
            throw new NoSuchInterfaceException(itfName);
        }
    }

    /**
     * Start the JMX RMI Server
     */
    public void startFc() {
        log.start();
        fcState = STARTED;
        try {
            // Retreive the port and host properties
            int port = Integer.parseInt(SystemUtil.getJmxPort());
            String host = SystemUtil.getHost();

            if (cs != null && cs.isActive()
                && cs.getAddress().getPort() == port) {
                return;
            }
            if (cs != null) {
                cs.stop();
            }
            // create an rmi connector
            Registry registry = LocateRegistry.getRegistry(port);
            try {
                registry.list();
            } catch (Exception e) {
                LocateRegistry.createRegistry(port);
            }
            String sJmxURL = "service:jmx:rmi:///jndi/rmi://" + host + ":"
                + port + "/management/rmi-jmx-connector";
            JMXServiceURL jmxUrl = new JMXServiceURL(sJmxURL);
            // Server with security (need keystore that are not yet done)
            // HashMap<String, Object> env = new HashMap<String, Object>();
            // env.put("jmx.remote.x.password.file", "conf" + File.separator
            // + "password.properties");
            // env.put("jmx.remote.x.access.file", "conf" + File.separator
            // + "access.properties");
            // RMIConnectorServer connectorServer = (RMIConnectorServer)
            // JMXConnectorServerFactory
            // .newJMXConnectorServer(jmxUrl, env, beanServer);

            // Server without security
            RMIConnectorServer connectorServer = (RMIConnectorServer) JMXConnectorServerFactory
                .newJMXConnectorServer(jmxUrl, null, adminAttributes
                    .getRawMBeanServer());
            cs = connectorServer;
            // Start the server
            cs.start();
            // Write the address of the server on the console
            log.info("JMX RMI server started at : "
                + cs.getAddress().getURLPath());
        } catch (Exception e) {
            log.error(
                "Error during the instanciation of the MX4J RMI adaptor:", e);
        }
        log.end();
    }

    /**
     * Stop the RMI Server
     */
    public void stopFc() {
        log.start();
        fcState = STOPPED;
        if (cs != null) {
            StopJMXServerThread stopJMXServerThread = new StopJMXServerThread(
                cs, log);
            stopJMXServerThread.start();
        }
        log.end();
    }

    /**
     * @see BindingController#unbindFc(String)
     */
    public void unbindFc(final String itfName) throws NoSuchInterfaceException {
        if (JMXAGENT_ADMINATT_NAME.equals(itfName)) {
            adminAttributes = null;
        } else if (LOGGER_NAME.equals(itfName)) {
            logger = null;
        } else {
            throw new NoSuchInterfaceException(itfName);
        }
    }

}
