/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JoramConfigurationBuilder.java 2:04:08 PM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.kernel.joram.util;

import java.util.ArrayList;
import java.util.List;

import org.objectweb.petals.util.XMLUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * This class is used to build the Joram configuration file a3servers.xml
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class JoramConfigurationBuilder {

    private JoramConfigurationBuilder() {
        // do nothing
    }

    /**
     * Build a joram configuration file (a3servers.xml) for a single Joram
     * server
     * 
     * @param domain
     *            domain of the server
     * @param id
     *            id of the server
     * @param name
     *            name of the server
     * @param host
     *            host of the server
     * @param username
     *            username to access to the admin module
     * @param password
     *            password to access to the admin module
     * @param domainPort
     *            port of the domain
     * @param tcpPort
     *            port for the tcp connector
     * @param uid
     *            uid of the server
     * @return xml String containing the configuration
     */
    public static String buildFullMasterConf(String domain, String id,
        String name, String host, String username, String password,
        String domainPort, String tcpPort, String uid) {
        StringBuffer xml = new StringBuffer();
        xml.append("<?xml version=\"1.0\"?>\n");
        xml.append("<!-- <!DOCTYPE config SYSTEM \"a3config.dtd\">-->\n");
        xml.append("<config>\n");
        xml.append("\t<domain name=\"" + domain + "\"/>\n");
        // xml
        // .append("\t<property name=\"Transaction\"
        // value=\"fr.dyade.aaa.util.NullTransaction\"/>\n");
        xml.append(buildMasterServerConf(domain, id, name, host, username,
            password, domainPort, tcpPort, uid));
        xml.append("</config>\n");

        return xml.toString();
    }

    /**
     * Build the Joram server element of the configuration file (a3servers.xml)
     * 
     * @param domain
     *            domain of the server
     * @param id
     *            id of the server
     * @param name
     *            name of the server
     * @param host
     *            host of the server
     * @param username
     *            username to access to the admin module
     * @param password
     *            password to access to the admin module
     * @param domainPort
     *            port of the domain
     * @param tcpPort
     *            port for the tcp connector
     * @param uid
     *            uid of the server
     * @return xml String representing the element server
     */
    protected static String buildMasterServerConf(String domain, String id,
        String name, String host, String username, String password,
        String domainPort, String tcpPort, String uid) {
        StringBuffer xml = new StringBuffer();
        xml.append("\t<server id=\"" + id + "\" name=\"" + name
            + "\" hostname=\"" + host + "\">\n");
        xml.append("\t\t<property name=\"uid\" value=\"" + uid + "\"/>\n");
        xml.append("\t\t<network domain=\"" + domain + "\" port=\""
            + domainPort + "\"/>\n");
        xml
            .append("\t\t<service class=\"org.objectweb.joram.mom.proxies.ConnectionManager\" args=\""
                + username + " " + password + "\"/>\n");
        xml
            .append("\t\t<service class=\"org.objectweb.joram.mom.proxies.tcp.TcpProxyService\" args=\""
                + tcpPort + "\"/>\n");
        // xml
        // .append("\t\t<service
        // class=\"fr.dyade.aaa.jndi2.distributed.DistributedJndiServer\"
        // args=\""
        // + jndiPort + "\"/>\n");
        xml.append("\t</server>\n");
        return xml.toString();
    }

    /**
     * Build the Joram server element of the configuration file (a3servers.xml)
     * 
     * @param domain
     *            domain of the server
     * @param id
     *            id of the server
     * @param name
     *            name of the server
     * @param host
     *            host of the server
     * @param username
     *            username to access to the admin module
     * @param password
     *            password to access to the admin module
     * @param domainPort
     *            port of the domain
     * @param tcpPort
     *            port for the tcp connector
     * @param uid
     *            uid of the server
     * @return xml String representing the element server
     */
    protected static String buildSlaveServerConf(String domain, String id,
        String name, String host, String username, String password,
        String domainPort, String tcpPort, String uid) {
        StringBuffer xml = new StringBuffer();
        xml.append("\t<server id=\"" + id + "\" name=\"" + name
            + "\" hostname=\"" + host + "\">\n");
        xml.append("\t\t<property name=\"uid\" value=\"" + uid + "\"/>\n");
        xml.append("\t\t<network domain=\"" + domain + "\" port=\""
            + domainPort + "\"/>\n");
        xml
            .append("\t\t<service class=\"org.objectweb.joram.mom.proxies.ConnectionManager\" args=\""
                + username + " " + password + "\"/>\n");
        xml
            .append("\t\t<service class=\"org.objectweb.joram.mom.proxies.tcp.TcpProxyService\" args=\""
                + tcpPort + "\"/>\n");
        // xml
        // .append("\t\t<service
        // class=\"fr.dyade.aaa.jndi2.distributed.DistributedJndiServer\"
        // args=\""
        // + jndiPort + " " + masterId + "\"/>\n");

        xml.append("\t</server>\n");
        return xml.toString();
    }

    /**
     * Build a joram configuration file (a3servers.xml) for a distributed Joram
     * server
     * 
     * @param masterConf
     *            configuration of the others Joram servers
     * @param domain
     *            domain of the server
     * @param id
     *            id of the server
     * @param name
     *            name of the server
     * @param host
     *            host of the server
     * @param username
     *            username to access to the admin module
     * @param password
     *            password to access to the admin module
     * @param domainPort
     *            port of the domain
     * @param tcpPort
     *            port for the tcp connector
     * @param uid
     *            uid of the server
     * @return xml String containing the full configuration
     */
    public static String buildFullConfiguration(String masterConf,
        String domain, String id, String name, String host, String username,
        String password, String domainPort, String tcpPort, String uid) {
        StringBuffer xml = new StringBuffer(masterConf);

        // We add only this configuration if its necessary
        if (xml.indexOf("id=\"" + id) == -1) {
            xml.insert(xml.indexOf("</config>"), buildSlaveServerConf(domain,
                id, name, host, username, password, domainPort, tcpPort, uid));
        }

        return xml.toString();
    }

    /**
     * Return a list of JoramServer whom the information have been extracted
     * from a Joram configuration file
     * 
     * @param configuration
     *            xml Joram configuration file
     * @param thisJoramId
     *            id of the local server
     * @return List of JoramServer
     */
    public static List<JoramServer> retrieveJoramServers(String configuration,
        String thisJoramId) {
        List<JoramServer> servers = new ArrayList<JoramServer>();
        if (configuration != null) {
            Document document = XMLUtil.createDocumentFromString(configuration);
            NodeList servs = document.getDocumentElement()
                .getElementsByTagName("server");
            for (int i = 0; i < servs.getLength(); i++) {
                Element serv = (Element) servs.item(i);
                if (!serv.getAttribute("id").equals(thisJoramId)) {
                    JoramServer joramServer = new JoramServer();
                    joramServer.setHost(serv.getAttribute("hostname"));
                    NodeList services = serv.getElementsByTagName("service");
                    for (int j = 0; j < services.getLength(); j++) {
                        Element service = (Element) services.item(j);
                        if ("org.objectweb.joram.mom.proxies.tcp.TcpProxyService"
                            .equals(service.getAttribute("class"))) {
                            joramServer.setPort(service.getAttribute("args"));
                            servers.add(joramServer);
                        }
                    }
                }
            }
        }
        return servers;
    }
}
