/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: AutoLoaderImplTest.java 3:54:28 PM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.autoload;

import java.io.File;
import java.io.IOException;

import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.ReflectionException;

import junit.framework.TestCase;

import org.apache.commons.io.FileUtils;
import org.easymock.classextension.EasyMock;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.petals.PetalsException;
import org.objectweb.petals.jbi.management.service.DeploymentService;
import org.objectweb.petals.jbi.management.service.InstallationService;
import org.objectweb.petals.jbi.management.service.LifeCycleManagerImpl;
import org.objectweb.petals.jbi.management.service.PackageHandler;
import org.objectweb.petals.jbi.management.systemstate.SystemStateImpl;
import org.objectweb.petals.repository.RepositoryImpl;
import org.objectweb.petals.tools.jbicommon.descriptor.ComponentDescription;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptor;
import org.objectweb.petals.util.InterfaceNamesHelper;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.Logger;

/**
 * Test of the AutoLoaderImpl
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class AutoLoaderImplTest extends TestCase {

    private AutoLoaderImpl autoLoaderImpl;

    private String baseDir;

    public void setUp() {
        baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);
        System.setProperty("petals.home", new File(baseDir + "target")
            .getAbsolutePath());
        new File(baseDir + "target" + File.separator + "lib").mkdir();
        new File(baseDir + "target" + File.separator + "lib" + File.separator
            + "petals-1.0-SNAPSHOT.jar").mkdir();
        autoLoaderImpl = new AutoLoaderImpl();
    }

    public void testBindFc() throws NoSuchInterfaceException,
        IllegalBindingException, IllegalLifeCycleException {
        InstallationService installationService = new InstallationService();
        autoLoaderImpl.bindFc(InterfaceNamesHelper.JBI_INSTALLATION_NAME,
            installationService);
        assertEquals(autoLoaderImpl
            .lookupFc(InterfaceNamesHelper.JBI_INSTALLATION_NAME),
            installationService);

        Logger logger = Monolog.initialize().getLogger("test");
        autoLoaderImpl.bindFc(InterfaceNamesHelper.LOGGER_NAME, logger);
        assertEquals(autoLoaderImpl.lookupFc(InterfaceNamesHelper.LOGGER_NAME),
            logger);

        DeploymentService deploymentService = new DeploymentService();
        autoLoaderImpl.bindFc(InterfaceNamesHelper.JBI_DEPLOYMENT_NAME,
            deploymentService);
        assertEquals(autoLoaderImpl
            .lookupFc(InterfaceNamesHelper.JBI_DEPLOYMENT_NAME),
            deploymentService);

        LifeCycleManagerImpl lifeCycleManagerImpl = new LifeCycleManagerImpl();
        autoLoaderImpl.bindFc(InterfaceNamesHelper.JBI_LIFECYCLEMANAGER_NAME,
            lifeCycleManagerImpl);
        assertEquals(autoLoaderImpl
            .lookupFc(InterfaceNamesHelper.JBI_LIFECYCLEMANAGER_NAME),
            lifeCycleManagerImpl);

        RepositoryImpl repositoryImpl = new RepositoryImpl();
        autoLoaderImpl.bindFc(InterfaceNamesHelper.PLATFORM_REPOSITORY_NAME,
            repositoryImpl);
        assertEquals(autoLoaderImpl
            .lookupFc(InterfaceNamesHelper.PLATFORM_REPOSITORY_NAME),
            repositoryImpl);

        SystemStateImpl systemStateImpl = new SystemStateImpl();
        autoLoaderImpl.bindFc(InterfaceNamesHelper.JBI_SYSTEMSTATE_NAME,
            systemStateImpl);
        assertEquals(autoLoaderImpl
            .lookupFc(InterfaceNamesHelper.JBI_SYSTEMSTATE_NAME),
            systemStateImpl);

        try {
            autoLoaderImpl.bindFc("test", null);
            fail();
        } catch (Exception e) {
            // Do nothing
        }

        try {
            autoLoaderImpl.lookupFc("test");
            fail();
        } catch (Exception e) {
            // Do nothing
        }
    }

    public void testGetFcState() {
        assertEquals(autoLoaderImpl.getFcState(), LifeCycleController.STOPPED);
    }

    public void testInit() throws IOException {
        autoLoaderImpl.logger = Monolog.initialize().getLogger("test");
        autoLoaderImpl.init();
        assertEquals(AutoLoaderImpl.getInstallDirectory(), new File(baseDir
            + "target" + File.separator + "install"));
        assertEquals(AutoLoaderImpl.getInstalledDirectory(), new File(baseDir
            + "target" + File.separator + "installed"));
        assertEquals(AutoLoaderImpl.getWorkDirectory(), new File(baseDir
            + "target" + File.separator + "work"));
        assertEquals(AutoLoaderImpl.getUninstalledDirectory(), new File(baseDir
            + "target" + File.separator + "uninstalled"));
    }

    public void tearDown() {
        System.clearProperty("petals.home");
    }

    public void testStart() throws IOException {
        testInit();
        autoLoaderImpl.log = new LoggingUtil(autoLoaderImpl.logger);
        autoLoaderImpl.start();
        assertNotNull(autoLoaderImpl.installDirectoryScanTimer);
        assertNotNull(autoLoaderImpl.installedDirectoryScanTimer);
    }

    public void testInstallComponent() throws IOException, PetalsException,
        MalformedObjectNameException, NullPointerException,
        InstanceNotFoundException, MBeanException, ReflectionException {
        FileUtils.copyFileToDirectory(new File(baseDir + "src" + File.separator
            + "test-data" + File.separator + "autoload" + File.separator
            + "toInstall.zip"), new File(baseDir + "target"));
        testInit();

        JBIDescriptor descriptor = EasyMock.createMock(JBIDescriptor.class);
        ComponentDescription componentDescription = EasyMock
            .createMock(ComponentDescription.class);
        EasyMock.replay(componentDescription);

        EasyMock.expect(descriptor.getComponent()).andReturn(
            componentDescription).anyTimes();
        EasyMock.replay(descriptor);

        PackageHandler packageHandler = EasyMock
            .createMock(PackageHandler.class);
        EasyMock.expect(
            packageHandler.loadDescriptor(new File(baseDir + "target"
                + File.separator + "work" + File.separator + "toInstall.zip")
                .toURI())).andReturn(descriptor).anyTimes();
        EasyMock.replay(packageHandler);

        LifeCycleManagerImpl lifeCycleManagerImpl = EasyMock
            .createMock(LifeCycleManagerImpl.class);

        MockMbeanServer server = new MockMbeanServer();

        EasyMock.expect(lifeCycleManagerImpl.getMBeanServer())
            .andReturn(server).anyTimes();

        EasyMock.replay(lifeCycleManagerImpl);

        InstallationService installationService = EasyMock
            .createMock(InstallationService.class);
        EasyMock.expect(
            installationService.loadNewInstaller(new File(baseDir + "target"
                + File.separator + "work" + File.separator + "toInstall.zip")
                .toURI().toString())).andReturn(
            new ObjectName("test@test:name=installer")).anyTimes();
        EasyMock.replay(installationService);

        autoLoaderImpl.installationService = installationService;
        autoLoaderImpl.managerService = lifeCycleManagerImpl;
        autoLoaderImpl.packageHandler = packageHandler;
        autoLoaderImpl.log = new LoggingUtil(autoLoaderImpl.logger);
        autoLoaderImpl.install(new File[] {new File(baseDir + "target"
            + File.separator + "toInstall.zip")});

        assertTrue(server.isInvokeStart());
        assertFalse(new File(baseDir + "target" + File.separator + "work"
            + File.separator + "toInstall.zip").exists());
    }
}
