/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: DeploymentServiceTest.java 16:39:45 wjoseph $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.service;

import java.io.File;
import java.io.InputStream;
import java.lang.reflect.Method;
import java.net.URI;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.zip.ZipFile;

import javax.jbi.component.Component;
import javax.jbi.component.ServiceUnitManager;
import javax.jbi.management.DeploymentException;
import javax.management.ObjectName;
import javax.xml.namespace.QName;

import junit.framework.TestCase;

import org.objectweb.petals.PetalsException;
import org.objectweb.petals.jbi.component.mock.MockEndpointService;
import org.objectweb.petals.jbi.component.mock.MockServiceAssemblyLifeCycle;
import org.objectweb.petals.jbi.component.mock.MockServiceAssemblyLifeCycle2;
import org.objectweb.petals.jbi.management.lifecycle.ComponentLifeCycle;
import org.objectweb.petals.jbi.management.lifecycle.LifeCycleAbstract;
import org.objectweb.petals.jbi.management.lifecycle.LifeCycleMBean;
import org.objectweb.petals.jbi.management.lifecycle.ServiceAssemblyLifeCycle;
import org.objectweb.petals.jbi.management.lifecycle.ServiceUnitLifeCycle;
import org.objectweb.petals.jbi.management.lifecycle.mock.ServiceUnitManagerMock;
import org.objectweb.petals.jbi.management.service.util.XMLResult;
import org.objectweb.petals.jbi.management.systemstate.SystemState;
import org.objectweb.petals.jbi.management.systemstate.SystemStateImpl;
import org.objectweb.petals.jbi.mock.MockZipFile;
import org.objectweb.petals.jbi.routing.mock.ComponentMock;
import org.objectweb.petals.repository.RepositoryImpl;
import org.objectweb.petals.repository.RepositoryService;
import org.objectweb.petals.tools.jbicommon.descriptor.Connection;
import org.objectweb.petals.tools.jbicommon.descriptor.Identification;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptor;
import org.objectweb.petals.tools.jbicommon.descriptor.ServiceAssembly;
import org.objectweb.petals.tools.jbicommon.descriptor.ServiceUnit;
import org.objectweb.petals.util.InterfaceNamesHelper;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.wrapper.printwriter.LoggerImpl;

import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.isA;
import static org.easymock.classextension.EasyMock.createMock;
import static org.easymock.classextension.EasyMock.replay;
import static org.easymock.classextension.EasyMock.reset;
import static org.easymock.classextension.EasyMock.verify;

/**
 * Test of the DeploymentService
 * 
 * @author wjoseph - eBMWebsourcing
 */
public class DeploymentServiceTest extends TestCase {

    public void testBindFc() {
        /*
         * Test Case 1 The parameter clientItfName is null, an exception will be
         * raised
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Create Test parameters
         */
        String clientItfName = null;
        Object serverItf = null;
        /*
         * Run Test Case 1
         */
        try {
            deploymentService.bindFc(clientItfName, serverItf);
            fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }

        /*
         * Test Case 2 The parameter clientItfName match the logger interface
         * name The logger attribute of deploymentService will be set to
         * serverItf value
         */
        serverItf = createMock(LoggerImpl.class);
        clientItfName = InterfaceNamesHelper.LOGGER_NAME;
        /*
         * Run Test Case 2
         */
        try {
            deploymentService.bindFc(clientItfName, serverItf);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("logger attribute was not properly set",
                deploymentService.logger, serverItf);

        /*
         * Test Case 3 The parameter clientItfName match the
         * InterfaceNamesHelper.JBI_LIFECYCLEMANAGER_NAME The managerService
         * attribute of deploymentService will be set to serverItf value
         */
        serverItf = createMock(LifeCycleManagerImpl.class);
        clientItfName = InterfaceNamesHelper.JBI_LIFECYCLEMANAGER_NAME;
        /*
         * Run Test Case 3
         */
        try {
            deploymentService.bindFc(clientItfName, serverItf);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("manager attribute was not properly set",
                deploymentService.managerService, serverItf);

        /*
         * Test Case 4 The parameter clientItfName match the
         * InterfaceNamesHelper.PLATFORM_REPOSITORY_NAME The repositorySrv
         * attribute of deploymentService will be set to serverItf value
         */
        serverItf = createMock(RepositoryImpl.class);
        clientItfName = InterfaceNamesHelper.PLATFORM_REPOSITORY_NAME;
        /*
         * Run Test Case 4
         */
        try {
            deploymentService.bindFc(clientItfName, serverItf);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("manager attribute was not properly set",
                deploymentService.repositorySrv, serverItf);

        /*
         * Test Case 5 The parameter clientItfName match the
         * InterfaceNamesHelper.JBI_ADMIN_NAMEThe adminSrv attribute of
         * deploymentService will be set to serverItf value
         */
        serverItf = createMock(AdminService.class);
        clientItfName = InterfaceNamesHelper.JBI_ADMIN_NAME;
        /*
         * Run Test Case 5
         */
        try {
            deploymentService.bindFc(clientItfName, serverItf);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("manager attribute was not properly set",
                deploymentService.adminSrv, serverItf);

        /*
         * Test Case 6 The parameter clientItfName match the
         * InterfaceNamesHelper.JBI_SYSTEMSTATE_NAME The recoverySrv attribute
         * of deploymentService will be set to serverItf value
         */
        serverItf = createMock(SystemStateImpl.class);
        clientItfName = InterfaceNamesHelper.JBI_SYSTEMSTATE_NAME;
        /*
         * Run Test Case 6
         */
        try {
            deploymentService.bindFc(clientItfName, serverItf);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("manager attribute was not properly set",
                deploymentService.recoverySrv, serverItf);

        /*
         * Test Case 7 The parameter clientItfName match the
         * InterfaceNamesHelper.JBI_ENDPOINTSERVICE_NAME The recoverySrv
         * attribute of endpointService will be set to serverItf value
         */
        serverItf = createMock(EndpointServiceImpl.class);
        clientItfName = InterfaceNamesHelper.JBI_ENDPOINTSERVICE_NAME;
        /*
         * Run Test Case 7
         */
        try {
            deploymentService.bindFc(clientItfName, serverItf);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("manager attribute was not properly set",
                deploymentService.endpointService, serverItf);
    }

    public void testCanDeployToComponent() {
        /*
         * Test Case 1 The parameter componentName is null, an exception will be
         * raised
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Create Test parameters
         */
        String componentName = null;
        LoggingUtil log = createMock(LoggingUtil.class);
        deploymentService.log = log;
        /*
         * Run Test Case 1
         */
        try {
            deploymentService.canDeployToComponent(componentName);
            fail("No exception was raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be null"));
        }

        /*
         * Test Case 2 The parameter componentName is empty, an exception will
         * be raised
         */
        /*
         * Create Test parameters
         */
        componentName = "";
        /*
         * Run Test Case 2
         */
        try {
            deploymentService.canDeployToComponent(componentName);
            fail("No exception was raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be empty"));
        }

        /*
         * Test Case 3 One component is found with this name and accept
         * deployment
         */
        /*
         * Create Test parameters
         */
        componentName = "fooCompo";
        LifeCycleManagerService lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        ComponentLifeCycle componentLifeCycle = createMock(ComponentLifeCycle.class);
        Component component = new ComponentMock() {
            public javax.jbi.component.ServiceUnitManager getServiceUnitManager() {
                return new ServiceUnitManager() {

                    public String deploy(String serviceUnitName,
                            String serviceUnitRootPath)
                        throws DeploymentException {
                        return null;
                    }

                    public void init(String serviceUnitName,
                            String serviceUnitRootPath)
                        throws DeploymentException {

                    }

                    public void shutDown(String serviceUnitName)
                        throws DeploymentException {

                    }

                    public void start(String serviceUnitName)
                        throws DeploymentException {

                    }

                    public void stop(String serviceUnitName)
                        throws DeploymentException {

                    }

                    public String undeploy(String serviceUnitName,
                            String serviceUnitRootPath)
                        throws DeploymentException {
                        return null;
                    }

                };
            }
        };

        /*
         * Initialize Mocks
         */
        expect(lifeCycleManager.getComponentByName(componentName)).andReturn(
                componentLifeCycle);
        expect(componentLifeCycle.getCurrentState()).andReturn(
                LifeCycleAbstract.STARTED);
        expect(componentLifeCycle.getComponent()).andReturn(component);
        replay(lifeCycleManager);
        replay(componentLifeCycle);
        deploymentService.managerService = lifeCycleManager;
        boolean result = false;
        /*
         * Run Test Case 3
         */
        try {
            result = deploymentService.canDeployToComponent(componentName);
        } catch (Exception e) {
            fail("Error during invokation");
        }
        assertTrue("Wrong invokation result", result);
    }
    
    public void testCheckServiceAssembly() {
        /*
         * Test Case 1 
         * The JBI descriptor does not provide a Serviceassembly element
         */
        /*
         * Create Mocks
         */
        Method method1 = null;
        Method method2 = null;
        Method method3 = null;
        try {
            method1 = DeploymentService.class.getDeclaredMethod("isLoaded", new Class[]{String.class});
            method2 = DeploymentService.class.getDeclaredMethod("openZipFile", new Class[]{URI.class});
            method3 = DeploymentService.class.getDeclaredMethod("hasEntry", new Class[]{ZipFile.class,String.class});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mocked methods");
        }
        Method[] mockedMethods = new Method[]{method1, method2, method3};
        DeploymentService deploymentService = createMock(DeploymentService.class,mockedMethods);
        /*
         * Create Test parameters
         */
        LoggingUtil log = createMock(LoggingUtil.class);
        deploymentService.log = log;
        JBIDescriptor descriptor = createMock(JBIDescriptor.class);
        XMLResult xmlResult = new XMLResult("deploy");
        URI archiveURI = null;
        String result = null;
        try {
            archiveURI = new URI("file://test/URI");
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not create test parameters");
        }
        /*
         * Run Test Case 1
         */
        try {
            result = deploymentService.checkServiceAssembly(descriptor, archiveURI, xmlResult);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        assertNotNull("The checkServiceAssembly in not ok, the result should not be null",result);
        assertTrue("Wrong invokation result",result.contains("ERROR") && result.contains("FAILED"));
        
        /*
         * Test Case 2 
         * The JBI descriptor provides a ServiceAssembly element
         * but a service assembly with the same name is already loaded on the platform
         */
        reset(descriptor);
        /*
         * Create Test parameters
         */
        xmlResult = new XMLResult("deploy");
        result = null;
        ServiceAssembly sa = createMock(ServiceAssembly.class);
        Identification identification = createMock(Identification.class);
        String saName = "serviceAssemblyName";
        /*
         * Initialize Mocks
         */
        try {
            expect(descriptor.getServiceAssembly()).andReturn(sa);
            expect(sa.getIdentification()).andReturn(identification);
            expect(identification.getName()).andReturn(saName);
            expect(deploymentService.isLoaded(saName)).andReturn(true);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize mocks");
        }
        replay(descriptor);
        replay(sa);
        replay(identification);
        replay(deploymentService);
        /*
         * Run Test Case 2
         */
        try {
            result = deploymentService.checkServiceAssembly(descriptor, archiveURI, xmlResult);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        assertNotNull("The checkServiceAssembly in not ok, the result should not be null",result);
        assertTrue("Wrong invokation result",result.contains("ERROR") && result.contains("FAILED"));
        verify(descriptor);
        verify(sa);
        verify(identification);
        verify(deploymentService);
        
        /*
         * Test Case 3 
         * The JBI descriptor provides a ServiceAssembly element.
         * the service assembly is not yet loaded on the platform
         * the service assembly archive does not provide a zip file matching the service unit targetArtifactsZip
         */
        reset(descriptor);
        reset(sa);
        reset(identification);
        reset(deploymentService);
        /*
         * Create Test parameters
         */
        xmlResult = new XMLResult("deploy");
        result = null;
        List<ServiceUnit> sus = new ArrayList<ServiceUnit>();
        ServiceUnit su = createMock(ServiceUnit.class);
        sus.add(su);
        ZipFile zipFile = createMock(ZipFile.class);
        String artifactZip = "su.zip";
        /*
         * Initialize Mocks
         */
        try {
            expect(descriptor.getServiceAssembly()).andReturn(sa);
            expect(sa.getIdentification()).andReturn(identification);
            expect(sa.getServiceUnits()).andReturn(sus);
            expect(identification.getName()).andReturn(saName);
            expect(deploymentService.isLoaded(saName)).andReturn(false);
            expect(deploymentService.openZipFile(archiveURI)).andReturn(zipFile);
            expect(deploymentService.hasEntry(zipFile, artifactZip)).andReturn(false);
            expect(su.getTargetArtifactsZip()).andReturn(artifactZip);
            
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize mocks");
        }
        replay(descriptor);
        replay(sa);
        replay(identification);
        replay(deploymentService);
        replay(su);
        /*
         * Run Test Case 3
         */
        try {
            deploymentService.checkServiceAssembly(descriptor, archiveURI, xmlResult);
            fail("No exception was raised");
        } catch (Exception e) {
            verify(descriptor);
            verify(sa);
            verify(identification);
            verify(deploymentService);
            verify(su);
        }
        
        /*
         * Test Case 4 
         * The JBI descriptor provides a ServiceAssembly element.
         * the service assembly is not yet loaded on the platform
         * the service assembly archive provides a zip file matching the service unit targetArtifactsZip
         */
        reset(descriptor);
        reset(sa);
        reset(identification);
        reset(deploymentService);
        reset(su);
        /*
         * Create Test parameters
         */
        xmlResult = new XMLResult("deploy");
        result = null;
        zipFile = createMock(MockZipFile.class,new Method[]{});
        /*
         * Initialize Mocks
         */
        try {
            expect(descriptor.getServiceAssembly()).andReturn(sa);
            expect(sa.getIdentification()).andReturn(identification);
            expect(sa.getServiceUnits()).andReturn(sus);
            expect(identification.getName()).andReturn(saName);
            expect(deploymentService.isLoaded(saName)).andReturn(false);
            expect(deploymentService.openZipFile(archiveURI)).andReturn(zipFile);
            expect(deploymentService.hasEntry(zipFile, artifactZip)).andReturn(true);
            expect(su.getTargetArtifactsZip()).andReturn(artifactZip);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize mocks");
        }
        replay(descriptor);
        replay(sa);
        replay(identification);
        replay(deploymentService);
        replay(su);
        /*
         * Run Test Case 4
         */
        try {
            deploymentService.checkServiceAssembly(descriptor, archiveURI, xmlResult);
            fail("No exception was raised");
        } catch (Exception e) {
            verify(descriptor);
            verify(sa);
            verify(identification);
            verify(deploymentService);
            verify(su);
        }
    }

    public void testCreateError() {
        /*
         * Test Case 1
         * addComponentTask is true
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Create Test parameters
         */
        String result = null;
        XMLResult xmlResult = new XMLResult("undeploy");
        StringBuffer componentOutput = new StringBuffer();
        String reason = "error reason";
        String saName = "serviceAssemblyName";
        String suName = "serviceUnitName";
        String[] stringArray = new String[]{};
        /*
         * Run Test Case 1
         */
        result = deploymentService.createErrorMessage(xmlResult, reason, componentOutput, true, saName, suName, stringArray);
        assertTrue("Wrong invokation result",result.contains(reason) && ! result.contains(suName));
        
        /*
         * Test Case 2
         * addComponentTask is false
         */
        /*
         * Create Test parameters
         */
        xmlResult = new XMLResult("undeploy");
        /*
         * Run Test Case 1
         */
        result = deploymentService.createErrorMessage(xmlResult, reason, componentOutput, false, saName, suName, stringArray);
        assertTrue("Wrong invokation result",result.contains(reason) && ! result.contains(suName));
    }
    
    public void testCreateNewServiceAssemblyLifeCycle() {
        /*
         * Test Case 1
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Create Test parameters
         */
        ServiceAssemblyLifeCycle result = null;
        ObjectName objectName = createMock(ObjectName.class);
        ServiceAssembly saDesc = new ServiceAssembly();
        URI installationRoot = null;
        try {
            installationRoot = new URI("file://installroot");
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not create test method");
        }
        LoggingUtil log = createMock(LoggingUtil.class);
        deploymentService.log = log;
        RepositoryService repoSrv = createMock(RepositoryImpl.class);
        deploymentService.repositorySrv = repoSrv;
        SystemState recoverySrv = createMock(SystemStateImpl.class);
        deploymentService.recoverySrv = recoverySrv;
        /*
         * Run Test Case 1
         */
        result = deploymentService.createNewServiceAssemblyLifeCycle(objectName, saDesc, installationRoot);
        assertNotNull("Result of invokation should be a new instance of ServiceAssemblyLifeCycle",result);
    }
    
    public void testCreateNewServiceUnitLifeCycle() {
        /*
         * Test Case 1
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Create Test parameters
         */
        ServiceUnitLifeCycle result = null;
        ServiceUnit suDesc = createMock(ServiceUnit.class);
        String rootPath = "rootPath";
        LoggingUtil log = createMock(LoggingUtil.class);
        deploymentService.log = log;
        LifeCycleManagerService serviceManager = createMock(LifeCycleManagerImpl.class);
        deploymentService.managerService = serviceManager;
        Identification id = createMock(Identification.class);
        String suName = "ServiceUnitName";
        /*
         * Initiliaze Mocks
         */
        expect(id.getName()).andReturn(suName);
        expect(suDesc.getIdentification()).andReturn(id);
        replay(id);
        replay(suDesc);
        /*
         * Run Test Case 1
         */
        result = deploymentService.createNewServiceUnitLifeCycle(suDesc, rootPath);
        assertNotNull("Result of invokation should be a new instance of ServiceUnitLifeCycle",result);
        verify(id);
        verify(suDesc);
    }
    
    public void testCreateWarningMessage() {
        /*
         * Test Case 1
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Create Test parameters
         */
        String result = null;
        XMLResult xmlResult = new XMLResult("undeploy");
        String exceptionMessage = "exception message";
        StringBuffer componentOutput = new StringBuffer();
        List<String> exceptionParams = new ArrayList<String>();
        /*
         * Run Test Case 1
         */
        result = deploymentService.createWarningMessage(xmlResult, exceptionMessage, exceptionParams, componentOutput);
        assertTrue("Wrong invokation result",result.contains("WARNING") && result.contains(exceptionMessage));
    }
    
    public void testExpandSUIntoSA() {
        /*
         * Test Case 1
         * the explodeSUIntoSAInstallFolder mthod throw an exception
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Create Test parameters
         */
        URI result = null;
        String suId = "serviceUnitId";
        String saId = "serviceAssemblyId";
        URI suZipLocation = null;
        try {
            suZipLocation = new URI("file://zipLocation");
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not create test parameters");
        }
        LoggingUtil log = createMock(LoggingUtil.class);
        deploymentService.log = log;
        RepositoryService repoSrv = createMock(RepositoryImpl.class);
        /*
         * Initialize Mocks
         */
        try{
        expect(repoSrv.explodeSUIntoSAInstallFolder(suId, suZipLocation, saId)).andThrow(new PetalsException("explodeSUIntoSA"));
        } catch (Exception e) {
            e.printStackTrace();
            fail ("Could not initialize Mocks");
        }
        deploymentService.repositorySrv = repoSrv;
        /*
         * Run Test Case 1
         */
        result = deploymentService.expandSUIntoSA(suId, suZipLocation, saId);
        assertNull("Wrong invokation result",result);
    }

    public void testGetServiceAssemblyName() {
        /*
         * Test Case 1
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Create Test parameters
         */
        String saName = "serviceAssemblyName";
        JBIDescriptor descriptor = createMock(JBIDescriptor.class);
        ServiceAssembly sa = createMock(ServiceAssembly.class);
        Identification id = createMock(Identification.class);
        /*
         * Initialize Mocks
         */
        expect(descriptor.getServiceAssembly()).andReturn(sa);
        expect(sa.getIdentification()).andReturn(id);
        expect(id.getName()).andReturn(saName);
        replay(descriptor);
        replay(sa);
        replay(id);
        /*
         * Run Test Case 1
         */
        assertEquals("Wrong invokation result",saName,deploymentService.getServiceAssemblyName(descriptor));
        verify(descriptor);
        verify(sa);
        verify(id);
    }
    
    public void testDeploy() {
        /*
         * Test Case 1 The parameter serviceAssemblyZipUrl is null, an exception
         * will be raised
         */
        /*
         * Create Mocks
         */
        Method method1 = null;
        Method method2 = null;
        Method method3 = null;
        Method method4 = null;
        Method method5 = null;
        Method method6 = null;
        Method method7 = null;
        try {
            method1 = DeploymentService.class.getDeclaredMethod(
                    "checkServiceAssembly", new Class[] {JBIDescriptor.class,
                        URI.class,XMLResult.class});
            method2 = DeploymentService.class
                    .getDeclaredMethod("getServiceAssemblyName",
                            new Class[] {JBIDescriptor.class});
            method3 = DeploymentService.class.getDeclaredMethod(
                    "expandSUIntoSA", new Class[] {String.class, URI.class,
                        String.class});
            method4 = DeploymentService.class.getDeclaredMethod("deploy",
                    new Class[] {URI.class, JBIDescriptor.class});
            method5 = DeploymentService.class.getDeclaredMethod(
                    "getArchiveFile", new Class[] {URI.class});
            method6 = DeploymentService.class.getDeclaredMethod(
                    "getInstalledArchive", new Class[] {File.class});
            method7 = DeploymentService.class.getDeclaredMethod("copyFile",
                    new Class[] {File.class, File.class});

        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mocked methods");
        }
        Method[] mockedMethods = new Method[] {method1, method2, method3,
            method4, method5, method6, method7};
        DeploymentService deploymentService = createMock(
                DeploymentService.class, mockedMethods);
        LoggingUtil log = createMock(LoggingUtil.class);
        deploymentService.log = log;
        /*
         * Create Test parameters
         */
        String serviceAssemblyZipUrl = null;
        /*
         * Run Test Case 1
         */
        try {
            deploymentService.deploy(serviceAssemblyZipUrl);
            fail("No exception was raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be null"));
        }

        /*
         * Test Case 2 The parameter serviceAssemblyZipUrl is empty, an
         * exception will be raised
         */
        /*
         * Create Test parameters
         */
        serviceAssemblyZipUrl = "";
        /*
         * Run Test Case 2
         */
        try {
            deploymentService.deploy(serviceAssemblyZipUrl);
            fail("No exception was raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be empty"));
        }

        /*
         * Test Case 3 The parameter serviceAssemblyZipUrl is not a valid URL,
         * an exception will be raised
         */
        /*
         * Create Test parameters
         */
        serviceAssemblyZipUrl = "someURL";
        /*
         * Run Test Case 3
         */
        try {
            deploymentService.deploy(serviceAssemblyZipUrl);
            fail("No exception was raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().equals(
                    "serviceAssemblyZipUrl is not correctly formed"));
        }

        /*
         * Test Case 4 Everything goes well
         */
        /*
         * Create Test parameters
         */
        serviceAssemblyZipUrl = "file://someURL";
        PackageHandler packageHandler = createMock(PackageHandler.class);
        deploymentService.packageHandler = packageHandler;
        URI archiveURI = null;
        URI installationURI = null;
        URI suZipLocation = null;
        try {
            archiveURI = new URI("file://archiveURI");// createMock(URI.class);
            installationURI = new URI("file://installationURI");// createMock(URI.class);
            suZipLocation = new URI("file://zipLocaion");
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not create URIs");
        }
        JBIDescriptor descriptor = createMock(JBIDescriptor.class);
        String saName = "serviceAssemblyName";
        ServiceAssembly saDesc = createMock(ServiceAssembly.class);
        List<ServiceUnit> suList = new ArrayList<ServiceUnit>();
        ServiceUnit serviceUnit = createMock(ServiceUnit.class);
        suList.add(serviceUnit);
        Identification identification = createMock(Identification.class);
        String suName = "serviceUnitName";
        String suArtifactsZip = "foo.zip";
        String deployResult = "result";
        SystemState systemState = createMock(SystemStateImpl.class);
        String result = null;
        File installedArchiveFile = new File(".");
        /*
         * Initialize Mocks
         */
        try {
            expect(
                    packageHandler.processAndGetPackageURI(
                            serviceAssemblyZipUrl, true)).andReturn(archiveURI);
            expect(packageHandler.loadDescriptor(archiveURI)).andReturn(
                    descriptor);
            expect(
                    packageHandler.expandPackageIntoRepository(saName,
                            archiveURI)).andReturn(installationURI);
            expect(
                    packageHandler.processAndGetPackageURI("file://"
                            + installationURI.getPath() + suArtifactsZip, true))
                    .andReturn(suZipLocation);
            expect(
                    deploymentService.checkServiceAssembly(isA(JBIDescriptor.class),
                            isA(URI.class),isA(XMLResult.class))).andReturn(null);
            expect(deploymentService.getServiceAssemblyName(descriptor))
                    .andReturn(saName);
            expect(
                    deploymentService.expandSUIntoSA(suName, suZipLocation,
                            saName)).andReturn(null);
            expect(deploymentService.deploy(installationURI, descriptor))
                    .andReturn(deployResult);
            expect(deploymentService.getArchiveFile(archiveURI))
                    .andReturn(null);
            expect(deploymentService.getInstalledArchive(null)).andReturn(
                    installedArchiveFile);
            deploymentService.copyFile(null, installedArchiveFile);
            expect(descriptor.getServiceAssembly()).andReturn(saDesc);
            expect(saDesc.getServiceUnits()).andReturn(suList);
            expect(serviceUnit.getIdentification()).andReturn(identification);
            expect(serviceUnit.getTargetArtifactsZip()).andReturn(
                    suArtifactsZip);
            expect(identification.getName()).andReturn(suName);
            // expect(installationURI.getPath()).andReturn(installationPath);
            expect(
                    systemState.createServiceAssemblyStateHolder(saName,
                            installationURI.toURL().toString(),
                            installedArchiveFile.toURL().toString(),
                            LifeCycleMBean.SHUTDOWN)).andReturn(null);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(packageHandler);
        replay(deploymentService);
        replay(descriptor);
        replay(saDesc);
        replay(serviceUnit);
        replay(identification);
        replay(systemState);
        deploymentService.recoverySrv = systemState;
        /*
         * 
         * Run Test Case 4
         */
        try {
            result = deploymentService.deploy(serviceAssemblyZipUrl);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        assertNotNull("deploy result must be non null", result);
        assertFalse("deploy result must be non empty", result.length() == 0);
        assertEquals("Wrong invokation result", result, deployResult);
        verify(packageHandler);
        verify(deploymentService);
        verify(descriptor);
        verify(saDesc);
        verify(serviceUnit);
        verify(identification);
        verify(systemState);

        /*
         * Test Case 5 Everything goes well
         */
        reset(packageHandler);
        reset(deploymentService);
        /*
         * Create Test parameters
         */
        serviceAssemblyZipUrl = "file://someURL";
        /*
         * Initialize Mocks
         */
        try {
            expect(
                    packageHandler.processAndGetPackageURI(
                            serviceAssemblyZipUrl, true)).andReturn(archiveURI);
            expect(packageHandler.loadDescriptor(archiveURI)).andReturn(
                    descriptor);
            expect(
                    deploymentService.checkServiceAssembly(isA(JBIDescriptor.class),
                            isA(URI.class),isA(XMLResult.class))).andReturn("");
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(packageHandler);
        replay(deploymentService);
        /*
         * 
         * Run Test Case 5
         */
        try {
            deploymentService.deploy(serviceAssemblyZipUrl);
            fail("No exception was raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().equals(
                    "Service assembly package isn't validated"));
            verify(packageHandler);
            verify(deploymentService);
        }
    }

    public void testDeploy2() {
        /*
         * Test Case 1 The parameter installationRoot is null, an exception will
         * be raised
         */
        /*
         * Create Mocks
         */
        Method method1 = null;
        Method method2 = null;
        Method method3 = null;
        Method method4 = null;
        try {
            method1 = DeploymentService.class.getDeclaredMethod(
                    "isValidComponentTask", new Class[] {String.class,
                        InputStream.class});
            method2 = DeploymentService.class.getDeclaredMethod(
                    "createNewServiceUnitLifeCycle", new Class[] {
                        ServiceUnit.class, String.class});
            method3 = DeploymentService.class.getDeclaredMethod(
                    "createNewServiceAssemblyLifeCycle", new Class[] {
                        ObjectName.class, ServiceAssembly.class, URI.class});
            method4 = DeploymentService.class.getDeclaredMethod(
                    "registerConnection", new Class[] {Connection.class,
                        XMLResult.class, StringBuffer.class, String.class});

        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mocked methods");
        }
        Method[] mockedMethods = new Method[] {method1, method2, method3,
            method4};
        DeploymentService deploymentService = createMock(
                DeploymentService.class, mockedMethods);
        /*
         * Create Test parameters
         */
        URI installationRoot = null;
        JBIDescriptor descriptor = new JBIDescriptor();
        LoggingUtil log = createMock(LoggingUtil.class);
        deploymentService.log = log;
        /*
         * Run Test Case 1
         */
        try {
            deploymentService.deploy(installationRoot, descriptor);
            fail("No exception was raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be null"));
        }

        /*
         * Test Case 2 The parameter descriptor is null, an exception will be
         * raised
         */
        /*
         * Create Test parameters
         */
        try {
            installationRoot = new URI("file://foo");
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not create test parameters");
        }
        descriptor = null;
        /*
         * Run Test Case 2
         */
        try {
            deploymentService.deploy(installationRoot, descriptor);
            fail("No exception was raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be null"));
        }

        /*
         * Test Case 3 The serviceManager can not retrieve the
         * DeploymentServiceMBean
         */
        /*
         * Create Test parameters
         */
        descriptor = createMock(JBIDescriptor.class);
        ServiceAssembly sa = createMock(ServiceAssembly.class);
        LifeCycleManagerService lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        /*
         * Initialise Mocks
         */
        expect(descriptor.getServiceAssembly()).andReturn(sa);
        try {
            expect(lifeCycleManager.getDeploymentServiceMBeanName()).andThrow(
                    new ManagementException(
                            "getDeploymentServiceMBeanName testing exception"));
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(descriptor);
        replay(lifeCycleManager);
        deploymentService.managerService = lifeCycleManager;
        String result = null;
        /*
         * Run Test Case 3
         */
        try {
            result = deploymentService.deploy(installationRoot, descriptor);
        } catch (Exception e) {
            fail("Error during invokation");
        }
        verify(descriptor);
        verify(lifeCycleManager);
        assertNotNull("Result of deploy must be non null", result);
        assertFalse("Result of deploy must be non empty", result.length() == 0);

        /*
         * Test Case 4 The serviceManager can not retrieve the
         * ComponentLifeCycle, null is returned
         * 
         */
        reset(descriptor);
        reset(lifeCycleManager);
        /*
         * Create Test parameters
         */
        ObjectName objectName = createMock(ObjectName.class);
        ServiceAssemblyLifeCycle saLifeCycle = createMock(
                MockServiceAssemblyLifeCycle.class, new Method[] {});
        List<ServiceUnit> sus = new ArrayList<ServiceUnit>();
        ServiceUnit su = createMock(ServiceUnit.class);
        sus.add(su);
        Identification identification = createMock(Identification.class);
        String saName = "service assembly name";
        String suName = "service unit name";
        RepositoryService repoSrv = createMock(RepositoryImpl.class);
        String componentName = "component name";
        File saInstallRootFile = new File(".");
        ComponentLifeCycle clc = null;
        /*
         * Initialise Mocks
         */
        try {
            expect(descriptor.getServiceAssembly()).andReturn(sa);
            expect(lifeCycleManager.getDeploymentServiceMBeanName()).andReturn(
                    objectName);
            expect(lifeCycleManager.getComponentByName(componentName))
                    .andReturn(clc);
            expect(
                    deploymentService.createNewServiceAssemblyLifeCycle(
                            objectName, sa, installationRoot)).andReturn(
                    saLifeCycle);
            expect(sa.getServiceUnits()).andReturn(sus).anyTimes();
            expect(sa.getIdentification()).andReturn(identification);
            expect(identification.getName()).andReturn(saName);
            expect(identification.getName()).andReturn(suName);
            expect(repoSrv.getComponentInstallRoot(saName)).andReturn(
                    saInstallRootFile);
            expect(su.getIdentification()).andReturn(identification);
            expect(su.getTargetComponentName()).andReturn(componentName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(descriptor);
        replay(lifeCycleManager);
        replay(deploymentService);
        replay(sa);
        replay(identification);
        replay(su);
        replay(repoSrv);
        deploymentService.managerService = lifeCycleManager;
        deploymentService.repositorySrv = repoSrv;
        result = null;
        /*
         * Run Test Case 4
         */
        try {
            result = deploymentService.deploy(installationRoot, descriptor);
        } catch (Exception e) {
            fail("Error during invokation");
        }
        verify(descriptor);
        verify(lifeCycleManager);
        verify(deploymentService);
        verify(sa);
        verify(identification);
        verify(su);
        verify(repoSrv);
        assertNotNull("Result of deploy must be non null", result);
        assertFalse("Result of deploy must be non empty", result.length() == 0);
        assertTrue("Wrong invokation result", result.contains("WARNING")
                && result.contains("FAILED"));

        /*
         * Test Case 5 The serviceManager retrieve a ComponentLifeCycle The
         * component retrieved from componentLifeCycle return a null
         * serviceUnitManager
         * 
         */
        reset(descriptor);
        reset(lifeCycleManager);
        reset(deploymentService);
        reset(sa);
        reset(identification);
        reset(su);
        reset(repoSrv);
        /*
         * Create Test parameters
         */
        clc = createMock(ComponentLifeCycle.class);
        Component component = createMock(Component.class);
        /*
         * Initialise Mocks
         */
        try {
            expect(descriptor.getServiceAssembly()).andReturn(sa);
            expect(lifeCycleManager.getDeploymentServiceMBeanName()).andReturn(
                    objectName);
            expect(lifeCycleManager.getComponentByName(componentName))
                    .andReturn(clc);
            expect(
                    deploymentService.createNewServiceAssemblyLifeCycle(
                            objectName, sa, installationRoot)).andReturn(
                    saLifeCycle);
            expect(sa.getServiceUnits()).andReturn(sus).anyTimes();
            expect(sa.getIdentification()).andReturn(identification);
            expect(identification.getName()).andReturn(saName);
            expect(identification.getName()).andReturn(suName);
            expect(repoSrv.getComponentInstallRoot(saName)).andReturn(
                    saInstallRootFile);
            expect(su.getIdentification()).andReturn(identification);
            expect(su.getTargetComponentName()).andReturn(componentName);
            expect(clc.getComponent()).andReturn(component);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(descriptor);
        replay(lifeCycleManager);
        replay(deploymentService);
        replay(sa);
        replay(identification);
        replay(su);
        replay(repoSrv);
        replay(clc);
        deploymentService.managerService = lifeCycleManager;
        deploymentService.repositorySrv = repoSrv;
        result = null;
        /*
         * Run Test Case 5
         */
        try {
            result = deploymentService.deploy(installationRoot, descriptor);
        } catch (Exception e) {
            fail("Error during invokation");
        }
        verify(descriptor);
        verify(lifeCycleManager);
        verify(deploymentService);
        verify(sa);
        verify(identification);
        verify(su);
        verify(repoSrv);
        verify(clc);
        assertNotNull("Result of deploy must be non null", result);
        assertFalse("Result of deploy must be non empty", result.length() == 0);
        assertTrue("Wrong invokation result", result.contains("WARNING")
                && result.contains("FAILED"));

        /*
         * Test Case 6 The serviceManager retrieve a ComponentLifeCycle The
         * component retrieved from componentLifeCycle returns a
         * serviceUnitManager the deploy invokation on the serviceUnitManager
         * throw an exception
         * 
         */
        reset(descriptor);
        reset(lifeCycleManager);
        reset(deploymentService);
        reset(sa);
        reset(identification);
        reset(su);
        reset(repoSrv);
        reset(clc);
        reset(component);
        /*
         * Create Test parameters
         */
        ServiceUnitManager sum = createMock(ServiceUnitManagerMock.class);
        String xmlResult = "xmlResult";
        /*
         * Initialise Mocks
         */
        try {
            expect(descriptor.getServiceAssembly()).andReturn(sa);
            expect(lifeCycleManager.getDeploymentServiceMBeanName()).andReturn(
                    objectName);
            expect(lifeCycleManager.getComponentByName(componentName))
                    .andReturn(clc);
            expect(
                    deploymentService.createNewServiceAssemblyLifeCycle(
                            objectName, sa, installationRoot)).andReturn(
                    saLifeCycle);
            expect(sa.getServiceUnits()).andReturn(sus).anyTimes();
            expect(sa.getIdentification()).andReturn(identification);
            expect(identification.getName()).andReturn(saName);
            expect(identification.getName()).andReturn(suName);
            expect(repoSrv.getComponentInstallRoot(saName)).andReturn(
                    saInstallRootFile);
            expect(su.getIdentification()).andReturn(identification);
            expect(su.getTargetComponentName()).andReturn(componentName);
            expect(clc.getComponent()).andReturn(component);
            expect(component.getServiceUnitManager()).andReturn(sum);
            expect(sum.deploy(isA(String.class), isA(String.class))).andThrow(
                    new DeploymentException("deploy testing exception"));
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(descriptor);
        replay(lifeCycleManager);
        replay(deploymentService);
        replay(sa);
        replay(identification);
        replay(su);
        replay(repoSrv);
        replay(clc);
        replay(component);
        replay(sum);
        deploymentService.managerService = lifeCycleManager;
        deploymentService.repositorySrv = repoSrv;
        result = null;
        /*
         * Run Test Case 6
         */
        try {
            result = deploymentService.deploy(installationRoot, descriptor);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        verify(descriptor);
        verify(lifeCycleManager);
        verify(deploymentService);
        verify(sa);
        verify(identification);
        verify(su);
        verify(repoSrv);
        verify(clc);
        verify(sum);
        verify(component);
        assertNotNull("Result of deploy must be non null", result);
        assertFalse("Result of deploy must be non empty", result.length() == 0);
        assertTrue("Wrong invokation result", result.contains("ERROR")
                && result.contains("FAILED"));

        /*
         * Test Case 7 
         * The serviceManager retrieve a ComponentLifeCycle The
         * component retrieved from componentLifeCycle returns a
         * serviceUnitManager The deploy invokation on the serviceUnitManager
         * return a valid result but this result is FAILED
         *  registerSUIntoSA throw an exception
         * 
         */
        reset(descriptor);
        reset(lifeCycleManager);
        reset(deploymentService);
        reset(sa);
        reset(identification);
        reset(su);
        reset(repoSrv);
        reset(clc);
        reset(component);
        reset(sum);
        /*
         * Create Test parameters
         */
        ServiceUnitLifeCycle sulf = createMock(ServiceUnitLifeCycle.class);
        /*
         * Initialise Mocks
         */
        try {
            expect(descriptor.getServiceAssembly()).andReturn(sa);
            expect(lifeCycleManager.getDeploymentServiceMBeanName()).andReturn(
                    objectName);
            expect(lifeCycleManager.getComponentByName(componentName))
                    .andReturn(clc);
            expect(
                    deploymentService.isValidComponentTask(isA(String.class),
                            isA(InputStream.class))).andReturn(true);
            expect(
                    deploymentService.createNewServiceAssemblyLifeCycle(
                            objectName, sa, installationRoot)).andReturn(
                    saLifeCycle);
            expect(sa.getServiceUnits()).andReturn(sus).anyTimes();
            expect(sa.getIdentification()).andReturn(identification);
            expect(identification.getName()).andReturn(saName);
            expect(identification.getName()).andReturn(suName);
            expect(repoSrv.getComponentInstallRoot(saName)).andReturn(
                    saInstallRootFile);
            expect(su.getIdentification()).andReturn(identification);
            expect(su.getTargetComponentName()).andReturn(componentName)
                    .anyTimes();
            expect(clc.getComponent()).andReturn(component);
            expect(component.getServiceUnitManager()).andReturn(sum);
            expect(sum.deploy(isA(String.class), isA(String.class))).andReturn(
                    xmlResult);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(descriptor);
        replay(lifeCycleManager);
        replay(deploymentService);
        replay(sa);
        replay(identification);
        replay(su);
        replay(repoSrv);
        replay(clc);
        replay(component);
        replay(sum);
        deploymentService.managerService = lifeCycleManager;
        deploymentService.repositorySrv = repoSrv;
        result = null;
        /*
         * Run Test Case 7
         */
        try {
            result = deploymentService.deploy(installationRoot, descriptor);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        verify(descriptor);
        verify(lifeCycleManager);
        verify(deploymentService);
        verify(sa);
        verify(identification);
        verify(su);
        verify(repoSrv);
        verify(clc);
        verify(sum);
        verify(component);
        assertNotNull("Result of deploy must be non null", result);
        assertFalse("Result of deploy must be non empty", result.length() == 0);
        assertTrue("Wrong invokation result", result.contains("WARNING")
                && result.contains("FAILED"));
        
        /*
         * Test Case 8 The serviceManager retrieve a ComponentLifeCycle The
         * component retrieved from componentLifeCycle returns a
         * serviceUnitManager The deploy invokation on the serviceUnitManager
         * return an invalid result registerSUIntoSA throw an exception
         * 
         */
        reset(descriptor);
        reset(lifeCycleManager);
        reset(deploymentService);
        reset(sa);
        reset(identification);
        reset(su);
        reset(repoSrv);
        reset(clc);
        reset(component);
        reset(sum);
        reset(sulf);
        /*
         * Create Test parameters
         */
        /*
         * Initialise Mocks
         */
        try {
            expect(descriptor.getServiceAssembly()).andReturn(sa);
            expect(lifeCycleManager.getDeploymentServiceMBeanName()).andReturn(
                    objectName);
            expect(lifeCycleManager.getComponentByName(componentName))
                    .andReturn(clc);
            expect(
                    deploymentService.isValidComponentTask(isA(String.class),
                            isA(InputStream.class))).andReturn(false);
            expect(
                    deploymentService.createNewServiceAssemblyLifeCycle(
                            objectName, sa, installationRoot)).andReturn(
                    saLifeCycle);
            expect(
                    deploymentService.createNewServiceUnitLifeCycle(
                            isA(ServiceUnit.class), isA(String.class)))
                    .andReturn(sulf);
            expect(sa.getServiceUnits()).andReturn(sus).anyTimes();
            expect(sa.getIdentification()).andReturn(identification);
            expect(identification.getName()).andReturn(saName);
            expect(identification.getName()).andReturn(suName);
            expect(repoSrv.getComponentInstallRoot(saName)).andReturn(
                    saInstallRootFile);
            expect(su.getIdentification()).andReturn(identification);
            expect(su.getTargetComponentName()).andReturn(componentName)
                    .anyTimes();
            expect(clc.getComponent()).andReturn(component);
            expect(component.getServiceUnitManager()).andReturn(sum);
            expect(sum.deploy(isA(String.class), isA(String.class))).andReturn(
                    xmlResult);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(descriptor);
        replay(lifeCycleManager);
        replay(deploymentService);
        replay(sa);
        replay(identification);
        replay(su);
        replay(repoSrv);
        replay(clc);
        replay(component);
        replay(sum);
        deploymentService.managerService = lifeCycleManager;
        deploymentService.repositorySrv = repoSrv;
        result = null;
        /*
         * Run Test Case 8
         */
        try {
            result = deploymentService.deploy(installationRoot, descriptor);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        verify(descriptor);
        verify(lifeCycleManager);
        verify(deploymentService);
        verify(sa);
        verify(identification);
        verify(su);
        verify(repoSrv);
        verify(clc);
        verify(sum);
        verify(component);
        assertNotNull("Result of deploy must be non null", result);
        assertFalse("Result of deploy must be non empty", result.length() == 0);
        assertTrue("Wrong invokation result", result.contains("ERROR")
                && result.contains("FAILED"));

        /*
         * Test Case 9 The serviceManager retrieve a ComponentLifeCycle The
         * component retrieved from componentLifeCycle returns a
         * serviceUnitManager The deploy invokation on the serviceUnitManager
         * return an invalid result registerSUIntoSA does not throw an exception
         * registerConnection fails
         * 
         */
        reset(descriptor);
        reset(lifeCycleManager);
        reset(deploymentService);
        reset(sa);
        reset(identification);
        reset(su);
        reset(repoSrv);
        reset(clc);
        reset(component);
        reset(sum);
        saLifeCycle = createMock(ServiceAssemblyLifeCycle.class);
        /*
         * Create Test parameters
         */
        Connection connection = createMock(Connection.class);
        List<Connection> connections = new ArrayList<Connection>();
        connections.add(connection);
        String registerConnectionResult = "result";
        /*
         * Initialise Mocks
         */
        try {
            expect(descriptor.getServiceAssembly()).andReturn(sa);
            expect(lifeCycleManager.getDeploymentServiceMBeanName()).andReturn(
                    objectName);
            expect(lifeCycleManager.getComponentByName(componentName))
                    .andReturn(clc);
            expect(
                    deploymentService.isValidComponentTask(isA(String.class),
                            isA(InputStream.class))).andReturn(false);
            expect(
                    deploymentService.createNewServiceAssemblyLifeCycle(
                            objectName, sa, installationRoot)).andReturn(
                    saLifeCycle);
            expect(
                    deploymentService.createNewServiceUnitLifeCycle(
                            isA(ServiceUnit.class), isA(String.class)))
                    .andReturn(sulf);
            expect(
                    deploymentService.registerConnection(isA(Connection.class),
                            isA(XMLResult.class), isA(StringBuffer.class),
                            isA(String.class))).andReturn(
                    registerConnectionResult);
            expect(sa.getServiceUnits()).andReturn(sus).anyTimes();
            expect(sa.getIdentification()).andReturn(identification);
            expect(sa.getConnections()).andReturn(connections);
            expect(identification.getName()).andReturn(saName);
            expect(identification.getName()).andReturn(suName);
            expect(repoSrv.getComponentInstallRoot(saName)).andReturn(
                    saInstallRootFile);
            expect(su.getIdentification()).andReturn(identification);
            expect(su.getTargetComponentName()).andReturn(componentName)
                    .anyTimes();
            expect(clc.getComponent()).andReturn(component);
            expect(component.getServiceUnitManager()).andReturn(sum);
            expect(sum.deploy(isA(String.class), isA(String.class))).andReturn(
                    xmlResult);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(descriptor);
        replay(lifeCycleManager);
        replay(deploymentService);
        replay(sa);
        replay(identification);
        replay(su);
        replay(repoSrv);
        replay(clc);
        replay(component);
        replay(sum);
        deploymentService.managerService = lifeCycleManager;
        deploymentService.repositorySrv = repoSrv;
        result = null;
        /*
         * Run Test Case 9
         */
        try {
            result = deploymentService.deploy(installationRoot, descriptor);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        verify(descriptor);
        verify(lifeCycleManager);
        verify(deploymentService);
        verify(sa);
        verify(identification);
        verify(su);
        verify(repoSrv);
        verify(clc);
        verify(sum);
        verify(component);
        assertNotNull("Result of deploy must be non null", result);
        assertFalse("Result of deploy must be non empty", result.length() == 0);
        assertEquals("Wrong invokation result", result,
                registerConnectionResult);

        /*
         * Test Case 8 The serviceManager retrieve a ComponentLifeCycle The
         * component retrieved from componentLifeCycle returns a
         * serviceUnitManager The deploy invokation on the serviceUnitManager
         * return an invalid result registerSUIntoSA does not throw an exception
         * registerConnection works fine
         * 
         */
        reset(descriptor);
        reset(lifeCycleManager);
        reset(deploymentService);
        reset(sa);
        reset(identification);
        reset(su);
        reset(repoSrv);
        reset(clc);
        reset(component);
        reset(sum);
        saLifeCycle = createMock(ServiceAssemblyLifeCycle.class);
        /*
         * Create Test parameters
         */
        registerConnectionResult = null;
        List<String> loadedPackages = new ArrayList<String>();
        /*
         * Initialise Mocks
         */
        try {
            expect(descriptor.getServiceAssembly()).andReturn(sa);
            expect(lifeCycleManager.getDeploymentServiceMBeanName()).andReturn(
                    objectName);
            expect(lifeCycleManager.getComponentByName(componentName))
                    .andReturn(clc);
            lifeCycleManager.registerServiceAssembly(saName, saLifeCycle);
            expect(
                    deploymentService.isValidComponentTask(isA(String.class),
                            isA(InputStream.class))).andReturn(false);
            expect(
                    deploymentService.createNewServiceAssemblyLifeCycle(
                            objectName, sa, installationRoot)).andReturn(
                    saLifeCycle);
            expect(
                    deploymentService.createNewServiceUnitLifeCycle(
                            isA(ServiceUnit.class), isA(String.class)))
                    .andReturn(sulf);
            expect(
                    deploymentService.registerConnection(isA(Connection.class),
                            isA(XMLResult.class), isA(StringBuffer.class),
                            isA(String.class))).andReturn(
                    registerConnectionResult);
            expect(sa.getServiceUnits()).andReturn(sus).anyTimes();
            expect(sa.getIdentification()).andReturn(identification);
            expect(sa.getConnections()).andReturn(connections);
            expect(identification.getName()).andReturn(saName);
            expect(identification.getName()).andReturn(suName);
            expect(repoSrv.getComponentInstallRoot(saName)).andReturn(
                    saInstallRootFile);
            expect(su.getIdentification()).andReturn(identification);
            expect(su.getTargetComponentName()).andReturn(componentName)
                    .anyTimes();
            expect(clc.getComponent()).andReturn(component);
            expect(component.getServiceUnitManager()).andReturn(sum);
            expect(sum.deploy(isA(String.class), isA(String.class))).andReturn(
                    xmlResult);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(descriptor);
        replay(lifeCycleManager);
        replay(deploymentService);
        replay(sa);
        replay(identification);
        replay(su);
        replay(repoSrv);
        replay(clc);
        replay(component);
        replay(sum);
        deploymentService.managerService = lifeCycleManager;
        deploymentService.repositorySrv = repoSrv;
        deploymentService.packagesLoadedCache = loadedPackages;
        result = null;
        /*
         * Run Test Case 8
         */
        try {
            result = deploymentService.deploy(installationRoot, descriptor);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        verify(descriptor);
        verify(lifeCycleManager);
        verify(deploymentService);
        verify(sa);
        verify(identification);
        verify(su);
        verify(repoSrv);
        verify(clc);
        verify(sum);
        verify(component);
        assertNotNull("Result of deploy must be non null", result);
        assertFalse("Result of deploy must be non empty", result.length() == 0);
        assertTrue("Wrong invokation result", result.contains("SUCCESS"));
    }

    public void testGetComponentsForDeployedServiceAssembly() {
        /*
         * Test Case 1 The parameter serviceAssemblyName is null, an exception
         * will be raised
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Create Test parameters
         */
        String serviceAssemblyName = null;
        LoggingUtil log = createMock(LoggingUtil.class);
        deploymentService.log = log;
        /*
         * Run Test Case 1
         */
        try {
            deploymentService
                    .getComponentsForDeployedServiceAssembly(serviceAssemblyName);
            fail("No exception was raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be null"));
        }

        /*
         * Test Case 2 The parameter serviceAssemblyName is empty, an exception
         * will be raised
         */
        /*
         * Create Test parameters
         */
        serviceAssemblyName = "";
        /*
         * Run Test Case 2
         */
        try {
            deploymentService
                    .getComponentsForDeployedServiceAssembly(serviceAssemblyName);
            fail("No exception was raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be empty"));
        }
        /*
         * Test Case 3 No service assembly for the given name is found, an
         * exception is raised
         */
        /*
         * Create Test parameters
         */
        LifeCycleManagerService lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        deploymentService.managerService = lifeCycleManager;
        serviceAssemblyName = "fooSA";
        /*
         * Run Test Case 3
         */
        try {
            deploymentService
                    .getComponentsForDeployedServiceAssembly(serviceAssemblyName);
            fail("No exception was raised");
        } catch (Exception e) {
            assertTrue(
                    "Wrong exception thrown",
                    e
                            .getMessage()
                            .contains(
                                    "Cannot retrieve service assembly with the specified name : "));
        }

        /*
         * Test Case 4 The service assembly has one service unit
         */
        /*
         * Create Test parameters
         */
        lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        serviceAssemblyName = "fooSA";
        ServiceAssemblyLifeCycle saLifeCycle = createMock(ServiceAssemblyLifeCycle.class);
        ServiceAssembly sa = createMock(ServiceAssembly.class);
        List<ServiceUnit> sus = new ArrayList<ServiceUnit>();
        ServiceUnit su = createMock(ServiceUnit.class);
        sus.add(su);
        String componentName = "componentName";
        /*
         * Initialize Mocks
         */
        expect(lifeCycleManager.getServiceAssemblyByName(serviceAssemblyName))
                .andReturn(saLifeCycle);
        expect(saLifeCycle.getServiceAssembly()).andReturn(sa);
        expect(sa.getServiceUnits()).andReturn(sus);
        expect(su.getTargetComponentName()).andReturn(componentName);
        replay(lifeCycleManager);
        replay(sa);
        replay(saLifeCycle);
        replay(su);
        deploymentService.managerService = lifeCycleManager;
        String[] result = null;
        /*
         * Run Test Case 4
         */
        try {
            result = deploymentService
                    .getComponentsForDeployedServiceAssembly(serviceAssemblyName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        assertNotNull(
                "The result of getComponentsForDeployedServiceAssembly must not be null",
                result);
        assertEquals("Wrong invokation result", result[0], componentName);
        assertEquals("Wrong invokation result", result.length, 1);
        verify(lifeCycleManager);
        verify(sa);
        verify(saLifeCycle);
        verify(su);
    }

    public void testGetDeployedServiceAssemblies() {
        /*
         * Test Case 1
         * 
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Create Test parameters
         */
        LifeCycleManagerImpl lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        Map<String, ServiceAssemblyLifeCycle> serviceAssemblies = new HashMap<String, ServiceAssemblyLifeCycle>();
        String saName = "service assembly";
        ServiceAssemblyLifeCycle saLifeCycle = createMock(ServiceAssemblyLifeCycle.class);
        serviceAssemblies.put(saName, saLifeCycle);
        /*
         * Initialize Mocks
         */
        expect(lifeCycleManager.getServiceAssemblyLifeCycles()).andReturn(
                serviceAssemblies);
        replay(lifeCycleManager);
        deploymentService.managerService = lifeCycleManager;
        String[] result = null;
        /*
         * Run Test Case 1
         */
        try {
            result = deploymentService.getDeployedServiceAssemblies();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        assertNotNull(
                "Result of getDeployedServiceAssemblies must not be null",
                result);
        assertEquals("Wrong invokation result", result[0], saName);
        assertEquals("Wrong invokation result", result.length, 1);
        verify(lifeCycleManager);
    }

    public void testGetDeployedServiceAssembliesForComponent() {
        /*
         * Test Case 1 The parameter componentName is null, an exception is
         * raised
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Create Test parameters
         */
        String componentName = null;
        LoggingUtil log = createMock(LoggingUtil.class);
        deploymentService.log = log;
        /*
         * Run Test Case 1
         */
        try {
            deploymentService
                    .getDeployedServiceAssembliesForComponent(componentName);
            fail("No exception raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be null"));
        }
        /*
         * Test Case 2 The parameter componentName is empty, an exception is
         * raised
         */
        /*
         * Create Test parameters
         */
        componentName = "";
        /*
         * Run Test Case 2
         */
        try {
            deploymentService
                    .getDeployedServiceAssembliesForComponent(componentName);
            fail("No exception raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be empty"));
        }

        /*
         * Test Case 3
         */
        /*
         * Create Test parameters
         */
        componentName = "fooComponent";
        LifeCycleManagerImpl lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        Map<String, ServiceAssemblyLifeCycle> serviceAssemblies = new HashMap<String, ServiceAssemblyLifeCycle>();
        String saName = "service assembly";
        ServiceAssemblyLifeCycle saLifeCycle = createMock(ServiceAssemblyLifeCycle.class);
        serviceAssemblies.put(saName, saLifeCycle);
        ServiceAssembly sa = createMock(ServiceAssembly.class);
        List<ServiceUnit> sus = new ArrayList<ServiceUnit>();
        ServiceUnit su = createMock(ServiceUnit.class);
        sus.add(su);
        Identification identification = createMock(Identification.class);
        /*
         * Initialize Mocks
         */
        expect(lifeCycleManager.getServiceAssemblyLifeCycles()).andReturn(
                serviceAssemblies);
        expect(saLifeCycle.getServiceAssembly()).andReturn(sa).times(2);
        expect(sa.getServiceUnits()).andReturn(sus);
        expect(sa.getIdentification()).andReturn(identification);
        expect(identification.getName()).andReturn(saName);
        expect(su.getTargetComponentName()).andReturn(componentName);
        replay(sa);
        replay(su);
        replay(lifeCycleManager);
        replay(saLifeCycle);
        replay(identification);
        deploymentService.managerService = lifeCycleManager;
        String[] result = null;
        /*
         * Run Test Case 3
         */
        try {
            result = deploymentService
                    .getDeployedServiceAssembliesForComponent(componentName);
        } catch (Exception e) {
            fail("Error during invokation");
        }
        assertNotNull(
                "Result of getDeployedServiceAssembliesForComponent must not be null",
                result);
        assertEquals("Wrong invokation result", result[0], saName);
        verify(saLifeCycle);
        verify(lifeCycleManager);
        verify(sa);
        verify(su);
        verify(identification);
    }

    public void testGetDeployedServiceUnitList() {
        /*
         * Test Case 1 The parameter componentName is null, an exception is
         * raised
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Create Test parameters
         */
        String componentName = null;
        LoggingUtil log = createMock(LoggingUtil.class);
        deploymentService.log = log;
        /*
         * Run Test Case 1
         */
        try {
            deploymentService.getDeployedServiceUnitList(componentName);
            fail("No exception raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be null"));
        }
        /*
         * Test Case 2 The parameter componentName is empty, an exception is
         * raised
         */
        /*
         * Create Test parameters
         */
        componentName = "";
        /*
         * Run Test Case 2
         */
        try {
            deploymentService.getDeployedServiceUnitList(componentName);
            fail("No exception raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be empty"));
        }

        /*
         * Test Case 3
         */
        /*
         * Create Test parameters
         */
        componentName = "fooComponent";
        LifeCycleManagerImpl lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        Map<String, ServiceAssemblyLifeCycle> serviceAssemblies = new HashMap<String, ServiceAssemblyLifeCycle>();
        String suName = "service unit";
        ServiceAssemblyLifeCycle saLifeCycle = createMock(ServiceAssemblyLifeCycle.class);
        serviceAssemblies.put(suName, saLifeCycle);
        ServiceAssembly sa = createMock(ServiceAssembly.class);
        List<ServiceUnit> sus = new ArrayList<ServiceUnit>();
        ServiceUnit su = createMock(ServiceUnit.class);
        sus.add(su);
        Identification identification = createMock(Identification.class);
        /*
         * Initialize Mocks
         */
        expect(lifeCycleManager.getServiceAssemblyLifeCycles()).andReturn(
                serviceAssemblies);
        expect(saLifeCycle.getServiceAssembly()).andReturn(sa);
        expect(sa.getServiceUnits()).andReturn(sus);
        expect(identification.getName()).andReturn(suName);
        expect(su.getTargetComponentName()).andReturn(componentName);
        expect(su.getIdentification()).andReturn(identification);
        replay(sa);
        replay(su);
        replay(lifeCycleManager);
        replay(saLifeCycle);
        replay(identification);
        deploymentService.managerService = lifeCycleManager;
        String[] result = null;
        /*
         * Run Test Case 3
         */
        try {
            result = deploymentService
                    .getDeployedServiceUnitList(componentName);
        } catch (Exception e) {
            fail("Error during invokation");
        }
        assertNotNull(
                "Result of getDeployedServiceAssembliesForComponent must not be null",
                result);
        assertEquals("Wrong invokation result", result[0], suName);
        verify(saLifeCycle);
        verify(lifeCycleManager);
        verify(sa);
        verify(su);
        verify(identification);
    }

    public void testGetFcState() {
        /*
         * Test Case 1
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Run Test Case 1
         */
        String result = deploymentService.getFcState();
        assertNotNull("Result of getFcState must not be null", result);
        assertEquals("Wrong invokation result", result,
                org.objectweb.fractal.api.control.LifeCycleController.STOPPED);
    }

    public void testGetServiceAssemblyDescriptor() {
        /*
         * Test Case 1 The parameter ServiceAssemblyName is null, an exception
         * is raised
         */
        /*
         * Create Mocks
         */
        Method method1 = null;
        try {
            method1 = DeploymentService.class.getDeclaredMethod(
                    "getJbiDescriptorAsString", new Class[] {URI.class});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mocked methods");
        }
        Method[] mockedMethods = new Method[] {method1};
        DeploymentService deploymentService = createMock(
                DeploymentService.class, mockedMethods);
        /*
         * Create Test parameters
         */
        String serviceAssemblyName = null;
        LoggingUtil log = createMock(LoggingUtil.class);
        deploymentService.log = log;
        /*
         * Run Test Case 1
         */
        try {
            deploymentService.getServiceAssemblyDescriptor(serviceAssemblyName);
            fail("No exception raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    " must not be null"));
        }
        /*
         * Test Case 2 The parameter ServiceAssemblyName is empty, an exception
         * is raised
         */
        /*
         * Create Test parameters
         */
        serviceAssemblyName = "";
        /*
         * Run Test Case 2
         */
        try {
            deploymentService.getServiceAssemblyDescriptor(serviceAssemblyName);
            fail("No exception raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be empty"));
        }

        /*
         * Test Case 3
         */
        /*
         * Create Test parameters
         */
        LifeCycleManagerImpl lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        serviceAssemblyName = "service assembly";
        ServiceAssemblyLifeCycle saLifeCycle = createMock(ServiceAssemblyLifeCycle.class);
        URI installationURI = null;
        try {
            installationURI = new URI("file://installation");
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not create test parameters");
        }
        org.objectweb.petals.util.monolog.wrapper.javalog.Logger logger = createMock(org.objectweb.petals.util.monolog.wrapper.javalog.Logger.class);
        deploymentService.logger = logger;
        String resultDescriptor = "result";
        String result = null;
        /*
         * Initialize Mocks
         */
        expect(lifeCycleManager.getServiceAssemblyByName(serviceAssemblyName))
                .andReturn(saLifeCycle);
        expect(saLifeCycle.getInstallationRoot()).andReturn(installationURI);
        expect(deploymentService.getJbiDescriptorAsString(isA(URI.class)))
                .andReturn(resultDescriptor);
        replay(lifeCycleManager);
        replay(saLifeCycle);
        replay(deploymentService);
        deploymentService.managerService = lifeCycleManager;
        /*
         * Run Test Case 3
         */
        try {
            result = deploymentService
                    .getServiceAssemblyDescriptor(serviceAssemblyName);
        } catch (Exception e) {
            fail("Error during invokation");
        }
        verify(saLifeCycle);
        verify(lifeCycleManager);
        verify(deploymentService);
        assertNotNull(
                "Result of getServiceAssemblyDescriptor must be non null",
                result);
        assertEquals("Wrong invokation result", result, resultDescriptor);
    }

    public void testGetState() {
        /*
         * Test Case 1 The parameter ServiceAssemblyName is null, an exception
         * is raised
         */
        /*
         * Create Mocks
         */
        Method method1 = null;
        try {
            method1 = DeploymentService.class.getDeclaredMethod(
                    "getJbiDescriptorAsString", new Class[] {URI.class});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mocked methods");
        }
        Method[] mockedMethods = new Method[] {method1};
        DeploymentService deploymentService = createMock(
                DeploymentService.class, mockedMethods);
        /*
         * Create Test parameters
         */
        String serviceAssemblyName = null;
        LoggingUtil log = createMock(LoggingUtil.class);
        deploymentService.log = log;
        /*
         * Run Test Case 1
         */
        try {
            deploymentService.getState(serviceAssemblyName);
            fail("No exception raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be null"));
        }
        /*
         * Test Case 2 The parameter ServiceAssemblyName is empty, an exception
         * is raised
         */
        /*
         * Create Test parameters
         */
        serviceAssemblyName = "";
        /*
         * Run Test Case 2
         */
        try {
            deploymentService.getState(serviceAssemblyName);
            fail("No exception raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be empty"));
        }

        /*
         * Test Case 3 A service assembly with this name is found , its state is
         * returned and this state is valid
         */
        /*
         * Create Test parameters
         */
        LifeCycleManagerImpl lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        serviceAssemblyName = "service assembly";
        ServiceAssemblyLifeCycle saLifeCycle = createMock(ServiceAssemblyLifeCycle.class);
        org.objectweb.petals.util.monolog.wrapper.javalog.Logger logger = createMock(org.objectweb.petals.util.monolog.wrapper.javalog.Logger.class);
        deploymentService.logger = logger;
        String result = null;
        /*
         * Initialize Mocks
         */
        expect(lifeCycleManager.getServiceAssemblyByName(serviceAssemblyName))
                .andReturn(saLifeCycle);
        expect(saLifeCycle.getCurrentState()).andReturn(
                DeploymentServiceMBean.SHUTDOWN);
        replay(lifeCycleManager);
        replay(saLifeCycle);
        deploymentService.managerService = lifeCycleManager;
        /*
         * Run Test Case 3
         */
        try {
            result = deploymentService.getState(serviceAssemblyName);
        } catch (Exception e) {
            fail("Error during invokation");
        }
        verify(saLifeCycle);
        verify(lifeCycleManager);
        assertNotNull(
                "Result of getServiceAssemblyDescriptor must be non null",
                result);
        assertTrue("Returned state is not valid",
                result == DeploymentServiceMBean.SHUTDOWN
                        || result == DeploymentServiceMBean.STARTED
                        || result == DeploymentServiceMBean.STOPPED);

        /*
         * Test Case 4 
         * No service assembly with this name is found , an
         * exception is raised
         */
        lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        /*
         * Initialize Mocks
         */
        expect(lifeCycleManager.getServiceAssemblyByName(serviceAssemblyName))
                .andReturn(null);
        replay(lifeCycleManager);
        deploymentService.managerService = lifeCycleManager;
        /*
         * Run Test Case 4
         */
        try {
            deploymentService.getState(serviceAssemblyName);
            fail("Error during invokation");
        } catch (Exception e) {
            verify(lifeCycleManager);
        }

        /*
         * Test Case 5
         *  A service assembly with this name is found , its state is
         * returned but this state is invalid
         */
        reset(saLifeCycle);
        /*
         * Create Test parameters
         */
        lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        /*
         * Initialize Mocks
         */
        expect(lifeCycleManager.getServiceAssemblyByName(serviceAssemblyName))
                .andReturn(saLifeCycle);
        expect(saLifeCycle.getCurrentState()).andReturn(
                "invalidState");
        replay(lifeCycleManager);
        replay(saLifeCycle);
        deploymentService.managerService = lifeCycleManager;
        /*
         * Run Test Case 5
         */
        try {
            deploymentService.getState(serviceAssemblyName);
            fail("No exception was raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown",e.getMessage().equals("Service Assembly state isn't correctly defined"));
            verify(saLifeCycle);
            verify(lifeCycleManager);
        }
    }
    
    public void testInit() {
        /*
         * Test Case 1
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Create Test parameters
         */
        /*
         * Run Test Case 1
         */
        deploymentService.init();
        assertNotNull("Initialization incomplete",deploymentService.packagesLoadedCache);
        assertNotNull("Initialization incomplete",deploymentService.log);
        assertNotNull("Initialization incomplete",deploymentService.packageHandler);
    }

    public void testIsDeployedServiceUnit() {
        /*
         * Test Case 1 The parameter serviceUnitName is null, an exception is
         * raised
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Create Test parameters
         * 
         */
        LoggingUtil log = createMock(LoggingUtil.class);
        deploymentService.log = log;
        String serviceUnitName = null;
        String componentName = null;
        /*
         * Run Test Case 1
         */
        try {
            deploymentService.isDeployedServiceUnit(componentName,
                    serviceUnitName);
            fail("No exception raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be null"));
        }

        /*
         * Test Case 2 The parameter serviceUnitName is empty, an exception is
         * raised
         */
        /*
         * Create Test parameters
         */
        serviceUnitName = "";
        /*
         * Run Test Case 2
         */
        try {
            deploymentService.isDeployedServiceUnit(componentName,
                    serviceUnitName);
            fail("No exception raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be empty"));
        }

        /*
         * Test Case 3 The parameter componentName is null, an exception is
         * raised
         */
        /*
         * Create Test parameters
         */
        serviceUnitName = "service unit name";
        /*
         * Run Test Case 3
         */
        try {
            deploymentService.isDeployedServiceUnit(componentName,
                    serviceUnitName);
            fail("No exception raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be null"));
        }

        /*
         * Test Case 4 The parameter componentName is empty, an exception is
         * raised
         */
        /*
         * Create Test parameters
         */
        componentName = "";
        /*
         * Run Test Case 4
         */
        try {
            deploymentService.isDeployedServiceUnit(componentName,
                    serviceUnitName);
            fail("No exception raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be empty"));
        }

        /*
         * Test Case 5 There is a service unit with the given name deployed for
         * this component
         */
        /*
         * Create Test parameters
         */
        componentName = "fooComponent";
        String saName = "service assembly name";
        LifeCycleManagerImpl lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        Map<String, ServiceAssemblyLifeCycle> serviceAssemblies = new HashMap<String, ServiceAssemblyLifeCycle>();
        ServiceAssemblyLifeCycle saLifeCycle = createMock(ServiceAssemblyLifeCycle.class);
        serviceAssemblies.put(saName, saLifeCycle);
        ServiceAssembly sa = createMock(ServiceAssembly.class);
        List<ServiceUnit> sus = new ArrayList<ServiceUnit>();
        ServiceUnit su = createMock(ServiceUnit.class);
        sus.add(su);
        Identification identification = createMock(Identification.class);
        /*
         * Initialize Mocks
         */
        expect(lifeCycleManager.getServiceAssemblyLifeCycles()).andReturn(
                serviceAssemblies);
        expect(saLifeCycle.getServiceAssembly()).andReturn(sa);
        expect(sa.getServiceUnits()).andReturn(sus);
        expect(identification.getName()).andReturn(serviceUnitName);
        expect(su.getTargetComponentName()).andReturn(componentName);
        expect(su.getIdentification()).andReturn(identification);
        replay(sa);
        replay(su);
        replay(lifeCycleManager);
        replay(saLifeCycle);
        replay(identification);
        deploymentService.managerService = lifeCycleManager;
        boolean result = false;
        /*
         * Run Test Case 5
         */
        try {
            result = deploymentService.isDeployedServiceUnit(componentName,
                    serviceUnitName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        assertTrue("Wrong invokation result", result);
        verify(sa);
        verify(su);
        verify(lifeCycleManager);
        verify(saLifeCycle);
        verify(identification);
    }

    public void testIsLoaded() {
        /*
         * Test Case 1 There is no service assembly with the same id already
         * loaded in the platform
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Create Test parameters
         */
        List<String> loadedPackages = new ArrayList<String>();
        deploymentService.packagesLoadedCache = loadedPackages;
        String saID = "service assembly";
        boolean result = false;
        /*
         * Run Test Case 1
         */
        try {
            result = deploymentService.isLoaded(saID);
        } catch (Exception e) {
            fail("Error during invokation");
        }
        assertFalse("Wrong invokation result", result);
    }

    public void testListFc() {
        /*
         * Test Case 1
         */
        DeploymentService deploymentService = new DeploymentService();
        String[] result = null;
        /*
         * Run Test Case 1
         */
        result = deploymentService.listFc();
        assertNotNull("The result of listFc must not be null", result);
        assertEquals("Wrong invokation result",
                DeploymentService.CLIENT_INTERFACES, result);
    }

    public void testLookupFc() {
        /*
         * Test Case 1 The requested interface does not belong to adminService
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Create test parameters
         */
        String clientItf = "foo";
        /*
         * Run Test Case 1
         */
        try {
            deploymentService.lookupFc(clientItf);
            fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }

        /*
         * Test Case 2 The requested interface belongs to adminService
         */
        /*
         * Create test parameters
         */
        clientItf = InterfaceNamesHelper.JBI_LIFECYCLEMANAGER_NAME;
        Object result = null;
        LifeCycleManagerService lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        deploymentService.managerService = lifeCycleManager;
        /*
         * Run Test Case 2
         */
        try {
            result = deploymentService.lookupFc(clientItf);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Exception during method invokation");
        }
        assertEquals("Wrong result for invokation", result, lifeCycleManager);

        /*
         * Test Case 3 The requested interface belongs to adminService
         */
        /*
         * Create test parameters
         */
        clientItf = InterfaceNamesHelper.PLATFORM_REPOSITORY_NAME;
        RepositoryService repoService = createMock(RepositoryImpl.class);
        deploymentService.repositorySrv = repoService;
        /*
         * Run Test Case 3
         */
        try {
            result = deploymentService.lookupFc(clientItf);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Exception during method invokation");
        }
        assertEquals("Wrong result for invokation", result, repoService);

        /*
         * Test Case 4 The requested interface belongs to adminService
         */
        /*
         * Create test parameters
         */
        clientItf = InterfaceNamesHelper.JBI_ADMIN_NAME;
        AdminServiceMBean adminService = createMock(AdminService.class);
        deploymentService.adminSrv = adminService;
        /*
         * Run Test Case 4
         */
        try {
            result = deploymentService.lookupFc(clientItf);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Exception during method invokation");
        }
        assertEquals("Wrong result for invokation", result, adminService);

        /*
         * Test Case 5 The requested interface belongs to adminService
         */
        /*
         * Create test parameters
         */
        clientItf = InterfaceNamesHelper.JBI_SYSTEMSTATE_NAME;
        SystemState sysState = createMock(SystemStateImpl.class);
        deploymentService.recoverySrv = sysState;
        /*
         * Run Test Case 5
         */
        try {
            result = deploymentService.lookupFc(clientItf);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Exception during method invokation");
        }
        assertEquals("Wrong result for invokation", result, sysState);

        /*
         * Test Case 6 The requested interface belongs to adminService
         */
        /*
         * Create test parameters
         */
        clientItf = InterfaceNamesHelper.JBI_ENDPOINTSERVICE_NAME;
        EndpointService epService = createMock(EndpointServiceImpl.class);
        deploymentService.endpointService = epService;
        /*
         * Run Test Case 6
         */
        try {
            result = deploymentService.lookupFc(clientItf);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Exception during method invokation");
        }
        assertEquals("Wrong result for invokation", result, epService);
    }

    public void testRegisterConnection() {
        /*
         * Test Case 1 
         * The connection does not provide a provider endpoint, the invokation result is not null
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Create Test parameters
         */
        String result = null;
        Connection connection = createMock(Connection.class);
        XMLResult xmlResult = new XMLResult("deploy");
        StringBuffer componentOutput = new StringBuffer();
        String saName = "serviceAssemblyName";
        /*
         * Initialize Mocks
         */
        expect(connection.getProviderEndpointName()).andReturn(null);
        expect(connection.getProviderServiceName()).andReturn(null);
        expect(connection.getConsumerServiceName()).andReturn(null);
        expect(connection.getConsumerEndpointName()).andReturn(null);
        expect(connection.getConsumerInterfaceName()).andReturn(null);
        replay(connection);
        /*
         * Run Test Case 1
         */
        result = deploymentService.registerConnection(connection, xmlResult, componentOutput, saName);
        assertTrue("Wrong invokation result",result.contains("ERROR") && result.contains("FAILED"));
        
        /*
         * Test Case 2 
         * The connection provides a provider endpoint and service and a consumer interface, the invokation result is null
         */
        reset(connection);
        /*
         * Create Test parameters
         */
        QName provService = new QName("provService");
        String provEndpoint = "provEndpoint";
        QName consInterface = new QName("consInterface");
        EndpointService epService = createMock(EndpointServiceImpl.class);
        /*
         * Initialize Mocks
         */
        try {
        expect(connection.getProviderEndpointName()).andReturn(provEndpoint);
        expect(connection.getProviderServiceName()).andReturn(provService);
        expect(connection.getConsumerServiceName()).andReturn(null);
        expect(connection.getConsumerEndpointName()).andReturn(null);
        expect(connection.getConsumerInterfaceName()).andReturn(consInterface);
        epService.createConnection(consInterface, provService, provEndpoint);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(connection);
        replay(epService);
        deploymentService.endpointService = epService;
        /*
         * Run Test Case 2
         */
        result = deploymentService.registerConnection(connection, xmlResult, componentOutput, saName);
        assertNull("Wrong invokation result",result);
        verify(connection);
        verify(epService);
        
        /*
         * Test Case 3 
         * The connection provides a provider endpoint and service, the invokation result is not null
         */
        reset(connection);
        reset(epService);
        /*
         * Create Test parameters
         */
        /*
         * Initialize Mocks
         */
        try {
        expect(connection.getProviderEndpointName()).andReturn(provEndpoint);
        expect(connection.getProviderServiceName()).andReturn(provService);
        expect(connection.getConsumerServiceName()).andReturn(null);
        expect(connection.getConsumerEndpointName()).andReturn(null);
        expect(connection.getConsumerInterfaceName()).andReturn(null);
        epService.createConnection(null, null, provService, provEndpoint);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(connection);
        replay(epService);
        deploymentService.endpointService = epService;
        /*
         * Run Test Case 3
         */
        result = deploymentService.registerConnection(connection, xmlResult, componentOutput, saName);
        assertNull("Wrong invokation result",result);
        verify(connection);
        verify(epService);
        
        /*
         * Test Case 4 
         * The connection provides a provider endpoint and service, an exception is raised during
         * createConnection invokation
         */
        reset(connection);
        reset(epService);
        /*
         * Create Test parameters
         */
        epService = createMock(MockEndpointService.class,new Method[]{});
        xmlResult = new XMLResult("deploy");
        /*
         * Initialize Mocks
         */
        try {
        expect(connection.getProviderEndpointName()).andReturn(provEndpoint);
        expect(connection.getProviderServiceName()).andReturn(provService);
        expect(connection.getConsumerServiceName()).andReturn(null);
        expect(connection.getConsumerEndpointName()).andReturn(null);
        expect(connection.getConsumerInterfaceName()).andReturn(null);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(connection);
        deploymentService.endpointService = epService;
        /*
         * Run Test Case 4
         */
        result = deploymentService.registerConnection(connection, xmlResult, componentOutput, saName);
        assertTrue("Wrong invokation result",result.contains("ERROR") && result.contains("FAILED"));
        verify(connection);
    }
    
    public void testShutdown() {
        /*
         * Test Case 1 serviceAssemblyName parameter is null, an exception is
         * raised
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Create Test parameters
         */
        LoggingUtil log = createMock(LoggingUtil.class);
        deploymentService.log = log;
        String serviceAssemblyName = null;
        /*
         * Run Test Case 1
         */
        try {
            deploymentService.shutDown(serviceAssemblyName);
            fail("No exception was raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be null"));
        }
        /*
         * Test Case 2 serviceAssemblyName parameter is empty, an exception is
         * raised
         */
        /*
         * Create Test parameters
         */
        serviceAssemblyName = "";
        /*
         * Run Test Case 2
         */
        try {
            deploymentService.shutDown(serviceAssemblyName);
            fail("No exception was raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be empty"));
        }
        /*
         * Test Case 3 No service assembly life cycle is found for this service
         * assembly
         */
        serviceAssemblyName = "service assembly";
        LifeCycleManagerImpl lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        deploymentService.managerService = lifeCycleManager;
        String result = null;
        /*
         * Run Test Case 3
         */
        try {
            result = deploymentService.shutDown(serviceAssemblyName);
        } catch (Exception e) {
            fail("Error during invokation");
        }
        assertNotNull("Result of shutdown must not be null", result);
        assertFalse("Result of shutdown must not be empty",
                result.length() == 0);
        assertTrue("Wrong invokation result", result.contains("FAILED"));

        /*
         * Test Case 4 One service assembly life cycle is found for this service
         * assembly and the shutdown operation works with no error
         */
        serviceAssemblyName = "service assembly";
        lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        deploymentService.managerService = lifeCycleManager;
        ServiceAssemblyLifeCycle saLifeCycle = createMock(ServiceAssemblyLifeCycle.class);
        result = null;
        /*
         * Initialize Mocks
         */
        expect(lifeCycleManager.getServiceAssemblyByName(serviceAssemblyName))
                .andReturn(saLifeCycle);
        replay(lifeCycleManager);
        deploymentService.managerService = lifeCycleManager;
        /*
         * Run Test Case 4
         */
        try {
            result = deploymentService.shutDown(serviceAssemblyName);
        } catch (Exception e) {
            fail("Error during invokation");
        }
        assertNotNull("Result of shutdown must not be null", result);
        assertFalse("Result of shutdown must not be empty",
                result.length() == 0);
        assertTrue("Wrong invokation result", result.contains("SUCCESS"));
        verify(lifeCycleManager);
        /*
         * Test Case 5 One service assembly life cycle is found for this service
         * assembly and the shutdown operation throw an exception
         */
        Method method1 = null;
        try {
            method1 = LifeCycleAbstract.class.getDeclaredMethod(
                    "getCurrentState", new Class[] {});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mocked method");
        }
        Method[] mockedMethods = new Method[] {method1};
        serviceAssemblyName = "service assembly";
        lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        deploymentService.managerService = lifeCycleManager;
        saLifeCycle = createMock(MockServiceAssemblyLifeCycle.class,
                mockedMethods);
        result = null;
        /*
         * Initialize Mocks
         */
        expect(lifeCycleManager.getServiceAssemblyByName(serviceAssemblyName))
                .andReturn(saLifeCycle);
        expect(saLifeCycle.getCurrentState())
                .andReturn(LifeCycleMBean.SHUTDOWN);
        replay(lifeCycleManager);
        replay(saLifeCycle);
        deploymentService.managerService = lifeCycleManager;
        /*
         * Run Test Case 5
         */
        try {
            result = deploymentService.shutDown(serviceAssemblyName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        assertNotNull("Result of shutdown must not be null", result);
        assertFalse("Result of shutdown must not be empty",
                result.length() == 0);
        assertTrue("Wrong invokation result", result.contains("FAILED"));
        verify(lifeCycleManager);
        verify(saLifeCycle);
        
        /*
         * Test Case 6 One service assembly life cycle is found for this service
         * assembly and the shutdown operation throw an exception
         */
        reset(lifeCycleManager);
        reset(saLifeCycle);
        serviceAssemblyName = "service assembly";
        deploymentService.managerService = lifeCycleManager;
        saLifeCycle = createMock(MockServiceAssemblyLifeCycle2.class,
                mockedMethods);
        result = null;
        /*
         * Initialize Mocks
         */
        expect(lifeCycleManager.getServiceAssemblyByName(serviceAssemblyName))
                .andReturn(saLifeCycle);
        expect(saLifeCycle.getCurrentState())
                .andReturn(LifeCycleMBean.SHUTDOWN);
        replay(lifeCycleManager);
        replay(saLifeCycle);
        deploymentService.managerService = lifeCycleManager;
        /*
         * Run Test Case 6
         */
        try {
            result = deploymentService.shutDown(serviceAssemblyName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        assertNotNull("Result of shutdown must not be null", result);
        assertFalse("Result of shutdown must not be empty",
                result.length() == 0);
        assertTrue("Wrong invokation result", result.contains("FAILED"));
        verify(lifeCycleManager);
        verify(saLifeCycle);
    }
    
    public void testShutdown2() {
        /*
         * Test Case 1 
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Create Test parameters
         */
        LoggingUtil log = createMock(LoggingUtil.class);
        deploymentService.log = log;
        List<String> loadedPackages = new ArrayList<String>();
        deploymentService.packagesLoadedCache = loadedPackages;
        Logger logger = createMock(LoggerImpl.class);
        deploymentService.logger = logger;
        /*
         * Run Test Case 1
         */
        deploymentService.shutdown();
        assertNull("ShutDown action was not properly realized",deploymentService.logger);
        assertNull("ShutDown action was not properly realized",deploymentService.packagesLoadedCache);
    }

    public void testStart() {
        /*
         * Test Case 1 serviceAssemblyName parameter is null, an exception is
         * raised
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Create Test parameters
         */
        LoggingUtil log = createMock(LoggingUtil.class);
        deploymentService.log = log;
        String serviceAssemblyName = null;
        /*
         * Run Test Case 1
         */
        try {
            deploymentService.start(serviceAssemblyName);
            fail("No exception was raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be null"));
        }
        /*
         * Test Case 2 serviceAssemblyName parameter is empty, an exception is
         * raised
         */
        /*
         * Create Test parameters
         */
        serviceAssemblyName = "";
        /*
         * Run Test Case 2
         */
        try {
            deploymentService.start(serviceAssemblyName);
            fail("No exception was raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be empty"));
        }

        /*
         * Test Case 3 No service assembly life cycle is found for this service
         * assembly
         */
        serviceAssemblyName = "service assembly";
        LifeCycleManagerImpl lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        deploymentService.managerService = lifeCycleManager;
        String result = null;
        /*
         * Run Test Case 3
         */
        try {
            result = deploymentService.start(serviceAssemblyName);
        } catch (Exception e) {
            fail("Error during invokation");
        }
        assertNotNull("Result of shutdown must not be null", result);
        assertFalse("Result of shutdown must not be empty",
                result.length() == 0);
        assertTrue("Wrong invokation result", result.contains("FAILED"));

        /*
         * Test Case 4 One service assembly life cycle is found for this service
         * assembly and the shutdown operation works with no error
         */
        serviceAssemblyName = "service assembly";
        lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        deploymentService.managerService = lifeCycleManager;
        ServiceAssemblyLifeCycle saLifeCycle = createMock(ServiceAssemblyLifeCycle.class);
        result = null;
        /*
         * Initialize Mocks
         */
        expect(lifeCycleManager.getServiceAssemblyByName(serviceAssemblyName))
                .andReturn(saLifeCycle);
        replay(lifeCycleManager);
        deploymentService.managerService = lifeCycleManager;
        /*
         * Run Test Case 4
         */
        try {
            result = deploymentService.start(serviceAssemblyName);
        } catch (Exception e) {
            fail("Error during invokation");
        }
        assertNotNull("Result of shutdown must not be null", result);
        assertFalse("Result of shutdown must not be empty",
                result.length() == 0);
        assertTrue("Wrong invokation result", result.contains("SUCCESS"));
        verify(lifeCycleManager);
        /*
         * Test Case 5 One service assembly life cycle is found for this service
         * assembly and the shutdown operation throw an exception
         */
        Method method1 = null;
        try {
            method1 = LifeCycleAbstract.class.getDeclaredMethod(
                    "getCurrentState", new Class[] {});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mocked method");
        }
        Method[] mockedMethods = new Method[] {method1};
        serviceAssemblyName = "service assembly";
        lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        deploymentService.managerService = lifeCycleManager;
        saLifeCycle = createMock(MockServiceAssemblyLifeCycle.class,
                mockedMethods);
        result = null;
        /*
         * Initialize Mocks
         */
        expect(lifeCycleManager.getServiceAssemblyByName(serviceAssemblyName))
                .andReturn(saLifeCycle);
        expect(saLifeCycle.getCurrentState())
                .andReturn(LifeCycleMBean.SHUTDOWN);
        replay(lifeCycleManager);
        replay(saLifeCycle);
        deploymentService.managerService = lifeCycleManager;
        /*
         * Run Test Case 5
         */
        try {
            result = deploymentService.start(serviceAssemblyName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        assertNotNull("Result of shutdown must not be null", result);
        assertFalse("Result of shutdown must not be empty",
                result.length() == 0);
        assertTrue("Wrong invokation result", result.contains("FAILED"));
        verify(lifeCycleManager);
        verify(saLifeCycle);
        
        /*
         * Test Case 6 One service assembly life cycle is found for this service
         * assembly and the shutdown operation throw an exception wich does not have a cause
         */
        reset(lifeCycleManager);
        serviceAssemblyName = "service assembly";
        deploymentService.managerService = lifeCycleManager;
        saLifeCycle = createMock(MockServiceAssemblyLifeCycle2.class,
                mockedMethods);
        result = null;
        /*
         * Initialize Mocks
         */
        expect(lifeCycleManager.getServiceAssemblyByName(serviceAssemblyName))
                .andReturn(saLifeCycle);
        expect(saLifeCycle.getCurrentState())
                .andReturn(LifeCycleMBean.SHUTDOWN);
        replay(lifeCycleManager);
        replay(saLifeCycle);
        deploymentService.managerService = lifeCycleManager;
        /*
         * Run Test Case 6
         */
        try {
            result = deploymentService.start(serviceAssemblyName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        assertNotNull("Result of shutdown must not be null", result);
        assertFalse("Result of shutdown must not be empty",
                result.length() == 0);
        assertTrue("Wrong invokation result", result.contains("FAILED"));
        verify(lifeCycleManager);
        verify(saLifeCycle);
    }

    public void testStartFc() {
        /*
         * Test Case 1
         */
        /*
         * Create Mocks
         */
        Method method1 = null;
        try {
            method1 = DeploymentService.class.getDeclaredMethod("init",
                    new Class[] {});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mocked methods");
        }
        Method[] mockedMethods = new Method[] {method1};
        DeploymentService deploymentService = createMock(
                DeploymentService.class, mockedMethods);
        deploymentService.init();
        replay(deploymentService);
        /*
         * Run Test Case 1
         */
        try {
            deploymentService.startFc();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        assertEquals("DeploymentService was not properly initialized",
                deploymentService.fcState,
                org.objectweb.fractal.api.control.LifeCycleController.STARTED);
        verify(deploymentService);
    }

    public void testStop() {
        /*
         * Test Case 1 serviceAssemblyName parameter is null, an exception is
         * raised
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Create Test parameters
         */
        LoggingUtil log = createMock(LoggingUtil.class);
        deploymentService.log = log;
        String serviceAssemblyName = null;
        /*
         * Run Test Case 1
         */
        try {
            deploymentService.stop(serviceAssemblyName);
            fail("No exception was raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be null"));
        }
        /*
         * Test Case 2 serviceAssemblyName parameter is empty, an exception is
         * raised
         */
        /*
         * Create Test parameters
         */
        serviceAssemblyName = "";
        /*
         * Run Test Case 2
         */
        try {
            deploymentService.stop(serviceAssemblyName);
            fail("No exception was raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be empty"));
        }

        /*
         * Test Case 3 No service assembly life cycle is found for this service
         * assembly
         */
        serviceAssemblyName = "service assembly";
        LifeCycleManagerImpl lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        deploymentService.managerService = lifeCycleManager;
        String result = null;
        /*
         * Run Test Case 3
         */
        try {
            result = deploymentService.stop(serviceAssemblyName);
        } catch (Exception e) {
            fail("Error during invokation");
        }
        assertNotNull("Result of shutdown must not be null", result);
        assertFalse("Result of shutdown must not be empty",
                result.length() == 0);
        assertTrue("Wrong invokation result", result.contains("FAILED"));

        /*
         * Test Case 4 One service assembly life cycle is found for this service
         * assembly and the shutdown operation works with no error
         */
        serviceAssemblyName = "service assembly";
        lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        deploymentService.managerService = lifeCycleManager;
        ServiceAssemblyLifeCycle saLifeCycle = createMock(ServiceAssemblyLifeCycle.class);
        result = null;
        /*
         * Initialize Mocks
         */
        expect(lifeCycleManager.getServiceAssemblyByName(serviceAssemblyName))
                .andReturn(saLifeCycle);
        replay(lifeCycleManager);
        deploymentService.managerService = lifeCycleManager;
        /*
         * Run Test Case 4
         */
        try {
            result = deploymentService.stop(serviceAssemblyName);
        } catch (Exception e) {
            fail("Error during invokation");
        }
        assertNotNull("Result of shutdown must not be null", result);
        assertFalse("Result of shutdown must not be empty",
                result.length() == 0);
        assertTrue("Wrong invokation result", result.contains("SUCCESS"));
        verify(lifeCycleManager);
        /*
         * Test Case 5 One service assembly life cycle is found for this service
         * assembly and the shutdown operation throw an exception
         */
        Method method1 = null;
        try {
            method1 = LifeCycleAbstract.class.getDeclaredMethod(
                    "getCurrentState", new Class[] {});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mocked method");
        }
        Method[] mockedMethods = new Method[] {method1};
        serviceAssemblyName = "service assembly";
        lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        deploymentService.managerService = lifeCycleManager;
        saLifeCycle = createMock(MockServiceAssemblyLifeCycle.class,
                mockedMethods);
        result = null;
        /*
         * Initialize Mocks
         */
        expect(lifeCycleManager.getServiceAssemblyByName(serviceAssemblyName))
                .andReturn(saLifeCycle);
        expect(saLifeCycle.getCurrentState())
                .andReturn(LifeCycleMBean.SHUTDOWN);
        replay(lifeCycleManager);
        replay(saLifeCycle);
        deploymentService.managerService = lifeCycleManager;
        /*
         * Run Test Case 5
         */
        try {
            result = deploymentService.stop(serviceAssemblyName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        assertNotNull("Result of shutdown must not be null", result);
        assertFalse("Result of shutdown must not be empty",
                result.length() == 0);
        assertTrue("Wrong invokation result", result.contains("FAILED"));
        verify(lifeCycleManager);
        verify(saLifeCycle);
        
        /*
        * Test Case 6 One service assembly life cycle is found for this service
        * assembly and the shutdown operation throw an exception which does not have a cause
        */
       reset(lifeCycleManager);
       serviceAssemblyName = "service assembly";
       lifeCycleManager = createMock(LifeCycleManagerImpl.class);
       deploymentService.managerService = lifeCycleManager;
       saLifeCycle = createMock(MockServiceAssemblyLifeCycle2.class,
               mockedMethods);
       result = null;
       /*
        * Initialize Mocks
        */
       expect(lifeCycleManager.getServiceAssemblyByName(serviceAssemblyName))
               .andReturn(saLifeCycle);
       expect(saLifeCycle.getCurrentState())
               .andReturn(LifeCycleMBean.SHUTDOWN);
       replay(lifeCycleManager);
       replay(saLifeCycle);
       deploymentService.managerService = lifeCycleManager;
       /*
        * Run Test Case 6
        */
       try {
           result = deploymentService.stop(serviceAssemblyName);
       } catch (Exception e) {
           e.printStackTrace();
           fail("Error during invokation");
       }
       assertNotNull("Result of shutdown must not be null", result);
       assertFalse("Result of shutdown must not be empty",
               result.length() == 0);
       assertTrue("Wrong invokation result", result.contains("FAILED"));
       verify(lifeCycleManager);
       verify(saLifeCycle);
    }

    public void testStopFc() {
        /*
         * Test Case 1
         */
        /*
         * Create Mocks
         */
        Method method1 = null;
        try {
            method1 = DeploymentService.class.getDeclaredMethod("shutdown",
                    new Class[] {});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mocked methods");
        }
        Method[] mockedMethods = new Method[] {method1};
        DeploymentService deploymentService = createMock(
                DeploymentService.class, mockedMethods);
        deploymentService.shutdown();
        replay(deploymentService);
        /*
         * Run Test Case 1
         */
        try {
            deploymentService.stopFc();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        assertEquals("DeploymentService was not properly initialized",
                deploymentService.fcState,
                org.objectweb.fractal.api.control.LifeCycleController.STOPPED);
        verify(deploymentService);
    }
    
    public void TestUpdateExceptionMessage() {
        /*
         * Test Case 1 
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Create Test parameters
         */
        String suName = "serviceUnitName";
        int posParams = 3;
        String exceptionMessage = "";
        List<String> exceptionParams = new ArrayList<String>();
        String targetedComponent = "componentName";
        String message = "This is the exception message";
        String result = null;
        /*
         * Run Test Case 1
         */
        result = deploymentService.updateExceptionMessage(exceptionParams, exceptionMessage, posParams, suName, targetedComponent, message, true);
        assertEquals("Wrong invokation result",exceptionParams.size(), 2);
        assertTrue("Wrong invokation result",exceptionParams.contains(suName));
        assertTrue("Wrong invokation result",exceptionParams.contains(targetedComponent));
        assertTrue("Wrong invokation result",result.contains(message));
    }
    public void testUnbindFc() {
        /*
         * Test Case 1 The parameter clientItfName is null, an exception will be
         * raised
         */
        DeploymentService deploymentService = new DeploymentService();
        /*
         * Create Test parameters
         */
        String clientItfName = null;
        Object serverItf = null;
        /*
         * Run Test Case 1
         */
        try {
            deploymentService.unbindFc(clientItfName);
            fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }

        /*
         * Test Case 2 The parameter clientItfName match the logger interface
         * name The logger attribute of deploymentService will be set to null
         */
        serverItf = createMock(LoggerImpl.class);
        deploymentService.logger = (Logger) serverItf;
        clientItfName = InterfaceNamesHelper.LOGGER_NAME;
        /*
         * Run Test Case 2
         */
        try {
            deploymentService.unbindFc(clientItfName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("logger attribute was not properly set",
                deploymentService.logger, null);

        /*
         * Test Case 3 The parameter clientItfName match the
         * InterfaceNamesHelper.JBI_LIFECYCLEMANAGER_NAME The managerService
         * attribute of deploymentService will be set to null
         */
        serverItf = createMock(LifeCycleManagerImpl.class);
        deploymentService.managerService = (LifeCycleManagerService) serverItf;
        clientItfName = InterfaceNamesHelper.JBI_LIFECYCLEMANAGER_NAME;
        /*
         * Run Test Case 3
         */
        try {
            deploymentService.unbindFc(clientItfName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("manager attribute was not properly set",
                deploymentService.managerService, null);

        /*
         * Test Case 4 The parameter clientItfName match the
         * InterfaceNamesHelper.PLATFORM_REPOSITORY_NAME The repositorySrv
         * attribute of deploymentService will be set to null
         */
        serverItf = createMock(RepositoryImpl.class);
        clientItfName = InterfaceNamesHelper.PLATFORM_REPOSITORY_NAME;
        deploymentService.repositorySrv = (RepositoryService) serverItf;
        /*
         * Run Test Case 4
         */
        try {
            deploymentService.unbindFc(clientItfName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("manager attribute was not properly set",
                deploymentService.repositorySrv, null);

        /*
         * Test Case 5 The parameter clientItfName match the
         * InterfaceNamesHelper.JBI_ADMIN_NAMEThe adminSrv attribute of
         * deploymentService will be set to null
         */
        serverItf = createMock(AdminService.class);
        clientItfName = InterfaceNamesHelper.JBI_ADMIN_NAME;
        deploymentService.adminSrv = (javax.jbi.management.AdminServiceMBean) serverItf;
        /*
         * Run Test Case 5
         */
        try {
            deploymentService.unbindFc(clientItfName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("manager attribute was not properly set",
                deploymentService.adminSrv, null);

        /*
         * Test Case 6 The parameter clientItfName match the
         * InterfaceNamesHelper.JBI_SYSTEMSTATE_NAME The recoverySrv attribute
         * of deploymentService will be set to null
         */
        serverItf = createMock(SystemStateImpl.class);
        clientItfName = InterfaceNamesHelper.JBI_SYSTEMSTATE_NAME;
        deploymentService.recoverySrv = (SystemState) serverItf;
        /*
         * Run Test Case 6
         */
        try {
            deploymentService.unbindFc(clientItfName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("manager attribute was not properly set",
                deploymentService.recoverySrv, null);

        /*
         * Test Case 7 The parameter clientItfName match the
         * InterfaceNamesHelper.JBI_ENDPOINTSERVICE_NAME The recoverySrv
         * attribute of endpointService will be set to serverItf value
         */
        serverItf = createMock(EndpointServiceImpl.class);
        clientItfName = InterfaceNamesHelper.JBI_ENDPOINTSERVICE_NAME;
        deploymentService.endpointService = (EndpointService) serverItf;
        /*
         * Run Test Case 7
         */
        try {
            deploymentService.unbindFc(clientItfName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("manager attribute was not properly set",
                deploymentService.endpointService, null);
    }

    public void testUndeploy() {
        /*
         * Test Case 1 The parameter serviceAssemblyZipUrl is null, an exception
         * will be raised
         */
        /*
         * Create Mocks
         */
        Method method1 = null;
        try {
            method1 = DeploymentService.class.getDeclaredMethod(
                    "isValidComponentTask", new Class[] {String.class,
                        InputStream.class});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mocked methods");
        }
        Method[] mockedMethods = new Method[] {method1};
        DeploymentService deploymentService = createMock(
                DeploymentService.class, mockedMethods);
        /*
         * Create Test parameters
         */
        LoggingUtil log = createMock(LoggingUtil.class);
        deploymentService.log = log;
        String serviceAssemblyName = null;
        /*
         * Run Test Case 1
         */
        try {
            deploymentService.undeploy(serviceAssemblyName);
            fail("No exception was raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be null"));
        }

        /*
         * Test Case 2 The parameter serviceAssemblyZipUrl is empty, an
         * exception will be raised
         */
        /*
         * Create Test parameters
         */
        serviceAssemblyName = "";
        /*
         * Run Test Case 2
         */
        try {
            deploymentService.undeploy(serviceAssemblyName);
            fail("No exception was raised");
        } catch (Exception e) {
            assertTrue("Wrong exception thrown", e.getMessage().contains(
                    "must not be empty"));
        }

        /*
         * Test Case 3 No component matching the service unit target component
         * is found There is connection for this service assembly which provides
         * a consumer interface
         */
        /*
         * Create Test parameters
         */
        serviceAssemblyName = "service assembly";
        LifeCycleManagerService lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        ServiceAssemblyLifeCycle saLifeCycle = createMock(ServiceAssemblyLifeCycle.class);
        ServiceAssembly sa = createMock(ServiceAssembly.class);
        String result = null;
        List<ServiceUnit> sus = new ArrayList<ServiceUnit>();
        ServiceUnit su = createMock(ServiceUnit.class);
        sus.add(su);
        Identification identification = createMock(Identification.class);
        Map<String, ServiceUnitLifeCycle> sulfs = new HashMap<String, ServiceUnitLifeCycle>();
        ServiceUnitLifeCycle sulf = createMock(ServiceUnitLifeCycle.class);
        String suName = "serviceUnitName";
        sulfs.put(suName, sulf);
        String componentName = "fooComponent";
        Connection connection = createMock(Connection.class);
        List<Connection> connections = new ArrayList<Connection>();
        connections.add(connection);
        QName consInterface = new QName("consumerItf");
        QName provService = new QName("providerItf");
        String provEndpoint = "providerEndpoint";
        EndpointService epSrv = createMock(EndpointServiceImpl.class);
        SystemState sysState = createMock(SystemStateImpl.class);
        List<String> loadedPackages = new ArrayList<String>();
        deploymentService.packagesLoadedCache = loadedPackages;
        /*
         * Initialize Mocks
         */
        try {
            expect(
                    lifeCycleManager
                            .getServiceAssemblyByName(serviceAssemblyName))
                    .andReturn(saLifeCycle);
            expect(lifeCycleManager.getComponentByName(componentName))
                    .andReturn(null);
            lifeCycleManager.unregisterServiceAssembly(serviceAssemblyName);
            expect(saLifeCycle.getCurrentState()).andReturn(
                    LifeCycleMBean.STARTED);
            saLifeCycle.stop();
            saLifeCycle.shutDown();
            expect(saLifeCycle.getCurrentState()).andReturn(
                    LifeCycleMBean.SHUTDOWN);
            expect(saLifeCycle.getServiceAssembly()).andReturn(sa).anyTimes();
            expect(saLifeCycle.getServiceUnitsLifeCycles()).andReturn(sulfs);
            saLifeCycle.removePackage();
            expect(sa.getServiceUnits()).andReturn(sus).anyTimes();
            expect(sa.getConnections()).andReturn(connections);
            expect(su.getIdentification()).andReturn(identification);
            expect(su.getTargetComponentName()).andReturn(componentName);
            expect(identification.getName()).andReturn(suName);
            expect(connection.getConsumerInterfaceName()).andReturn(
                    consInterface);
            expect(connection.getProviderServiceName()).andReturn(provService);
            expect(connection.getProviderEndpointName())
                    .andReturn(provEndpoint);
            expect(connection.getConsumerServiceName()).andReturn(null);
            expect(connection.getConsumerEndpointName()).andReturn(null);
            epSrv.deleteConnection(consInterface, provService, provEndpoint);
            expect(
                    sysState
                            .deleteServiceAssemblyStateHolder(serviceAssemblyName))
                    .andReturn(null);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(sa);
        replay(su);
        replay(lifeCycleManager);
        replay(saLifeCycle);
        replay(identification);
        replay(connection);
        replay(epSrv);
        replay(sysState);
        deploymentService.managerService = lifeCycleManager;
        deploymentService.endpointService = epSrv;
        deploymentService.recoverySrv = sysState;
        /*
         * 
         * Run Test Case 3
         */
        try {
            result = deploymentService.undeploy(serviceAssemblyName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        assertNotNull("deploy result must be non null", result);
        assertFalse("deploy result must be non empty", result.length() == 0);
        assertTrue("Wrong invokation result", result.contains("SUCCESS"));
        verify(sa);
        verify(su);
        verify(lifeCycleManager);
        verify(saLifeCycle);
        verify(identification);
        verify(connection);
        verify(epSrv);
        verify(sysState);

        /*
         * Test Case 4 No component matching the service unit target component
         * is found There is connection for this service assembly which does not
         * provide a consumer interface
         */
        reset(sa);
        reset(su);
        reset(lifeCycleManager);
        reset(saLifeCycle);
        reset(identification);
        reset(connection);
        reset(epSrv);
        reset(sysState);
        /*
         * Create Test parameters
         */
        deploymentService.log = log;
        deploymentService.packagesLoadedCache = loadedPackages;
        consInterface = null;
        /*
         * Initialize Mocks
         */
        try {
            expect(
                    lifeCycleManager
                            .getServiceAssemblyByName(serviceAssemblyName))
                    .andReturn(saLifeCycle);
            expect(lifeCycleManager.getComponentByName(componentName))
                    .andReturn(null);
            lifeCycleManager.unregisterServiceAssembly(serviceAssemblyName);
            expect(saLifeCycle.getCurrentState()).andReturn(
                    LifeCycleMBean.STARTED);
            saLifeCycle.stop();
            saLifeCycle.shutDown();
            expect(saLifeCycle.getCurrentState()).andReturn(
                    LifeCycleMBean.SHUTDOWN);
            expect(saLifeCycle.getServiceAssembly()).andReturn(sa).anyTimes();
            expect(saLifeCycle.getServiceUnitsLifeCycles()).andReturn(sulfs);
            saLifeCycle.removePackage();
            expect(sa.getServiceUnits()).andReturn(sus).anyTimes();
            expect(sa.getConnections()).andReturn(connections);
            expect(su.getIdentification()).andReturn(identification);
            expect(su.getTargetComponentName()).andReturn(componentName);
            expect(identification.getName()).andReturn(suName);
            expect(connection.getConsumerInterfaceName()).andReturn(
                    consInterface);
            expect(connection.getProviderServiceName()).andReturn(provService);
            expect(connection.getProviderEndpointName())
                    .andReturn(provEndpoint);
            expect(connection.getConsumerServiceName()).andReturn(null);
            expect(connection.getConsumerEndpointName()).andReturn(null);
            epSrv.deleteConnection(null, null, provService, provEndpoint);
            expect(
                    sysState
                            .deleteServiceAssemblyStateHolder(serviceAssemblyName))
                    .andReturn(null);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(sa);
        replay(su);
        replay(lifeCycleManager);
        replay(saLifeCycle);
        replay(identification);
        replay(connection);
        replay(epSrv);
        replay(sysState);
        deploymentService.managerService = lifeCycleManager;
        deploymentService.endpointService = epSrv;
        deploymentService.recoverySrv = sysState;
        /*
         * 
         * Run Test Case 4
         */
        try {
            result = deploymentService.undeploy(serviceAssemblyName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        assertNotNull("deploy result must be non null", result);
        assertFalse("deploy result must be non empty", result.length() == 0);
        assertTrue("Wrong invokation result", result.contains("SUCCESS"));
        verify(sa);
        verify(su);
        verify(lifeCycleManager);
        verify(saLifeCycle);
        verify(identification);
        verify(connection);

        /*
         * Test Case 5 A component matching the service unit target component is
         * found but does not provides a service unit manager There is
         * connection for this service assembly which does not provide a
         * consumer interface
         */
        reset(sa);
        reset(su);
        reset(lifeCycleManager);
        reset(saLifeCycle);
        reset(identification);
        reset(connection);
        reset(epSrv);
        reset(sysState);
        /*
         * Create Test parameters
         */
        deploymentService.log = log;
        deploymentService.packagesLoadedCache = loadedPackages;
        consInterface = null;
        ComponentLifeCycle clc = createMock(ComponentLifeCycle.class);
        Component component = createMock(Component.class);
        /*
         * Initialize Mocks
         */
        try {
            expect(
                    lifeCycleManager
                            .getServiceAssemblyByName(serviceAssemblyName))
                    .andReturn(saLifeCycle);
            expect(lifeCycleManager.getComponentByName(componentName))
                    .andReturn(clc);
            expect(clc.getComponent()).andReturn(component);
            expect(saLifeCycle.getCurrentState()).andReturn(
                    LifeCycleMBean.STARTED);
            saLifeCycle.stop();
            saLifeCycle.shutDown();
            expect(saLifeCycle.getCurrentState()).andReturn(
                    LifeCycleMBean.SHUTDOWN);
            expect(saLifeCycle.getServiceAssembly()).andReturn(sa).anyTimes();
            expect(saLifeCycle.getServiceUnitsLifeCycles()).andReturn(sulfs);
            expect(sa.getServiceUnits()).andReturn(sus).anyTimes();
            expect(su.getIdentification()).andReturn(identification);
            expect(su.getTargetComponentName()).andReturn(componentName);
            expect(identification.getName()).andReturn(suName);
            expect(
                    sysState
                            .deleteServiceAssemblyStateHolder(serviceAssemblyName))
                    .andReturn(null);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(sa);
        replay(su);
        replay(lifeCycleManager);
        replay(saLifeCycle);
        replay(identification);
        replay(connection);
        replay(epSrv);
        replay(sysState);
        replay(clc);
        deploymentService.managerService = lifeCycleManager;
        deploymentService.endpointService = epSrv;
        deploymentService.recoverySrv = sysState;
        /*
         * 
         * Run Test Case 5
         */
        try {
            result = deploymentService.undeploy(serviceAssemblyName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        assertNotNull("deploy result must be non null", result);
        assertFalse("deploy result must be non empty", result.length() == 0);
        assertTrue("Wrong invokation result", result.contains("FAILED"));
        verify(sa);
        verify(su);
        verify(lifeCycleManager);
        verify(saLifeCycle);
        verify(identification);
        verify(connection);
        verify(clc);

        /*
         * Test Case 6 A component matching the service unit target component is
         * found and provides a service unit manager the result of undeploy is
         * not a valid message There is connection for this service assembly
         * which does not provide a consumer interface
         */
        reset(sa);
        reset(su);
        reset(lifeCycleManager);
        reset(saLifeCycle);
        reset(identification);
        reset(connection);
        reset(epSrv);
        reset(sysState);
        reset(clc);
        reset(component);
        /*
         * Create Test parameters
         */
        deploymentService.log = log;
        deploymentService.packagesLoadedCache = loadedPackages;
        consInterface = null;
        ServiceUnitManager sum = createMock(ServiceUnitManagerMock.class);
        String suRootPath = "Service unit root path";
        String xmlResult = "xmlresult";

        /*
         * Initialize Mocks
         */
        try {
            expect(
                    lifeCycleManager
                            .getServiceAssemblyByName(serviceAssemblyName))
                    .andReturn(saLifeCycle);
            expect(lifeCycleManager.getComponentByName(componentName))
                    .andReturn(clc);
            lifeCycleManager.unregisterServiceAssembly(serviceAssemblyName);
            expect(clc.getComponent()).andReturn(component);
            expect(saLifeCycle.getCurrentState()).andReturn(
                    LifeCycleMBean.STARTED);
            saLifeCycle.stop();
            saLifeCycle.shutDown();
            expect(saLifeCycle.getCurrentState()).andReturn(
                    LifeCycleMBean.SHUTDOWN);
            expect(saLifeCycle.getServiceAssembly()).andReturn(sa).anyTimes();
            expect(saLifeCycle.getServiceUnitsLifeCycles()).andReturn(sulfs)
                    .anyTimes();
            saLifeCycle.removePackage();
            expect(sa.getServiceUnits()).andReturn(sus).anyTimes();
            expect(sa.getConnections()).andReturn(connections);
            expect(su.getIdentification()).andReturn(identification);
            expect(su.getTargetComponentName()).andReturn(componentName)
                    .anyTimes();
            expect(identification.getName()).andReturn(suName);
            expect(
                    sysState
                            .deleteServiceAssemblyStateHolder(serviceAssemblyName))
                    .andReturn(null);
            expect(component.getServiceUnitManager()).andReturn(sum);
            expect(sum.undeploy(suName, suRootPath)).andReturn(xmlResult);
            expect(sulf.getServiceUnitRootPath()).andReturn(suRootPath);
            expect(
                    deploymentService.isValidComponentTask(isA(String.class),
                            isA(InputStream.class))).andReturn(false);
            expect(connection.getConsumerInterfaceName()).andReturn(
                    consInterface);
            expect(connection.getProviderServiceName()).andReturn(provService);
            expect(connection.getProviderEndpointName())
                    .andReturn(provEndpoint);
            expect(connection.getConsumerServiceName()).andReturn(null);
            expect(connection.getConsumerEndpointName()).andReturn(null);
            epSrv.deleteConnection(null, null, provService, provEndpoint);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(sa);
        replay(su);
        replay(lifeCycleManager);
        replay(saLifeCycle);
        replay(identification);
        replay(connection);
        replay(epSrv);
        replay(sysState);
        replay(clc);
        replay(sum);
        replay(sulf);
        replay(component);
        replay(deploymentService);
        deploymentService.managerService = lifeCycleManager;
        deploymentService.endpointService = epSrv;
        deploymentService.recoverySrv = sysState;
        /*
         * 
         * Run Test Case 6
         */
        try {
            result = deploymentService.undeploy(serviceAssemblyName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        assertNotNull("deploy result must be non null", result);
        assertFalse("deploy result must be non empty", result.length() == 0);
        assertTrue("Wrong invokation result", result.contains("SUCCESS"));
        verify(sa);
        verify(su);
        verify(lifeCycleManager);
        verify(saLifeCycle);
        verify(identification);
        verify(connection);
        verify(clc);
        verify(sum);
        verify(sulf);
        verify(component);
        verify(deploymentService);

        /*
         * Test Case 7 
         * A component matching the service unit target component is found and provides a service unit manager
         * the result of undeploy is a valid message
         * There is connection for this service assembly which does not provide a consumer interface
         */
        reset(sa);
        reset(su);
        reset(sulf);
        reset(lifeCycleManager);
        reset(saLifeCycle);
        reset(identification);
        reset(connection);
        reset(epSrv);
        reset(sysState);
        reset(clc);
        reset(component);
        reset(deploymentService);
        reset(sum);
        /*
         * Create Test parameters
         */
        deploymentService.log = log;
        deploymentService.packagesLoadedCache = loadedPackages;
        sulfs.put(suName, sulf);

        /*
         * Initialize Mocks
         */
        try {
            expect(
                    lifeCycleManager
                            .getServiceAssemblyByName(serviceAssemblyName))
                    .andReturn(saLifeCycle);
            expect(lifeCycleManager.getComponentByName(componentName))
                    .andReturn(clc);
            lifeCycleManager.unregisterServiceAssembly(serviceAssemblyName);
            expect(clc.getComponent()).andReturn(component);
            expect(saLifeCycle.getCurrentState()).andReturn(
                    LifeCycleMBean.STOPPED);
            saLifeCycle.shutDown();
            expect(saLifeCycle.getCurrentState()).andReturn(
                    LifeCycleMBean.SHUTDOWN);
            expect(saLifeCycle.getServiceAssembly()).andReturn(sa).anyTimes();
            expect(saLifeCycle.getServiceUnitsLifeCycles()).andReturn(sulfs)
                    .anyTimes();
            saLifeCycle.removePackage();
            expect(sa.getServiceUnits()).andReturn(sus).anyTimes();
            expect(sa.getConnections()).andReturn(connections);
            expect(su.getIdentification()).andReturn(identification);
            expect(su.getTargetComponentName()).andReturn(componentName)
                    .anyTimes();
            expect(identification.getName()).andReturn(suName);
            expect(
                    sysState
                            .deleteServiceAssemblyStateHolder(serviceAssemblyName))
                    .andReturn(null);
            expect(component.getServiceUnitManager()).andReturn(sum);
            expect(sum.undeploy(suName, suRootPath)).andReturn(xmlResult);
            expect(sulf.getServiceUnitRootPath()).andReturn(suRootPath);
            expect(
                    deploymentService.isValidComponentTask(isA(String.class),
                            isA(InputStream.class))).andReturn(false);
            expect(connection.getConsumerInterfaceName()).andReturn(
                    consInterface);
            expect(connection.getProviderServiceName()).andReturn(provService);
            expect(connection.getProviderEndpointName())
                    .andReturn(provEndpoint);
            expect(connection.getConsumerServiceName()).andReturn(null);
            expect(connection.getConsumerEndpointName()).andReturn(null);
            epSrv.deleteConnection(null, null, provService, provEndpoint);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(sa);
        replay(su);
        replay(lifeCycleManager);
        replay(saLifeCycle);
        replay(identification);
        replay(connection);
        replay(epSrv);
        replay(sysState);
        replay(clc);
        replay(sum);
        replay(sulf);
        replay(component);
        replay(deploymentService);
        deploymentService.managerService = lifeCycleManager;
        deploymentService.endpointService = epSrv;
        deploymentService.recoverySrv = sysState;
        /*
         * 
         * Run Test Case 7
         */
        try {
            result = deploymentService.undeploy(serviceAssemblyName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        assertNotNull("deploy result must be non null", result);
        assertFalse("deploy result must be non empty", result.length() == 0);
        assertTrue("Wrong invokation result", result.contains("SUCCESS"));
        verify(sa);
        verify(su);
        verify(lifeCycleManager);
        verify(saLifeCycle);
        verify(identification);
        verify(connection);
        verify(clc);
        verify(sum);
        verify(sulf);
        verify(component);
        verify(deploymentService);

    }

    @Override
    protected void setUp() {

    }
}
