/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: InstallationServiceTest.java 16:39:45 wjoseph $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.service;

import java.io.File;
import java.lang.reflect.Method;
import java.net.URI;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.zip.ZipFile;

import javax.management.AttributeNotFoundException;
import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.management.ReflectionException;

import junit.framework.TestCase;

import org.objectweb.petals.PetalsException;
import org.objectweb.petals.classloader.LoaderManager;
import org.objectweb.petals.classloader.LoaderManagerImpl;
import org.objectweb.petals.jbi.component.ComponentContextImpl;
import org.objectweb.petals.jbi.component.InstallationContextImpl;
import org.objectweb.petals.jbi.management.lifecycle.ComponentLifeCycle;
import org.objectweb.petals.jbi.management.lifecycle.Installer;
import org.objectweb.petals.jbi.management.lifecycle.LifeCycleMBean;
import org.objectweb.petals.jbi.management.systemstate.ComponentState;
import org.objectweb.petals.jbi.management.systemstate.SharedLibraryState;
import org.objectweb.petals.jbi.management.systemstate.SystemState;
import org.objectweb.petals.jbi.management.systemstate.SystemStateImpl;
import org.objectweb.petals.jbi.mock.MockLifeCycleManagerService;
import org.objectweb.petals.jbi.mock.MockZipFile;
import org.objectweb.petals.jbi.routing.Router;
import org.objectweb.petals.jbi.routing.RouterImpl;
import org.objectweb.petals.repository.RepositoryImpl;
import org.objectweb.petals.repository.RepositoryService;
import org.objectweb.petals.tools.jbicommon.descriptor.ComponentDescription;
import org.objectweb.petals.tools.jbicommon.descriptor.Identification;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptor;
import org.objectweb.petals.tools.jbicommon.descriptor.SharedLibrary;
import org.objectweb.petals.tools.jbicommon.descriptor.SharedLibraryList;
import org.objectweb.petals.util.InterfaceNamesHelper;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.wrapper.printwriter.LoggerImpl;

import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.isA;
import static org.easymock.classextension.EasyMock.createMock;
import static org.easymock.classextension.EasyMock.replay;
import static org.easymock.classextension.EasyMock.reset;
import static org.easymock.classextension.EasyMock.verify;

/**
 * Test of the InstallationService
 * 
 * @author wjoseph - eBMWebsourcing
 */
public class InstallationServiceTest extends TestCase {

    private File testDataDir = null;

    public void testBindFc() {
        /*
         * Test Case 1 The parameter clientItfName is null, an exception will be
         * raised
         */
        InstallationService installationService = new InstallationService();
        /*
         * Create Test parameters
         */
        String clientItfName = null;
        Object serverItf = null;
        /*
         * Run Test Case 1
         */
        try {
            installationService.bindFc(clientItfName, serverItf);
            fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }

        /*
         * Test Case 2 The parameter clientItfName match the logger interface
         * name The logger attribute of installationService will be set to
         * serverItf value
         */
        serverItf = createMock(LoggerImpl.class);
        clientItfName = InterfaceNamesHelper.LOGGER_NAME;
        /*
         * Run Test Case 2
         */
        try {
            installationService.bindFc(clientItfName, serverItf);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("logger attribute was not properly set",
                installationService.logger, serverItf);

        /*
         * Test Case 3 The parameter clientItfName match the
         * InterfaceNamesHelper.JBI_LIFECYCLEMANAGER_NAME The managerService
         * attribute of installationService will be set to serverItf value
         */
        serverItf = createMock(LifeCycleManagerImpl.class);
        clientItfName = InterfaceNamesHelper.JBI_LIFECYCLEMANAGER_NAME;
        /*
         * Run Test Case 3
         */
        try {
            installationService.bindFc(clientItfName, serverItf);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("manager attribute was not properly set",
                installationService.jmxAdmin, serverItf);

        /*
         * Test Case 4 The parameter clientItfName match the
         * InterfaceNamesHelper.PLATFORM_REPOSITORY_NAME The repositorySrv
         * attribute of installationService will be set to serverItf value
         */
        serverItf = createMock(RepositoryImpl.class);
        clientItfName = InterfaceNamesHelper.PLATFORM_REPOSITORY_NAME;
        /*
         * Run Test Case 4
         */
        try {
            installationService.bindFc(clientItfName, serverItf);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("manager attribute was not properly set",
                installationService.repositorySrv, serverItf);

        /*
         * Test Case 5 The parameter clientItfName match the
         * InterfaceNamesHelper.JBI_ADMIN_NAMEThe adminSrv attribute of
         * installationService will be set to serverItf value
         */
        serverItf = createMock(RouterImpl.class);
        clientItfName = InterfaceNamesHelper.JBI_ROUTER_NAME;
        /*
         * Run Test Case 5
         */
        try {
            installationService.bindFc(clientItfName, serverItf);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("manager attribute was not properly set",
                installationService.router, serverItf);

        /*
         * Test Case 6 The parameter clientItfName match the
         * InterfaceNamesHelper.JBI_SYSTEMSTATE_NAME The recoverySrv attribute
         * of installationService will be set to serverItf value
         */
        serverItf = createMock(SystemStateImpl.class);
        clientItfName = InterfaceNamesHelper.JBI_SYSTEMSTATE_NAME;
        /*
         * Run Test Case 6
         */
        try {
            installationService.bindFc(clientItfName, serverItf);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("manager attribute was not properly set",
                installationService.recoverySrv, serverItf);

        /*
         * Test Case 7 The parameter clientItfName match the
         * InterfaceNamesHelper.JBI_ENDPOINTSERVICE_NAME The recoverySrv
         * attribute of endpointService will be set to serverItf value
         */
        serverItf = createMock(EndpointServiceImpl.class);
        clientItfName = InterfaceNamesHelper.JBI_ENDPOINTSERVICE_NAME;
        /*
         * Run Test Case 7
         */
        try {
            installationService.bindFc(clientItfName, serverItf);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("manager attribute was not properly set",
                installationService.endpointService, serverItf);

        /*
         * Test Case 8 The parameter clientItfName match the
         * InterfaceNamesHelper.PLATFORM_CLASSLOADING_NAME The loaderSrv
         * attribute of endpointService will be set to serverItf value
         */
        serverItf = createMock(LoaderManagerImpl.class);
        clientItfName = InterfaceNamesHelper.PLATFORM_CLASSLOADING_NAME;
        /*
         * Run Test Case 8
         */
        try {
            installationService.bindFc(clientItfName, serverItf);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("manager attribute was not properly set",
                installationService.loaderSrv, serverItf);
    }

    public void testCheckClassPathElements() {
        /*
         * Test Case 1 An empty classpath element is found
         */
        /*
         * Create Mocks
         */
        Method method1 = null;
        try {
            method1 = InstallationService.class.getDeclaredMethod("hasEntry",
                    new Class[] {ZipFile.class, String.class});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mocked methods");
        }
        Method[] mockedMethods = new Method[] {method1};
        InstallationService installationService = createMock(
                InstallationService.class, mockedMethods);
        /*
         * Create Test parameters
         */
        List<String> classPathElements = new ArrayList<String>();
        classPathElements.add("");
        ZipFile archive = createMock(ZipFile.class);
        /*
         * Run Test Case 1
         */
        try {
            installationService.checkClassPathElements(archive,
                    classPathElements);
            fail("No exception raised");
        } catch (Exception e) {
            // Do nothing
        }

        /*
         * Test Case 2 An empty classpath element is found
         */
        /*
         * Create Test parameters
         */
        classPathElements = new ArrayList<String>();
        String element = "someElement";
        classPathElements.add(element);
        /*
         * Initialize mocks
         */
        expect(installationService.hasEntry(archive, element)).andReturn(false);
        replay(installationService);
        /*
         * Run Test Case 2
         */
        try {
            installationService.checkClassPathElements(archive,
                    classPathElements);
            fail("No exception raised");
        } catch (Exception e) {
            verify(installationService);
        }
    }

    public void testCheckComponent() {
        /*
         * Test Case 1 descriptor.getComponent() returns null
         */
        /*
         * Create Mocks
         */
        Method method1 = null;
        Method method2 = null;
        Method method3 = null;
        try {
            method1 = InstallationService.class.getDeclaredMethod("isLoaded",
                    new Class[] {String.class});
            method2 = InstallationService.class.getDeclaredMethod(
                    "checkClassPathElements", new Class[] {ZipFile.class,
                        List.class});
            method3 = InstallationService.class.getDeclaredMethod(
                    "openZipFile", new Class[] {URI.class});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mocked methods");
        }
        Method[] mockedMethods = new Method[] {method1, method2, method3};
        InstallationService installationService = createMock(
                InstallationService.class, mockedMethods);
        LoggingUtil log = createMock(LoggingUtil.class);
        installationService.log = log;
        /*
         * Create test parameters
         */
        JBIDescriptor descriptor = createMock(JBIDescriptor.class);
        URI archiveURI = new File(testDataDir, "fakearchive" + File.separator
                + "archive").toURI();
        boolean result = false;
        /*
         * Run Test Case 1
         */
        try {
            result = installationService.checkComponent(descriptor, archiveURI);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        assertFalse("Wrong invokation result", result);
        /*
         * Test Case 2 descriptor.getComponent() return the componentDescription
         * isLoaded returns true
         */
        reset(descriptor);
        /*
         * Create test parameters
         */
        ComponentDescription comp = createMock(ComponentDescription.class);
        Identification id = createMock(Identification.class);
        String componentName = "componentName";

        /*
         * Initialize Mocks
         */
        expect(descriptor.getComponent()).andReturn(comp);
        expect(comp.getIdentification()).andReturn(id);
        expect(id.getName()).andReturn(componentName);
        expect(installationService.isLoaded(componentName)).andReturn(true);
        replay(descriptor);
        replay(comp);
        replay(id);
        replay(installationService);
        /*
         * Run Test Case 2
         */
        try {
            result = installationService.checkComponent(descriptor, archiveURI);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        verify(descriptor);
        verify(installationService);
        verify(comp);
        verify(id);
        assertFalse("Wrong invokation result", result);

        /*
         * Test Case 3 descriptor.getComponent() return the componentDescription
         * isLoaded returns true archive.close() throw an exception
         */
        reset(descriptor);
        reset(installationService);
        reset(comp);
        reset(id);
        /*
         * Create test parameters
         */
        ZipFile archive = createMock(MockZipFile.class, new Method[] {});
        /*
         * Initialize Mocks
         */
        try {
            expect(descriptor.getComponent()).andReturn(comp);
            expect(comp.getIdentification()).andReturn(id);
            expect(comp.getBootstrapClassPath()).andReturn(null);
            expect(comp.getComponentClassPath()).andReturn(null);
            expect(id.getName()).andReturn(componentName);
            expect(installationService.isLoaded(componentName))
                    .andReturn(false);
            expect(installationService.openZipFile(archiveURI)).andReturn(
                    archive);
            installationService.checkClassPathElements(archive, null);
            installationService.checkClassPathElements(archive, null);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialise Mocks");
        }
        replay(descriptor);
        replay(comp);
        replay(id);
        replay(installationService);
        /*
         * Run Test Case 3
         */
        try {
            result = installationService.checkComponent(descriptor, archiveURI);
            fail("No exception was raised");
        } catch (Exception e) {
            verify(descriptor);
            verify(installationService);
            verify(comp);
            verify(id);
        }
    }

    public void testCheckComponentShutdownStateForSL() {
        /*
         * Test Case 1 One of the binding component unsing the shared library is
         * not shutdown
         */
        InstallationService installationService = new InstallationService();
        /*
         * Create Test parameters
         */
        LifeCycleManagerService lcms = createMock(LifeCycleManagerImpl.class);
        Map<ObjectName, ComponentLifeCycle> bindingComponentLifecycles = new HashMap<ObjectName, ComponentLifeCycle>();
        Map<ObjectName, ComponentLifeCycle> engineLifecycles = new HashMap<ObjectName, ComponentLifeCycle>();
        ObjectName componentName = null;
        ObjectName engineName = null;
        try {
            componentName = new ObjectName("component", "component",
                    "component");
            engineName = new ObjectName("engine", "engine", "engine");
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not create test parameters");
        }
        ComponentLifeCycle componentLifeCycle = createMock(ComponentLifeCycle.class);
        bindingComponentLifecycles.put(componentName, componentLifeCycle);
        engineLifecycles.put(engineName, componentLifeCycle);
        ComponentDescription componentDescription = createMock(ComponentDescription.class);
        SharedLibraryList sharedLibrayList = createMock(SharedLibraryList.class);
        List<SharedLibraryList> slList = new ArrayList<SharedLibraryList>();
        slList.add(sharedLibrayList);
        String slName = "sharedLibraryName";
        /*
         * Initialize Mocks
         */
        try {
            expect(lcms.getBindingCompoLifeCycles()).andReturn(
                    bindingComponentLifecycles);
            expect(lcms.getEngineCompoLifeCycles()).andReturn(engineLifecycles);
            expect(componentLifeCycle.getComponentDescription()).andReturn(
                    componentDescription);
            expect(componentLifeCycle.getCurrentState()).andReturn(
                    LifeCycleMBean.STOPPED);
            expect(componentDescription.getSharedLibraryList()).andReturn(
                    slList);
            expect(sharedLibrayList.getName()).andReturn(slName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(sharedLibrayList);
        replay(componentDescription);
        replay(componentLifeCycle);
        replay(lcms);
        installationService.jmxAdmin = lcms;
        boolean result = false;
        /*
         * Run Test Case 1
         */
        try {
            result = installationService
                    .checkComponentShutdownStateForSL(slName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        verify(sharedLibrayList);
        verify(componentDescription);
        verify(componentLifeCycle);
        verify(lcms);
        assertFalse("Wrong invokation result", result);

        /*
         * Test Case 2 One of the engine component unsing this shared library is
         * not shutdown
         */
        reset(sharedLibrayList);
        reset(componentDescription);
        reset(componentLifeCycle);
        reset(lcms);
        /*
         * Create Test parameters
         */
        /*
         * Initialize Mocks
         */
        try {
            expect(lcms.getBindingCompoLifeCycles()).andReturn(
                    bindingComponentLifecycles);
            expect(lcms.getEngineCompoLifeCycles()).andReturn(engineLifecycles);
            expect(componentLifeCycle.getComponentDescription()).andReturn(
                    componentDescription).anyTimes();
            expect(componentLifeCycle.getCurrentState()).andReturn(
                    LifeCycleMBean.SHUTDOWN);
            expect(componentLifeCycle.getCurrentState()).andReturn(
                    LifeCycleMBean.STOPPED);
            expect(componentDescription.getSharedLibraryList()).andReturn(
                    slList).anyTimes();
            expect(sharedLibrayList.getName()).andReturn(slName).anyTimes();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(sharedLibrayList);
        replay(componentDescription);
        replay(componentLifeCycle);
        replay(lcms);
        installationService.jmxAdmin = lcms;
        result = false;
        /*
         * Run Test Case 2
         */
        try {
            result = installationService
                    .checkComponentShutdownStateForSL(slName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        verify(sharedLibrayList);
        verify(componentDescription);
        verify(componentLifeCycle);
        verify(lcms);
        assertFalse("Wrong invokation result", result);
    }

    public void testGetComponentNameFromJBIDescriptor() {
        /*
         * Test Case 1
         */
        InstallationService installationService = new InstallationService();
        /*
         * Create test parameters
         */
        String componentName = "componentName";
        JBIDescriptor descriptor = createMock(JBIDescriptor.class);
        ComponentDescription component = createMock(ComponentDescription.class);
        Identification id = createMock(Identification.class);
        /*
         * Initialize Mocks
         */
        expect(descriptor.getComponent()).andReturn(component);
        expect(component.getIdentification()).andReturn(id);
        expect(id.getName()).andReturn(componentName);
        replay(descriptor);
        replay(id);
        replay(component);
        /*
         * Run Test Case 1
         */
        assertEquals("Wrong invokation result", componentName,
                installationService
                        .getComponentNameFromJBIDescriptor(descriptor));
        verify(descriptor);
        verify(id);
        verify(component);
    }

    public void testGetFcState() {
        /*
         * Test Case 1
         */
        InstallationService installationService = new InstallationService();

        /*
         * Run Test Case 1
         */
        String result = installationService.getFcState();
        assertNotNull("Result of getFcState must not be null", result);
        assertEquals("Wrong invokation result", result,
                org.objectweb.fractal.api.control.LifeCycleController.STOPPED);
    }

    public void testGetSharedLibraryName() {
        /*
         * Test Case 1
         */
        InstallationService installationService = new InstallationService();
        /*
         * Create test parameters
         */
        String sharedLibraryName = "sharedLibraryName";
        JBIDescriptor descriptor = createMock(JBIDescriptor.class);
        SharedLibrary sharedLibrary = createMock(SharedLibrary.class);
        Identification id = createMock(Identification.class);
        /*
         * Initialize Mocks
         */
        expect(descriptor.getSharedLibrary()).andReturn(sharedLibrary);
        expect(sharedLibrary.getIdentification()).andReturn(id);
        expect(id.getName()).andReturn(sharedLibraryName);
        replay(descriptor);
        replay(id);
        replay(sharedLibrary);
        /*
         * Run Test Case 1
         */
        assertEquals("Wrong invokation result", sharedLibraryName,
                installationService.getSharedLibraryName(descriptor));
        verify(descriptor);
        verify(id);
        verify(sharedLibrary);
    }

    public void testInit() {
        /*
         * Test Case 1
         */
        InstallationService installationService = new InstallationService();
        /*
         * Create test parameters
         */
        /*
         * Run Test Case 1
         */
        installationService.init();
        assertNotNull("Initialization incorectly done",
                installationService.packagesLoadedCache);
        assertNotNull("Initialization incorectly done", installationService.log);
        assertNotNull("Initialization incorectly done",
                installationService.packageHandler);
    }

    public void testInstallSharedLibrary() {
        /*
         * Test Case 1 Parameter librayURL is null, an IllegalArgumentException
         * is raised
         */
        /*
         * Create Mocks
         */
        Method method1 = null;
        Method method2 = null;
        Method method3 = null;
        Method method4 = null;
        Method method5 = null;
        Method method6 = null;
        Method method7 = null;
        Method method8 = null;
        try {
            method1 = InstallationService.class.getDeclaredMethod(
                    "checkSharedLibrary", new Class[] {JBIDescriptor.class,
                        URI.class});
            method2 = InstallationService.class.getDeclaredMethod(
                    "getSharedLibraryName", new Class[] {JBIDescriptor.class});
            method3 = InstallationService.class.getDeclaredMethod(
                    "installSharedLibrary", new Class[] {URI.class,
                        JBIDescriptor.class});
            method4 = InstallationService.class.getDeclaredMethod(
                    "getInstalledArchive", new Class[] {File.class});
            method5 = InstallationService.class.getDeclaredMethod(
                    "createSuccessFile", new Class[] {File.class});
            method6 = InstallationService.class.getDeclaredMethod(
                    "createFileFromURI", new Class[] {URI.class});
            method7 = InstallationService.class.getDeclaredMethod("copyFile",
                    new Class[] {File.class, File.class});
            method8 = InstallationService.class.getDeclaredMethod(
                    "getSharedLibraryName", new Class[] {
                        JBIDescriptor.class});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mocked methods");
        }
        Method[] mockedMethods = new Method[] {method1, method2, method3,
            method4, method5, method6, method7, method8};
        InstallationService installationService = createMock(
                InstallationService.class, mockedMethods);
        String librayURL = null;
        String result = null;
        LoggingUtil log = createMock(LoggingUtil.class);
        installationService.log = log;
        /*
         * Run Test Case 1
         */
        try {
            result = installationService.installSharedLibrary(librayURL);
            fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }

        /*
         * Test Case 2 Parameter librayURL is empty, an IllegalArgumentException
         * is raised
         */
        librayURL = "";
        /*
         * Run Test Case 2
         */
        try {
            result = installationService.installSharedLibrary(librayURL);
            fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }

        /*
         * Test Case 3 Parameter librayURL is not legal , an
         * IllegalArgumentException is raised
         */
        librayURL = "illegalURL";
        /*
         * Run Test Case 3
         */
        try {
            result = installationService.installSharedLibrary(librayURL);
            fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }

        /*
         * Test Case 4 Parameter librayURL is valid, checkSharedLibrary return
         * false
         */
        librayURL = "file://some/url";
        PackageHandler packageHandler = createMock(PackageHandler.class);
        URI archiveURI = null;
        /*
         * Initialize Mocks
         */
        try {
            expect(packageHandler.processAndGetPackageURI(librayURL, true))
                    .andReturn(archiveURI);
            expect(packageHandler.loadDescriptor(null)).andReturn(null);
            expect(installationService.checkSharedLibrary(null, null))
                    .andReturn(false);
            expect(installationService.getSharedLibraryName(null))
                    .andReturn(null);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(packageHandler);
        replay(installationService);
        installationService.packageHandler = packageHandler;
        /*
         * Run Test Case 4
         */
        try {
            result = installationService.installSharedLibrary(librayURL);
        } catch (Exception e) {
            fail("Error during invokation");
        }
        assertTrue("Wrong invokation result", result
                .contains(InstallationService.ERROR_INSTALL_SHARED_LIBRARY));
        verify(packageHandler);
        verify(installationService);

        /*
         * Test Case 5 Parameter librayURL is valid, checkSharedLibrary return
         * true
         */
        reset(packageHandler);
        reset(installationService);
        librayURL = "file://some/url";
        String libraryName = "libraryName";
        URI installationRoot = null;
        File installedArchive = new File("");
        JBIDescriptor descriptor = new JBIDescriptor();
        try {
            installationRoot = new URI("file://installation/root");
            archiveURI = new URI("file://archive");
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not create Test parameters");
        }
        SystemState recoverySystem = createMock(SystemStateImpl.class);
        /*
         * Initialize Mocks
         */
        try {
            expect(packageHandler.processAndGetPackageURI(librayURL, true))
                    .andReturn(archiveURI);
            expect(packageHandler.loadDescriptor(archiveURI)).andReturn(
                    descriptor);
            expect(
                    packageHandler.expandPackageIntoRepository(libraryName,
                            archiveURI)).andReturn(installationRoot);
            expect(
                    installationService.checkSharedLibrary(descriptor,
                            archiveURI)).andReturn(true);
            expect(installationService.getSharedLibraryName(descriptor))
                    .andReturn(libraryName);
            expect(
                    installationService.installSharedLibrary(installationRoot,
                            descriptor)).andReturn(libraryName);
            expect(installationService.createFileFromURI(archiveURI))
                    .andReturn(null);
            expect(installationService.getInstalledArchive(null)).andReturn(
                    installedArchive);
            installationService.copyFile(null, installedArchive);
            installationService.createSuccessFile(installedArchive);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(packageHandler);
        replay(installationService);
        installationService.packageHandler = packageHandler;
        installationService.recoverySrv = recoverySystem;
        /*
         * Run Test Case 5
         */
        try {
            result = installationService.installSharedLibrary(librayURL);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        assertNotNull("Result of installSharedLibray must not be null", result);
        assertFalse("Result of installSharedLibray must not be empty", result
                .length() == 0);
        assertTrue("Wrong invokation result", result.equals(libraryName));
        verify(packageHandler);
        verify(installationService);

    }

    public void testInstallSharedLibrary2() {
        /*
         * Test Case 1 The parameter installationRoot is null, an exception will
         * be raised
         */
        Method method1 = null;
        try {
            method1 = InstallationService.class.getDeclaredMethod(
                    "isParentFirst", new Class[] {SharedLibrary.class});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mocked methods");
        }
        Method[] mockedMethods = new Method[] {method1};
        InstallationService installationService = createMock(
                InstallationService.class, mockedMethods);
        LoggingUtil log = createMock(LoggingUtil.class);
        installationService.log = log;
        /*
         * Create Test parameters
         */
        URI installationRoot = null;
        JBIDescriptor descriptor = null;
        /*
         * Run Test Case 1
         */
        try {
            installationService.installSharedLibrary(installationRoot,
                    descriptor);
            fail("No exception raised");
        } catch (Exception e) {
            // Do nothing
        }

        /*
         * Test Case 2 The parameter installationRoot is null, an exception will
         * be raised
         */
        /*
         * Create Test parameters
         */
        try {
            installationRoot = new URI("file://installation/root");
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not create test parameters");
        }
        /*
         * Run Test Case 2
         */
        try {
            installationService.installSharedLibrary(installationRoot,
                    descriptor);
            fail("No exception raised");
        } catch (Exception e) {
            // Do nothing
        }

        /*
         * Test Case 3
         * 
         */
        /*
         * Create Test parameters
         */
        descriptor = createMock(JBIDescriptor.class);
        SharedLibrary sl = createMock(SharedLibrary.class);
        Identification id = createMock(Identification.class);
        String slName = "sharedLibraryName";
        LoaderManager loaderManager = createMock(LoaderManagerImpl.class);
        /*
         * Initialize Mocks
         */
        expect(descriptor.getSharedLibrary()).andReturn(sl);
        expect(installationService.isParentFirst(sl)).andReturn(true);
        expect(sl.getIdentification()).andReturn(id);
        expect(sl.getSharedLibraryClassPath()).andReturn(null);
        expect(id.getName()).andReturn(slName);
        replay(descriptor);
        replay(installationService);
        replay(id);
        replay(sl);
        installationService.loaderSrv = loaderManager;
        installationService.packagesLoadedCache = new ArrayList<String>();
        String result = null;
        /*
         * Run Test Case 3
         */
        try {
            result = installationService.installSharedLibrary(installationRoot,
                    descriptor);
        } catch (Exception e) {
            e.printStackTrace();
            fail("No exception raised");
        }
        assertEquals("Wrong invokation result", result, slName);
        verify(descriptor);
        verify(installationService);
        verify(id);
        verify(sl);

        /*
         * Test Case 4 The installationRoot can not be parse to an url
         */
        reset(descriptor);
        reset(installationService);
        reset(id);
        reset(sl);
        /*
         * Create Test parameters
         */
        try {
            installationRoot = new URI("error://installation/root");
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not create test parameters");
        }
        /*
         * Initialize Mocks
         */
        expect(descriptor.getSharedLibrary()).andReturn(sl);
        expect(installationService.isParentFirst(sl)).andReturn(true);
        replay(descriptor);
        replay(installationService);
        installationService.loaderSrv = loaderManager;
        installationService.packagesLoadedCache = new ArrayList<String>();
        result = null;
        /*
         * Run Test Case 4
         */
        try {
            result = installationService.installSharedLibrary(installationRoot,
                    descriptor);
        } catch (Exception e) {
            e.printStackTrace();
            fail("No exception raised");
        }
        assertNull("Wrong invokation result", result);
        verify(descriptor);
        verify(installationService);

    }

    public void testInstallationService() {
        /*
         * Test Case 1
         */
        InstallationService installationService = new InstallationService();
        assertNotNull(installationService);
    }

    public void testIsLoaded() {
        /*
         * Test Case 1
         */
        InstallationService installationService = new InstallationService();
        installationService.packagesLoadedCache = new ArrayList<String>();
        /*
         * Run Test Case 1
         */
        assertFalse(installationService.isLoaded("someId"));

    }

    public void testListFc() {
        /*
         * Test Case 1
         */
        InstallationService iInstallationService = new InstallationService();
        String[] result = null;
        /*
         * Run Test Case 1
         */
        result = iInstallationService.listFc();
        assertNotNull("The result of listFc must not be null", result);
        assertEquals("Wrong invokation result",
                InstallationService.CLIENT_INTERFACES, result);
    }

    public void testLoadInstaller() {
        /*
         * Test Case 1 The parameter componentName is null, an exception is
         * raised
         */
        InstallationService installationService = new InstallationService();
        String componentName = null;
        LoggingUtil log = createMock(LoggingUtil.class);
        installationService.log = log;
        /*
         * Run Test Case 1
         */
        try {
            installationService.loadInstaller(componentName);
            fail("No exception was thrown");
        } catch (Exception e) {
            // Do nothing
        }

        /*
         * Test Case 2 The parameter componentName is empty, an exception is
         * raised
         */
        componentName = "";
        /*
         * Run Test Case 2
         */
        try {
            installationService.loadInstaller(componentName);
            fail("No exception was thrown");
        } catch (Exception e) {
            // Do nothing
        }

        /*
         * Test Case 3 There is no installer for this component name
         */
        componentName = "componentName";
        LifeCycleManagerService lcms = createMock(LifeCycleManagerImpl.class);
        installationService.jmxAdmin = lcms;
        /*
         * Run Test Case 3
         */
        ObjectName result = installationService.loadInstaller(componentName);
        assertNull("Wrong invokation result", result);
    }

    public void testLoadNewInstaller() {
        /*
         * Test Case 1 The parameter installZipUrl is null, an exception is
         * raised
         */
        /*
         * Create Mocks
         */
        Method method1 = null;
        Method method2 = null;
        Method method3 = null;
        Method method4 = null;
        Method method5 = null;
        Method method6 = null;
        Method method7 = null;
        try {
            method1 = InstallationService.class.getDeclaredMethod(
                    "checkComponent", new Class[] {JBIDescriptor.class,
                        URI.class});
            method2 = InstallationService.class.getDeclaredMethod(
                    "checkInstalledSLForComp",
                    new Class[] {JBIDescriptor.class});
            method3 = InstallationService.class.getDeclaredMethod(
                    "getComponentNameFromJBIDescriptor",
                    new Class[] {JBIDescriptor.class});
            method4 = InstallationService.class.getDeclaredMethod(
                    "loadNewInstaller", new Class[] {URI.class,
                        JBIDescriptor.class});
            method5 = InstallationService.class.getDeclaredMethod(
                    "createFileFromURI", new Class[] {URI.class});
            method6 = InstallationService.class.getDeclaredMethod(
                    "getInstalledArchive", new Class[] {File.class});
            method7 = InstallationService.class.getDeclaredMethod("copyFile",
                    new Class[] {File.class, File.class});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mockedMethods");
        }
        Method[] mockedMethods = new Method[] {method1, method2, method3,
            method4, method5, method6, method7};
        InstallationService installationService = createMock(
                InstallationService.class, mockedMethods);
        String installZipUrl = null;
        LoggingUtil log = createMock(LoggingUtil.class);
        installationService.log = log;
        /*
         * Run Test Case 1
         */
        try {
            installationService.loadNewInstaller(installZipUrl);
            fail("No exception was thrown");
        } catch (Exception e) {
            // Do nothing
        }

        /*
         * Test Case 2 The parameter installZipUrl is empty, an exception is
         * raised
         */
        installZipUrl = "";
        /*
         * Run Test Case 2
         */
        try {
            installationService.loadNewInstaller(installZipUrl);
            fail("No exception was thrown");
        } catch (Exception e) {
            // Do nothing
        }

        /*
         * Test Case 3 The parameter installZipUrl is not a legal URL, an
         * exception is raised
         */
        installZipUrl = "illegalURL";
        /*
         * Run Test Case 3
         */
        try {
            installationService.loadNewInstaller(installZipUrl);
            fail("No exception was thrown");
        } catch (Exception e) {
            // Do nothing
        }

        /*
         * Test Case 4 The parameter installZipUrl is a valid URL,
         * checkComponent return false
         */
        installZipUrl = "file://zip/URL";
        PackageHandler packageHandler = createMock(PackageHandler.class);
        JBIDescriptor descriptor = createMock(JBIDescriptor.class);
        URI archiveURI = null;
        try {
            archiveURI = new URI("file://archive/uri");
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not create test parameters");
        }
        /*
         * Initialize Mocks
         */
        try {
            expect(packageHandler.processAndGetPackageURI(installZipUrl, true))
                    .andReturn(archiveURI);
            expect(packageHandler.loadDescriptor(archiveURI)).andReturn(
                    descriptor);
            expect(installationService.checkComponent(descriptor, archiveURI))
                    .andReturn(false);
            expect(installationService.getComponentNameFromJBIDescriptor(descriptor))
            .andReturn(null);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(installationService);
        replay(packageHandler);
        installationService.packageHandler = packageHandler;
        ObjectName result = null;
        /*
         * Run Test Case 4
         */
        try {
            result = installationService.loadNewInstaller(installZipUrl);
        } catch (Exception e) {
            fail("Error during invokation");
        }
        assertNotNull("Result of loadNewInstaller must not be null", result);
        assertEquals("Wrong invokation result ", result.getDomain(), "error");
        verify(installationService);
        verify(packageHandler);

        /*
         * Test Case 5 The parameter installZipUrl is a valid URL,
         * checkComponent return true checkInstalledSLForComp returns false
         */
        reset(installationService);
        reset(packageHandler);
        /*
         * Initialize Mocks
         */
        try {
            expect(packageHandler.processAndGetPackageURI(installZipUrl, true))
                    .andReturn(archiveURI);
            expect(packageHandler.loadDescriptor(archiveURI)).andReturn(
                    descriptor);
            expect(installationService.checkComponent(descriptor, archiveURI))
                    .andReturn(true);
            expect(installationService.checkInstalledSLForComp(descriptor))
                    .andReturn(false);
            expect(installationService.getComponentNameFromJBIDescriptor(descriptor))
            .andReturn(null);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(installationService);
        replay(packageHandler);
        installationService.packageHandler = packageHandler;
        /*
         * Run Test Case 5
         */
        try {
            result = installationService.loadNewInstaller(installZipUrl);
        } catch (Exception e) {
            fail("Error during invokation");
        }
        assertNotNull("Result of loadNewInstaller must not be null", result);
        assertEquals("Wrong invokation result ", result.getDomain(), "error");
        verify(installationService);
        verify(packageHandler);

        /*
         * Test Case 6 The parameter installZipUrl is a valid URL,
         * checkComponent return true checkInstalledSLForComp returns true
         */
        reset(installationService);
        reset(packageHandler);
        /*
         * Create test parameters
         */
        String componentName = "componentName";
        URI installRootURI = null;
        try {
            installRootURI = new URI("file://install/root");
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not create test parameters");
        }
        File file = new File("");
        File installedArchive = new File("");
        SystemState recoverySrv = createMock(SystemStateImpl.class);
        ObjectName invokationResult = createMock(ObjectName.class);
        /*
         * Initialize Mocks
         */
        try {
            expect(packageHandler.processAndGetPackageURI(installZipUrl, true))
                    .andReturn(archiveURI);
            expect(packageHandler.loadDescriptor(archiveURI)).andReturn(
                    descriptor);
            expect(
                    packageHandler.expandPackageIntoRepository(componentName,
                            archiveURI)).andReturn(installRootURI);
            expect(installationService.checkComponent(descriptor, archiveURI))
                    .andReturn(true);
            expect(installationService.checkInstalledSLForComp(descriptor))
                    .andReturn(true);
            expect(
                    installationService.loadNewInstaller(installRootURI,
                            descriptor)).andReturn(invokationResult);
            expect(
                    installationService
                            .getComponentNameFromJBIDescriptor(descriptor))
                    .andReturn(componentName);
            expect(installationService.createFileFromURI(archiveURI))
                    .andReturn(file);
            expect(installationService.getInstalledArchive(file)).andReturn(
                    installedArchive);
            installationService.copyFile(file, installedArchive);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(installationService);
        replay(packageHandler);
        installationService.packageHandler = packageHandler;
        installationService.recoverySrv = recoverySrv;
        /*
         * Run Test Case 6
         */
        try {
            result = installationService.loadNewInstaller(installZipUrl);
        } catch (Exception e) {
            fail("Error during invokation");
        }
        assertNotNull("Result of loadNewInstaller must not be null", result);
        assertEquals("Wrong invokation result ", result, invokationResult);
        verify(installationService);
        verify(packageHandler);
    }

    public void testCheckInstalledSLForComp() {
        /*
         * Test Case 1
         */
        InstallationService installationService = new InstallationService();
        /*
         * Create Test parameters
         */
        JBIDescriptor descriptor = createMock(JBIDescriptor.class);
        ComponentDescription componentDescription = createMock(ComponentDescription.class);
        SharedLibraryList sharedLibrayList = createMock(SharedLibraryList.class);
        List<SharedLibraryList> slList = new ArrayList<SharedLibraryList>();
        slList.add(sharedLibrayList);
        /*
         * Initialise Mocks
         */
        expect(descriptor.getComponent()).andReturn(componentDescription);
        expect(componentDescription.getSharedLibraryList()).andReturn(slList);
        replay(descriptor);
        replay(componentDescription);
        installationService.packagesLoadedCache = new ArrayList<String>();
        boolean result = false;
        /*
         * Run Test Case 1
         */
        result = installationService.checkInstalledSLForComp(descriptor);
        verify(descriptor);
        verify(componentDescription);
        assertFalse("Wrong invokation result", result);
    }

    public void testCheckSharedLibrary() {
        /*
         * Test Case 1 descriptor.getSharedLibrary() return null
         */
        /*
         * Create Mocks
         */
        Method method1 = null;
        Method method2 = null;
        Method method3 = null;
        try {
            method1 = InstallationService.class.getDeclaredMethod("isLoaded",
                    new Class[] {String.class});
            method2 = InstallationService.class.getDeclaredMethod(
                    "openZipFile", new Class[] {URI.class});
            method3 = InstallationService.class.getDeclaredMethod(
                    "checkClassPathElements", new Class[] {ZipFile.class,
                        List.class});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mocked methods");
        }
        Method[] mockedMethods = new Method[] {method1, method2, method3};
        InstallationService installationService = createMock(
                InstallationService.class, mockedMethods);
        LoggingUtil log = createMock(LoggingUtil.class);
        installationService.log = log;
        /*
         * Create Test parameters
         */
        JBIDescriptor descriptor = createMock(JBIDescriptor.class);
        URI archiveURI = new File(testDataDir, "fakearchive" + File.separator
                + "archive").toURI();
        SharedLibrary sharedLibray = createMock(SharedLibrary.class);
        boolean result = false;
        /*
         * Run Test Case 1
         */
        try {
            installationService.checkSharedLibrary(descriptor, archiveURI);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        assertFalse("Wrong invokation result", result);

        /*
         * Test Case 2 descriptor.getSharedLibrary() return an instance of
         * sharedLibrary isLoaded returns true, there is already a component
         * with this name on the platform
         */
        reset(descriptor);
        /*
         * Create Test parameters
         */
        Identification id = createMock(Identification.class);
        String slName = "sharedLibraryName";
        result = false;
        /*
         * Initialize Mocks
         */
        expect(descriptor.getSharedLibrary()).andReturn(sharedLibray);
        expect(sharedLibray.getIdentification()).andReturn(id);
        expect(id.getName()).andReturn(slName);
        expect(installationService.isLoaded(slName)).andReturn(true);
        replay(descriptor);
        replay(sharedLibray);
        replay(id);
        replay(installationService);
        /*
         * Run Test Case 2
         */
        try {
            installationService.checkSharedLibrary(descriptor, archiveURI);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        assertFalse("Wrong invokation result", result);
        verify(descriptor);
        verify(sharedLibray);
        verify(id);
        verify(installationService);

        /*
         * Test Case 3 descriptor.getSharedLibrary() return an instance of
         * sharedLibrary isLoaded returns false, there is no other component
         * with this name on the platform
         */
        reset(descriptor);
        reset(sharedLibray);
        reset(id);
        reset(installationService);
        /*
         * Create Test parameters
         */
        result = false;
        ZipFile zipFile = createMock(MockZipFile.class, new Method[] {});
        /*
         * Initialize Mocks
         */
        try {
            expect(descriptor.getSharedLibrary()).andReturn(sharedLibray);
            expect(sharedLibray.getIdentification()).andReturn(id);
            expect(sharedLibray.getSharedLibraryClassPath()).andReturn(null);
            expect(id.getName()).andReturn(slName);
            expect(installationService.isLoaded(slName)).andReturn(false);
            expect(installationService.openZipFile(archiveURI)).andReturn(
                    zipFile);
            installationService.checkClassPathElements(zipFile, null);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(descriptor);
        replay(sharedLibray);
        replay(id);
        replay(installationService);
        /*
         * Run Test Case 3
         */
        try {
            installationService.checkSharedLibrary(descriptor, archiveURI);
            fail("No exception was raised");
        } catch (Exception e) {
            verify(descriptor);
            verify(sharedLibray);
            verify(id);
            verify(installationService);
        }
    }

    public void testCreateAndRegisterInstallerMBean() {
        /*
         * Test Case 1
         */
        InstallationService installationService = new InstallationService();
        /*
         * Create Test parameters
         */
        LoggingUtil log = createMock(LoggingUtil.class);
        installationService.log = log;
        ComponentContextImpl componentCtx = createMock(ComponentContextImpl.class);
        InstallationContextImpl installationCtx = createMock(InstallationContextImpl.class);
        JBIDescriptor descriptor = createMock(JBIDescriptor.class);
        URI installationRoot = new File(testDataDir, "fakearchive"
                + File.separator + "archive").toURI();
        ComponentDescription componentDescription = createMock(ComponentDescription.class);
        ObjectName result = null;
        LifeCycleManagerService lcms = createMock(LifeCycleManagerImpl.class);
        MBeanNamesImpl beanNames = createMock(MBeanNamesImpl.class);
        Identification id = createMock(Identification.class);
        String componentName = "componentName";
        ObjectName objectName = createMock(ObjectName.class);
        /*
         * Initialize Mocks
         */
        try {
            expect(descriptor.getComponent()).andReturn(componentDescription);
            expect(lcms.getMBeanNames()).andReturn(beanNames);
            expect(lcms.registerInstaller(isA(Installer.class)))
                    .andReturn(null);
            expect(componentDescription.getIdentification()).andReturn(id);
            expect(id.getName()).andReturn(componentName);
            expect(beanNames.createInstallerMBeanName(componentName))
                    .andReturn(objectName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(lcms);
        replay(descriptor);
        replay(id);
        replay(beanNames);
        replay(componentDescription);
        installationService.jmxAdmin = lcms;
        /*
         * Run Test Case 1
         */
        result = installationService.createAndRegisterInstallerMBean(
                componentCtx, installationCtx, descriptor, installationRoot);
        verify(lcms);
        verify(descriptor);
        verify(id);
        verify(beanNames);
        verify(componentDescription);
        assertEquals("Wrong invokation result", result, objectName);
    }

    public void testCreateFileFromURI() {
        /*
         * Test Case 1
         */
        InstallationService installationService = new InstallationService();
        /*
         * Create Test parameters
         */
        URI fileURI = new File(testDataDir, "fakearchive" + File.separator
                + "archive").toURI();
        File result = null;
        /*
         * Run Test Case 1
         */
        result = installationService.createFileFromURI(fileURI);
        assertEquals("Wrong invokation result", result.toURI(), fileURI);
    }

    public void testLoadNewInstaller2() {
        /*
         * Test Case 1 The parameter installationURI is null, an exception is
         * raised
         */
        /*
         * Create Mocks
         */
        Method method1 = null;
        Method method2 = null;
        Method method3 = null;
        Method method4 = null;
        try {
            method1 = InstallationService.class.getDeclaredMethod(
                    "createComponentContext", new Class[] {JBIDescriptor.class,
                        URI.class});
            method2 = InstallationService.class.getDeclaredMethod(
                    "createInstallationContext", new Class[] {
                        ComponentContextImpl.class, JBIDescriptor.class});
            method3 = InstallationService.class.getDeclaredMethod(
                    "createAndRegisterInstallerMBean", new Class[] {
                        ComponentContextImpl.class,
                        InstallationContextImpl.class, JBIDescriptor.class,
                        URI.class});
            method4 = InstallationService.class.getDeclaredMethod(
                    "getComponentNameFromJBIDescriptor",
                    new Class[] {JBIDescriptor.class});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mockedMethods");
        }
        Method[] mockedMethods = new Method[] {method1, method2, method3,
            method4};
        InstallationService installationService = createMock(
                InstallationService.class, mockedMethods);
        LoggingUtil log = createMock(LoggingUtil.class);
        installationService.log = log;
        /*
         * Create Test parameters
         */
        URI installationRoot = null;
        JBIDescriptor descriptor = null;
        /*
         * Run Test Case 1
         */
        try {
            installationService.loadNewInstaller(installationRoot, descriptor);
            fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }

        /*
         * Test Case 2 The parameter descriptor is null, an exception is raised
         */
        /*
         * Create Test parameters
         */
        try {
            installationRoot = new URI("file://installation/root");
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not create test parameters");
        }
        /*
         * Run Test Case 2
         */
        try {
            installationService.loadNewInstaller(installationRoot, descriptor);
            fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }

        /*
         * Test Case 3
         * 
         */
        /*
         * Create Test parameters
         */
        descriptor = createMock(JBIDescriptor.class);
        ComponentContextImpl componentCtx = createMock(ComponentContextImpl.class);
        InstallationContextImpl installationCtx = createMock(InstallationContextImpl.class);
        ObjectName objectName = createMock(ObjectName.class);
        String componentName = "componentName";
        /*
         * Initialize Mocks
         */
        expect(
                installationService.createComponentContext(descriptor,
                        installationRoot)).andReturn(componentCtx);
        expect(
                installationService.createInstallationContext(componentCtx,
                        descriptor)).andReturn(installationCtx);
        expect(
                installationService.createAndRegisterInstallerMBean(
                        componentCtx, installationCtx, descriptor,
                        installationRoot)).andReturn(objectName);
        expect(
                installationService
                        .getComponentNameFromJBIDescriptor(descriptor))
                .andReturn(componentName);
        replay(installationService);
        installationService.packagesLoadedCache = new ArrayList<String>();
        ObjectName result = null;
        /*
         * Run Test Case 3
         */
        try {
            result = installationService.loadNewInstaller(installationRoot,
                    descriptor);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        verify(installationService);
        assertNotNull("Result of loadNewInstaller", result);
        assertEquals("Wrong invokation result", objectName, result);
    }

    public void testLookupFc() {
        /*
         * Test Case 1 The requested interface does not belong to adminService
         */
        InstallationService installationService = new InstallationService();
        /*
         * Create test parameters
         */
        String clientItf = "foo";
        /*
         * Run Test Case 1
         */
        try {
            installationService.lookupFc(clientItf);
            fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }

        /*
         * Test Case 2 The requested interface belongs to adminService
         */
        /*
         * Create test parameters
         */
        clientItf = InterfaceNamesHelper.JBI_LIFECYCLEMANAGER_NAME;
        Object result = null;
        LifeCycleManagerService lifeCycleManager = createMock(LifeCycleManagerImpl.class);
        installationService.jmxAdmin = lifeCycleManager;
        /*
         * Run Test Case 2
         */
        try {
            result = installationService.lookupFc(clientItf);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Exception during method invokation");
        }
        assertEquals("Wrong result for invokation", result, lifeCycleManager);

        /*
         * Test Case 3 The requested interface belongs to adminService
         */
        /*
         * Create test parameters
         */
        clientItf = InterfaceNamesHelper.PLATFORM_REPOSITORY_NAME;
        RepositoryService repoService = createMock(RepositoryImpl.class);
        installationService.repositorySrv = repoService;
        /*
         * Run Test Case 3
         */
        try {
            result = installationService.lookupFc(clientItf);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Exception during method invokation");
        }
        assertEquals("Wrong result for invokation", result, repoService);

        /*
         * Test Case 4 The requested interface belongs to adminService
         */
        /*
         * Create test parameters
         */
        clientItf = InterfaceNamesHelper.PLATFORM_CLASSLOADING_NAME;
        LoaderManager loaderSrv = createMock(LoaderManagerImpl.class);
        installationService.loaderSrv = loaderSrv;
        /*
         * Run Test Case 4
         */
        try {
            result = installationService.lookupFc(clientItf);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Exception during method invokation");
        }
        assertEquals("Wrong result for invokation", result, loaderSrv);

        /*
         * Test Case 5 The requested interface belongs to adminService
         */
        /*
         * Create test parameters
         */
        clientItf = InterfaceNamesHelper.JBI_SYSTEMSTATE_NAME;
        SystemState sysState = createMock(SystemStateImpl.class);
        installationService.recoverySrv = sysState;
        /*
         * Run Test Case 5
         */
        try {
            result = installationService.lookupFc(clientItf);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Exception during method invokation");
        }
        assertEquals("Wrong result for invokation", result, sysState);

        /*
         * Test Case 6 The requested interface belongs to adminService
         */
        /*
         * Create test parameters
         */
        clientItf = InterfaceNamesHelper.JBI_ENDPOINTSERVICE_NAME;
        EndpointService epService = createMock(EndpointServiceImpl.class);
        installationService.endpointService = epService;
        /*
         * Run Test Case 6
         */
        try {
            result = installationService.lookupFc(clientItf);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Exception during method invokation");
        }
        assertEquals("Wrong result for invokation", result, epService);

        /*
         * Test Case 7 The requested interface belongs to adminService
         */
        /*
         * Create test parameters
         */
        clientItf = InterfaceNamesHelper.JBI_ROUTER_NAME;
        Router router = createMock(RouterImpl.class);
        installationService.router = router;
        /*
         * Run Test Case 7
         */
        try {
            result = installationService.lookupFc(clientItf);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Exception during method invokation");
        }
        assertEquals("Wrong result for invokation", result, router);
    }

    public void testShutdown() {
        /*
         * Test Case 1 The requested interface does not belong to adminService
         */
        InstallationService installationService = new InstallationService();
        /*
         * Create test parameters
         */
        installationService.init();
        /*
         * Run Test Case 1
         */
        installationService.shutdown();
        assertNull("Shutdown process incomplete", installationService.logger);
        assertNull("Shutdown process incomplete",
                installationService.packagesLoadedCache);
    }

    public void testStartFc() {
        /*
         * Test Case 1
         */
        /*
         * Create Mocks
         */
        Method method1 = null;
        try {
            method1 = InstallationService.class.getDeclaredMethod("init",
                    new Class[] {});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mocked methods");
        }
        Method[] mockedMethods = new Method[] {method1};
        InstallationService installationService = createMock(
                InstallationService.class, mockedMethods);
        installationService.init();
        replay(installationService);
        /*
         * Run Test Case 1
         */
        try {
            installationService.startFc();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        assertEquals("InstallationService was not properly initialized",
                installationService.fcState,
                org.objectweb.fractal.api.control.LifeCycleController.STARTED);
        verify(installationService);
    }

    public void testStopFc() {
        /*
         * Test Case 1
         */
        /*
         * Create Mocks
         */
        Method method1 = null;
        try {
            method1 = InstallationService.class.getDeclaredMethod("shutdown",
                    new Class[] {});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mocked methods");
        }
        Method[] mockedMethods = new Method[] {method1};
        InstallationService installationService = createMock(
                InstallationService.class, mockedMethods);
        installationService.shutdown();
        replay(installationService);
        /*
         * Run Test Case 1
         */
        try {
            installationService.stopFc();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        assertEquals("InstallationService was not properly initialized",
                installationService.fcState,
                org.objectweb.fractal.api.control.LifeCycleController.STOPPED);
        verify(installationService);
    }

    public void testUnbindFc() {
        /*
         * Test Case 1 The parameter clientItfName is null, an exception will be
         * raised
         */
        InstallationService installationService = new InstallationService();
        /*
         * Create Test parameters
         */
        String clientItfName = null;
        Object serverItf = null;
        /*
         * Run Test Case 1
         */
        try {
            installationService.unbindFc(clientItfName);
            fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }

        /*
         * Test Case 2 The parameter clientItfName match the logger interface
         * name The logger attribute of installationService will be set to null
         */
        serverItf = createMock(LoggerImpl.class);
        installationService.logger = (Logger) serverItf;
        clientItfName = InterfaceNamesHelper.LOGGER_NAME;
        /*
         * Run Test Case 2
         */
        try {
            installationService.unbindFc(clientItfName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("logger attribute was not properly set",
                installationService.logger, null);

        /*
         * Test Case 3 The parameter clientItfName match the
         * InterfaceNamesHelper.JBI_LIFECYCLEMANAGER_NAME The managerService
         * attribute of installationService will be set to null
         */
        serverItf = createMock(LifeCycleManagerImpl.class);
        installationService.jmxAdmin = (LifeCycleManagerService) serverItf;
        clientItfName = InterfaceNamesHelper.JBI_LIFECYCLEMANAGER_NAME;
        /*
         * Run Test Case 3
         */
        try {
            installationService.unbindFc(clientItfName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("manager attribute was not properly set",
                installationService.jmxAdmin, null);

        /*
         * Test Case 4 The parameter clientItfName match the
         * InterfaceNamesHelper.PLATFORM_REPOSITORY_NAME The repositorySrv
         * attribute of installationService will be set to null
         */
        serverItf = createMock(RepositoryImpl.class);
        clientItfName = InterfaceNamesHelper.PLATFORM_REPOSITORY_NAME;
        installationService.repositorySrv = (RepositoryService) serverItf;
        /*
         * Run Test Case 4
         */
        try {
            installationService.unbindFc(clientItfName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("manager attribute was not properly set",
                installationService.repositorySrv, null);

        /*
         * Test Case 5 The parameter clientItfName match the
         * InterfaceNamesHelper.JBI_ROUTER_NAMEThe adminSrv attribute of
         * installationService will be set to null
         */
        serverItf = createMock(Router.class);
        clientItfName = InterfaceNamesHelper.JBI_ROUTER_NAME;
        installationService.router = (Router) serverItf;
        /*
         * Run Test Case 5
         */
        try {
            installationService.unbindFc(clientItfName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("manager attribute was not properly set",
                installationService.router, null);

        /*
         * Test Case 6 The parameter clientItfName match the
         * InterfaceNamesHelper.JBI_SYSTEMSTATE_NAME The recoverySrv attribute
         * of installationService will be set to null
         */
        serverItf = createMock(SystemStateImpl.class);
        clientItfName = InterfaceNamesHelper.JBI_SYSTEMSTATE_NAME;
        installationService.recoverySrv = (SystemState) serverItf;
        /*
         * Run Test Case 6
         */
        try {
            installationService.unbindFc(clientItfName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("manager attribute was not properly set",
                installationService.recoverySrv, null);

        /*
         * Test Case 7 The parameter clientItfName match the
         * InterfaceNamesHelper.JBI_ENDPOINTSERVICE_NAME The recoverySrv
         * attribute of endpointService will be set to serverItf value
         */
        serverItf = createMock(EndpointServiceImpl.class);
        clientItfName = InterfaceNamesHelper.JBI_ENDPOINTSERVICE_NAME;
        installationService.endpointService = (EndpointService) serverItf;
        /*
         * Run Test Case 7
         */
        try {
            installationService.unbindFc(clientItfName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("manager attribute was not properly set",
                installationService.endpointService, null);
        /*
         * Test Case 8 The parameter clientItfName match the
         * InterfaceNamesHelper.PLATFORM_CLASSLOADING_NAME The recoverySrv
         * attribute of endpointService will be set to serverItf value
         */
        serverItf = createMock(LoaderManagerImpl.class);
        clientItfName = InterfaceNamesHelper.PLATFORM_CLASSLOADING_NAME;
        installationService.loaderSrv = (LoaderManagerImpl) serverItf;
        /*
         * Run Test Case 8
         */
        try {
            installationService.unbindFc(clientItfName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("manager attribute was not properly set",
                installationService.loaderSrv, null);
    }

    public void testUninstallSharedLibrary() {
        /*
         * Test Case 1 The parameter slName is null, an exception is raised
         */
        /*
         * Create Mocks
         */
        Method method1 = null;
        Method method2 = null;
        Method method3 = null;
        Method method4 = null;
        try {
            method1 = InstallationService.class.getDeclaredMethod(
                    "checkComponentShutdownStateForSL",
                    new Class[] {String.class});
            method2 = InstallationService.class.getDeclaredMethod(
                    "createFileFromURI", new Class[] {URI.class});
            method3 = InstallationService.class.getDeclaredMethod(
                    "getUninstalledArchive", new Class[] {File.class});
            /*
             * method4 =
             * InstallationService.class.getDeclaredMethod("getComponentNameFromJBIDescriptor",new
             * Class[]{JBIDescriptor.class});
             */
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mockedMethods");
        }
        Method[] mockedMethods = new Method[] {method1, method2, method3,
            method4};
        InstallationService installationService = createMock(
                InstallationService.class, mockedMethods);
        LoggingUtil log = createMock(LoggingUtil.class);
        installationService.log = log;
        LoaderManager loaderSrv = createMock(LoaderManagerImpl.class);
        installationService.loaderSrv = loaderSrv;
        installationService.packagesLoadedCache = new ArrayList<String>();
        SystemState recoverySrv = createMock(SystemStateImpl.class);
        installationService.recoverySrv = recoverySrv;
        RepositoryService repositorySrv = createMock(RepositoryImpl.class);
        /*
         * Create Test parameters
         */
        String slName = null;
        /*
         * Run Test Case 1
         */
        try {
            installationService.uninstallSharedLibrary(slName);
            fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }

        /*
         * Test Case 2 The parameter slName is empty, an exception is raised
         */
        /*
         * Create Test parameters
         */
        slName = "";
        /*
         * Run Test Case 2
         */
        try {
            installationService.uninstallSharedLibrary(slName);
            fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }

        /*
         * Test Case 3
         */
        /*
         * Create Test parameters
         */
        slName = "sharedLibraryName";
        PackageHandler packageHandler = createMock(PackageHandler.class);
        SharedLibraryState slState = createMock(SharedLibraryState.class);
        String archiveZipURL = "file://achive.zip.url";
        URI archiveURI = null;
        try {
            archiveURI = new URI(archiveZipURL);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not create test parameters");
        }
        File installedArchive = new File(testDataDir, "fakearchive"
                + File.separator + "archive");
        File uninstalledArchive = installedArchive;
        /*
         * Initialize Mocks
         */
        try {
            expect(installationService.checkComponentShutdownStateForSL(slName))
                    .andReturn(true);
            expect(installationService.createFileFromURI(archiveURI))
                    .andReturn(installedArchive);
            expect(installationService.getUninstalledArchive(installedArchive))
                    .andReturn(uninstalledArchive);
            expect(repositorySrv.removePackage(slName)).andReturn(true);
            expect(recoverySrv.deleteSharedLibraryStateHolder(slName))
                    .andReturn(slState);
            expect(slState.getArchiveURL()).andReturn(archiveZipURL);
            expect(packageHandler.processAndGetPackageURI(archiveZipURL, false))
                    .andReturn(archiveURI);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(installationService);
        replay(repositorySrv);
        replay(slState);
        replay(packageHandler);
        replay(recoverySrv);
        installationService.repositorySrv = repositorySrv;
        installationService.packageHandler = packageHandler;
        installationService.recoverySrv = recoverySrv;
        boolean result = false;
        /*
         * Run Test Case 3
         */
        try {
            result = installationService.uninstallSharedLibrary(slName);
        } catch (Exception e) {
            fail("Error during invokation");
        }
        verify(installationService);
        verify(repositorySrv);
        verify(slState);
        verify(packageHandler);
        verify(recoverySrv);
        assertTrue("Wrong invokation result", result);

        /*
         * Test Case 4 processAndGetPackageURI throws an exception
         */
        /*
         * Create Test parameters
         */
        reset(installationService);
        reset(repositorySrv);
        reset(slState);
        reset(packageHandler);
        reset(recoverySrv);
        /*
         * Initialize Mocks
         */
        try {
            expect(installationService.checkComponentShutdownStateForSL(slName))
                    .andReturn(true);
            expect(repositorySrv.removePackage(slName)).andReturn(true);
            expect(recoverySrv.deleteSharedLibraryStateHolder(slName))
                    .andReturn(slState);
            expect(slState.getArchiveURL()).andReturn(archiveZipURL);
            expect(packageHandler.processAndGetPackageURI(archiveZipURL, false))
                    .andThrow(
                            new PetalsException(
                                    "processAndGetPackageURI testing exception"));
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(installationService);
        replay(repositorySrv);
        replay(slState);
        replay(packageHandler);
        replay(recoverySrv);
        installationService.repositorySrv = repositorySrv;
        installationService.packageHandler = packageHandler;
        installationService.recoverySrv = recoverySrv;
        result = false;
        /*
         * Run Test Case 4
         */
        try {
            result = installationService.uninstallSharedLibrary(slName);
        } catch (Exception e) {
            fail("Error during invokation");
        }
        verify(installationService);
        verify(repositorySrv);
        verify(slState);
        verify(packageHandler);
        assertFalse("Wrong invokation result", result);

        /*
         * Test Case 5 deleteSharedLibraryStateHolder throws an exception
         */
        /*
         * Create Test parameters
         */
        reset(installationService);
        reset(repositorySrv);
        reset(slState);
        reset(packageHandler);
        reset(recoverySrv);
        /*
         * Initialize Mocks
         */
        try {
            expect(installationService.checkComponentShutdownStateForSL(slName))
                    .andReturn(true);
            // expect(repositorySrv.removePackage(slName)).andReturn(true);
            expect(recoverySrv.deleteSharedLibraryStateHolder(slName))
                    .andThrow(
                            new Exception(
                                    "deleteSharedLibraryStateHolder testing exception"));
            // expect(slState.getArchiveURL()).andReturn(archiveZipURL);
            // expect(packageHandler.processAndGetPackageURI(archiveZipURL,
            // false)).andThrow(new PetalsException("processAndGetPackageURI
            // testing exception"));
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(installationService);
        // replay(repositorySrv);
        // replay(slState);
        // replay(packageHandler);
        replay(recoverySrv);
        installationService.repositorySrv = repositorySrv;
        installationService.packageHandler = packageHandler;
        installationService.recoverySrv = recoverySrv;
        result = false;
        /*
         * Run Test Case 5
         */
        try {
            result = installationService.uninstallSharedLibrary(slName);
        } catch (Exception e) {
            fail("Error during invokation");
        }
        verify(installationService);
        verify(recoverySrv);
        // verify(repositorySrv);
        // verify(slState);
        // verify(packageHandler);
        assertFalse("Wrong invokation result", result);
    }

    public void testUnloadInstaller() {
        /*
         * Test Case 1 The parameter componentName is null, an exception is
         * raised
         */
        /*
         * Create Mocks
         */
        Method method1 = null;
        Method method2 = null;
        Method method3 = null;
        Method method4 = null;
        Method method5 = null;
        try {
            method1 = MockLifeCycleManagerService.class.getDeclaredMethod(
                    "getInstallerByName", new Class[] {String.class});
            method2 = MockLifeCycleManagerService.class.getDeclaredMethod(
                    "getMBeanServer", new Class[] {});
            method3 = MockLifeCycleManagerService.class.getDeclaredMethod(
                    "unregisterInstaller", new Class[] {ObjectName.class});
            method4 = InstallationService.class.getDeclaredMethod(
                    "createFileFromURI", new Class[] {URI.class});
            method5 = InstallationService.class.getDeclaredMethod(
                    "getUninstalledArchive", new Class[] {File.class});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mockedMethods");
        }
        Method[] mockedMethods = new Method[] {method4, method5};
        InstallationService installationService = createMock(
                InstallationService.class, mockedMethods);
        mockedMethods = new Method[] {method1, method2};
        LoggingUtil log = createMock(LoggingUtil.class);
        installationService.log = log;
        /*
         * LoaderManager loaderSrv = createMock(LoaderManagerImpl.class);
         * installationService.loaderSrv = loaderSrv;
         * installationService.packagesLoadedCache = new ArrayList<String>();
         * SystemState recoverySrv = createMock(SystemStateImpl.class);
         * installationService.recoverySrv = recoverySrv; RepositoryService
         * repositorySrv = createMock(RepositoryImpl.class);
         */
        /*
         * Create Test parameters
         */
        String componentName = null;
        /*
         * Run Test Case 1
         */
        try {
            installationService.unloadInstaller(componentName, true);
            fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }

        /*
         * Test Case 2 The parameter componentName is empty, an exception is
         * raised
         */
        /*
         * Create Test parameters
         */
        componentName = "";
        /*
         * Run Test Case 2
         */
        try {
            installationService.unloadInstaller(componentName, true);
            fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }

        /*
         * Test Case 3 mbeanSrv.getAttribute throw an InstanceNotFoundException
         */
        /*
         * Create Test parameters
         */
        componentName = "ComponentName";
        LifeCycleManagerService managerSrv = createMock(
                MockLifeCycleManagerService.class, mockedMethods);
        ObjectName objectName = createMock(ObjectName.class);
        MBeanServer beanServer = createMock(MBeanServer.class);
        /*
         * Initialize Mocks
         */
        try {
            expect(managerSrv.getInstallerByName(componentName)).andReturn(
                    objectName);
            expect(managerSrv.getMBeanServer()).andReturn(beanServer);
            expect(beanServer.getAttribute(objectName, "Installed")).andThrow(
                    new InstanceNotFoundException(
                            "getAttribute testing exception"));
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(managerSrv);
        replay(beanServer);
        installationService.jmxAdmin = managerSrv;
        boolean result = false;
        /*
         * Run Test Case 3
         */
        try {
            result = installationService.unloadInstaller(componentName, true);
        } catch (Exception e) {
            fail("Error during invokation");
        }
        verify(managerSrv);
        verify(beanServer);
        assertFalse("Wrong invokation result", result);

        /*
         * Test Case 4 mbeanSrv.getAttribute throw an MBeanException
         */
        reset(managerSrv);
        reset(beanServer);
        /*
         * Create Test parameters
         */
        /*
         * Initialize Mocks
         */
        try {
            expect(managerSrv.getInstallerByName(componentName)).andReturn(
                    objectName);
            expect(managerSrv.getMBeanServer()).andReturn(beanServer);
            expect(beanServer.getAttribute(objectName, "Installed")).andThrow(
                    new MBeanException(new Exception(
                            "getAttribute testing exception")));
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(managerSrv);
        replay(beanServer);
        installationService.jmxAdmin = managerSrv;
        result = false;
        /*
         * Run Test Case 4
         */
        try {
            result = installationService.unloadInstaller(componentName, true);
        } catch (Exception e) {
            fail("Error during invokation");
        }
        verify(managerSrv);
        verify(beanServer);
        assertFalse("Wrong invokation result", result);

        /*
         * Test Case 5 mbeanSrv.getAttribute throw an ReflectionException
         */
        reset(managerSrv);
        reset(beanServer);
        /*
         * Create Test parameters
         */
        /*
         * Initialize Mocks
         */
        try {
            expect(managerSrv.getInstallerByName(componentName)).andReturn(
                    objectName);
            expect(managerSrv.getMBeanServer()).andReturn(beanServer);
            expect(beanServer.getAttribute(objectName, "Installed")).andThrow(
                    new ReflectionException(new Exception(
                            "getAttribute testing exception")));
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(managerSrv);
        replay(beanServer);
        installationService.jmxAdmin = managerSrv;
        result = false;
        /*
         * Run Test Case 5
         */
        try {
            result = installationService.unloadInstaller(componentName, true);
        } catch (Exception e) {
            fail("Error during invokation");
        }
        verify(managerSrv);
        verify(beanServer);
        assertFalse("Wrong invokation result", result);

        /*
         * Test Case 6 mbeanSrv.getAttribute throw an AttributeNotFoundException
         */
        reset(managerSrv);
        reset(beanServer);
        /*
         * Create Test parameters
         */
        /*
         * Initialize Mocks
         */
        try {
            expect(managerSrv.getInstallerByName(componentName)).andReturn(
                    objectName);
            expect(managerSrv.getMBeanServer()).andReturn(beanServer);
            expect(beanServer.getAttribute(objectName, "Installed")).andThrow(
                    new AttributeNotFoundException(
                            "getAttribute testing exception"));
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(managerSrv);
        replay(beanServer);
        installationService.jmxAdmin = managerSrv;
        result = false;
        /*
         * Run Test Case 6
         */
        try {
            result = installationService.unloadInstaller(componentName, true);
        } catch (Exception e) {
            fail("Error during invokation");
        }
        verify(managerSrv);
        verify(beanServer);
        assertFalse("Wrong invokation result", result);

        /*
         * Test Case 7 mbeanSrv.getAttribute return true
         * jmxAdmin.unregisterInstaller raise an exception
         */
        reset(managerSrv);
        reset(beanServer);
        /*
         * Create Test parameters
         */
        Boolean getAttribute = new Boolean(true);
        /*
         * Initialize Mocks
         */
        try {
            expect(managerSrv.getInstallerByName(componentName)).andReturn(
                    objectName);
            expect(managerSrv.getMBeanServer()).andReturn(beanServer);
            expect(beanServer.getAttribute(objectName, "Installed")).andReturn(
                    getAttribute);
            expect(
                    beanServer.invoke(isA(ObjectName.class), isA(String.class),
                            isA(Object[].class), isA(String[].class)))
                    .andReturn(null);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(managerSrv);
        replay(beanServer);
        installationService.jmxAdmin = managerSrv;
        result = false;
        /*
         * Run Test Case 7
         */
        try {
            result = installationService.unloadInstaller(componentName, true);
        } catch (Exception e) {
            fail("Error during invokation");
        }
        verify(managerSrv);
        verify(beanServer);
        assertFalse("Wrong invokation result", result);

        /*
         * Test Case 8 mbeanSrv.getAttribute return true
         * recoverySrv.deleteComponentStateHolder raise an exception
         */
        reset(managerSrv);
        reset(beanServer);
        /*
         * Create Test parameters
         */
        mockedMethods = new Method[] {method1, method2, method3};
        managerSrv = createMock(MockLifeCycleManagerService.class,
                mockedMethods);
        SystemState sysState = createMock(SystemStateImpl.class);
        ComponentState componentState = createMock(ComponentState.class);
        RepositoryService repositorySrv = createMock(RepositoryImpl.class);
        /*
         * Initialize Mocks
         */
        try {
            expect(managerSrv.getInstallerByName(componentName)).andReturn(
                    objectName);
            expect(managerSrv.getMBeanServer()).andReturn(beanServer);
            managerSrv.unregisterInstaller(objectName);
            expect(beanServer.getAttribute(objectName, "Installed")).andReturn(
                    getAttribute);
            expect(
                    beanServer.invoke(isA(ObjectName.class), isA(String.class),
                            isA(Object[].class), isA(String[].class)))
                    .andReturn(null);
            expect(sysState.deleteComponentStateHolder(componentName))
                    .andThrow(
                            new Exception(
                                    "deleteComponentStateHolder testing exception"));
            expect(repositorySrv.removePackage(componentName)).andReturn(true);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(managerSrv);
        replay(beanServer);
        replay(sysState);
        replay(repositorySrv);
        installationService.jmxAdmin = managerSrv;
        installationService.packagesLoadedCache = new ArrayList<String>();
        installationService.recoverySrv = sysState;
        installationService.repositorySrv = repositorySrv;
        result = false;
        /*
         * Run Test Case 8
         */
        try {
            result = installationService.unloadInstaller(componentName, true);
        } catch (Exception e) {
            fail("Error during invokation");
        }
        verify(managerSrv);
        verify(beanServer);
        verify(sysState);
        verify(repositorySrv);
        assertTrue("Wrong invokation result", result);

        /*
         * Test Case 9 mbeanSrv.getAttribute return true
         * recoverySrv.deleteComponentStateHolder raise an exception
         */
        reset(managerSrv);
        reset(beanServer);
        reset(repositorySrv);
        reset(sysState);
        /*
         * Create Test parameters
         */
        String archiveUrl = "archive/url";
        URI archiveURI = null;
        try {
            archiveURI = new URI("file://arvhice.uri");
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not create Test parameters");
        }
        File installedArchive = new File(testDataDir, "fakearchive"
                + File.separator + "archive");
        File uninstalledArchive = installedArchive;
        PackageHandler packageHandler = createMock(PackageHandler.class);
        /*
         * Initialize Mocks
         */
        try {
            expect(managerSrv.getInstallerByName(componentName)).andReturn(
                    objectName);
            expect(managerSrv.getMBeanServer()).andReturn(beanServer);
            managerSrv.unregisterInstaller(objectName);
            expect(beanServer.getAttribute(objectName, "Installed")).andReturn(
                    getAttribute);
            expect(
                    beanServer.invoke(isA(ObjectName.class), isA(String.class),
                            isA(Object[].class), isA(String[].class)))
                    .andReturn(null);
            expect(sysState.deleteComponentStateHolder(componentName))
                    .andReturn(componentState);
            expect(repositorySrv.removePackage(componentName)).andReturn(true);
            expect(componentState.getArchiveURL()).andReturn(archiveUrl);
            expect(installationService.createFileFromURI(archiveURI))
                    .andReturn(installedArchive);
            expect(installationService.getUninstalledArchive(installedArchive))
                    .andReturn(uninstalledArchive);
            expect(packageHandler.processAndGetPackageURI(archiveUrl, false))
                    .andReturn(archiveURI);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(managerSrv);
        replay(beanServer);
        replay(sysState);
        replay(repositorySrv);
        replay(componentState);
        replay(installationService);
        replay(packageHandler);
        installationService.jmxAdmin = managerSrv;
        installationService.packagesLoadedCache = new ArrayList<String>();
        installationService.recoverySrv = sysState;
        installationService.repositorySrv = repositorySrv;
        installationService.packageHandler = packageHandler;
        result = false;
        /*
         * Run Test Case 9
         */
        try {
            result = installationService.unloadInstaller(componentName, true);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        verify(managerSrv);
        verify(beanServer);
        verify(sysState);
        verify(componentState);
        verify(repositorySrv);
        verify(installationService);
        verify(packageHandler);
        assertTrue("Wrong invokation result", result);

        /*
         * Test Case 10 mbeanSrv.getAttribute return true
         * packageHandler.processAndGetPackageURI raise an exception
         */
        reset(managerSrv);
        reset(beanServer);
        reset(repositorySrv);
        reset(sysState);
        reset(componentState);
        reset(installationService);
        reset(packageHandler);
        /*
         * Create Test parameters
         */
        /*
         * Initialize Mocks
         */
        try {
            expect(managerSrv.getInstallerByName(componentName)).andReturn(
                    objectName);
            expect(managerSrv.getMBeanServer()).andReturn(beanServer);
            managerSrv.unregisterInstaller(objectName);
            expect(beanServer.getAttribute(objectName, "Installed")).andReturn(
                    getAttribute);
            expect(
                    beanServer.invoke(isA(ObjectName.class), isA(String.class),
                            isA(Object[].class), isA(String[].class)))
                    .andReturn(null);
            expect(sysState.deleteComponentStateHolder(componentName))
                    .andReturn(componentState);
            expect(repositorySrv.removePackage(componentName)).andReturn(true);
            expect(componentState.getArchiveURL()).andReturn(archiveUrl);
            expect(packageHandler.processAndGetPackageURI(archiveUrl, false))
                    .andThrow(
                            new PetalsException(
                                    "processAndGetPackageURI testing exception"));
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(managerSrv);
        replay(beanServer);
        replay(sysState);
        replay(repositorySrv);
        replay(componentState);
        replay(packageHandler);
        installationService.jmxAdmin = managerSrv;
        installationService.packagesLoadedCache = new ArrayList<String>();
        installationService.recoverySrv = sysState;
        installationService.repositorySrv = repositorySrv;
        installationService.packageHandler = packageHandler;
        result = false;
        /*
         * Run Test Case 10
         */
        try {
            result = installationService.unloadInstaller(componentName, true);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        verify(managerSrv);
        verify(beanServer);
        verify(sysState);
        verify(componentState);
        verify(repositorySrv);
        verify(packageHandler);
        assertFalse("Wrong invokation result", result);
    }

    @Override
    protected void setUp() {
        String baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);

        testDataDir = new File(baseDir + "src" + File.separator + "test-data");
    }
}
