package org.objectweb.petals.jbi.messaging;

import junit.framework.TestCase;

public class IncomingMessageQueueTest extends TestCase {

    IncomingMessageQueue queue = null;

    protected void setUp() throws Exception {
        super.setUp();
        queue = new IncomingMessageQueue(null);

    }

    public void testIncomingMessageQueue() {
        // check that the queue of msg is well initialized
        queue = new IncomingMessageQueue(null);
        assertNotNull(queue.messagesQueue);
    }

    /*
     * Test method for
     * 'org.objectweb.petals.jbi.messaging.IncomingMessageQueue.push(MessageExchangeImpl)'
     */
    public void testPush() {
        // init the queue with one msg
        queue.messagesQueue.add(new MessageExchangeImpl(null));

        // add a mesg
        MessageExchangeImpl msg = new MessageExchangeImpl(null);
        queue.push(msg);

        // check that there is 2msg, and the pushed msg is the last one
        assertTrue(queue.messagesQueue.size() == 2);
        assertEquals(msg, queue.messagesQueue.getLast());
    }

    /*
     * Test method for
     * 'org.objectweb.petals.jbi.messaging.IncomingMessageQueue.poll(long)'
     */
    public void testPoll() throws Exception {
        // 1.
        // test that timeout is ok when there is no msg
        assertNull(queue.poll(200L));

        // 2.
        // init the queue with 2 msg
        MessageExchangeImpl msg = new MessageExchangeImpl(null);
        queue.push(msg);
        queue.push(new MessageExchangeImpl(null));

        // check that the poll msg is the first, and there is 1msg after
        MessageExchangeImpl receivedmsg = queue.poll(10L);
        assertEquals(msg, receivedmsg);
        assertTrue(queue.messagesQueue.size() == 1);
    }

    /*
     * Test method for
     * 'org.objectweb.petals.jbi.messaging.IncomingMessageQueue.poll(long)'
     */
    public void testPollInterrupted() throws Exception {
        // check that an exception is thrown when wait is interrupted
        // wait on poll
        PollingThread t = new PollingThread(200);
        t.start();
        synchronized (this) {
            wait(10L);
        }
        // interrupt the wait on the queue
        queue.interrupt();
        synchronized (this) {
            wait(1L);
        }
        assertFalse(t.timeOutOccured);
        assertTrue(t.exception);
    }

    /*
     * 2msg in the queue and 2 poll-threads wait for mess each poll-thread must
     * owns a msg, and those ones must be differents. Test method for
     * 'org.objectweb.petals.jbi.messaging.IncomingMessageQueue.poll(long)'
     */
    public void testPollMultiThread() throws Exception {
        // init the queue with 2 msg
        queue.push(new MessageExchangeImpl(null));
        queue.push(new MessageExchangeImpl(null));

        PollingThread t1 = new PollingThread(0);
        PollingThread t2 = new PollingThread(0);
        t1.start();
        t2.start();
        synchronized (this) {
            wait(1L);
        }
        assertNotNull(t1.msg);
        assertNotNull(t2.msg);
        assertNotSame(t1.msg, t2.msg);
        assertEquals(queue.messagesQueue.size(), 0);
    }

    /*
     * 2 poll-threads are waiting then 1 msg is pushed 1 poll-thread owns ths
     * msg, the other owns nothing Test method for
     * 'org.objectweb.petals.jbi.messaging.IncomingMessageQueue.poll(long)'
     */
    public void testPollMultiThread2() throws Exception {

        receivedMsgCount = 0;

        PollingThread t1 = new PollingThread(100);
        PollingThread t2 = new PollingThread(100);
        t1.start();
        t2.start();

        queue.push(new MessageExchangeImpl(null));
        synchronized (this) {
            wait(1L);
        }
        assertEquals(receivedMsgCount, 1);
        assertEquals(queue.messagesQueue.size(), 0);
    }

    /*
     * test 3 poll-threads. one msg is pushed on the queue, then the queue is
     * interrupted finally at least one poll thread must own the msg, and the 2
     * others are in error Test method for
     * 'org.objectweb.petals.jbi.messaging.IncomingMessageQueue.poll(long)'
     */
    public void testPollMultithreadInterrupted() throws Exception {

        PollingThread t1 = new PollingThread(100);
        PollingThread t2 = new PollingThread(100);
        PollingThread t3 = new PollingThread(100);
        t1.start();
        t2.start();
        t3.start();

        receivedMsgCount = 0;
        errorPollThreadCount = 0;

        // add a msg
        queue.push(new MessageExchangeImpl(null));

        // interrupt the wait on the queue
        synchronized (this) {
            wait(10L);
        }
        queue.interrupt();
        synchronized (this) {
            wait(1L);
        }

        assertEquals(receivedMsgCount, 1);
        assertEquals(errorPollThreadCount, 2);
        assertEquals(queue.messagesQueue.size(), 0);

    }

    static int receivedMsgCount = 0;

    static int errorPollThreadCount = 0;

    /**
     * Simulation of a client which is waiting for a message on the
     * IncomingMsgQueue
     * 
     * @author alouis
     * 
     */
    public class PollingThread extends Thread {
        boolean exception = false;

        boolean timeOutOccured = false;

        MessageExchangeImpl msg = null;

        private long waitingtime = 0;

        /**
         * constructor
         * 
         * @param timeout
         */
        PollingThread(long timeout) {
            waitingtime = timeout;
        }

        public void run() {
            try {
                msg = queue.poll(waitingtime);
                timeOutOccured = true;
                receivedMsgCount++;
            } catch (Exception e) {
                exception = true;
                errorPollThreadCount++;
            }
        }
    }
}
