/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JoramQueueTest.java 12:34:54 PM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.transport;

import java.lang.reflect.Method;
import java.net.ConnectException;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.List;

import javax.jbi.JBIException;
import javax.jms.ConnectionFactory;
import javax.jms.JMSException;
import javax.jms.MessageListener;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueReceiver;
import javax.naming.InitialContext;
import javax.naming.NameClassPair;
import javax.naming.NamingException;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.joram.client.jms.Queue;
import org.objectweb.joram.client.jms.QueueConnection;
import org.objectweb.joram.client.jms.QueueSession;
import org.objectweb.joram.client.jms.admin.AdminException;
import org.objectweb.petals.jbi.registry.mock.MockNamingEnumeration;
import org.objectweb.petals.jbi.transport.mock.MockInitialContext;

/**
 * Tests of the JoramQueue
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class JoramQueueTest extends TestCase {

    protected MockInitialContext initialContext;

    protected JoramQueue joramQueue;

    public void setUp() throws NamingException {
        initialContext = new MockInitialContext();
        joramQueue = new JoramQueue(1, 16200, "petals", "petals", "127.0.0.1",
            initialContext);
    }

    public void testGetQueueConnectionFactory() {
        QueueConnectionFactory queueConnectionFactory = EasyMock
            .createMock(QueueConnectionFactory.class);
        joramQueue.qcf = queueConnectionFactory;
        assertEquals(queueConnectionFactory, joramQueue
            .getQueueConnectionFactory());
    }

    public void testInit() throws Exception {
        JoramQueue joramQueue = EasyMock.createMock(JoramQueue.class,
            new Method[] {
                JoramQueue.class.getDeclaredMethod("connectAdminModule",
                    new Class[] {}),
                JoramQueue.class.getDeclaredMethod("disconnectAdminModule",
                    new Class[] {}),
                JoramQueue.class.getDeclaredMethod(
                    "setupQueueConnectionFactory", new Class[] {}),
                JoramQueue.class.getDeclaredMethod("setupConnectionFactory",
                    new Class[] {}),
                JoramQueue.class
                    .getDeclaredMethod("setupQueue", new Class[] {})});
        QueueConnectionFactory queueConnectionFactory = EasyMock
            .createMock(QueueConnectionFactory.class);
        Queue queue = EasyMock.createMock(Queue.class);

        joramQueue.connectAdminModule();
        EasyMock.expect(joramQueue.setupConnectionFactory()).andReturn(
            EasyMock.createMock(ConnectionFactory.class)).anyTimes();
        EasyMock.expect(joramQueue.setupQueueConnectionFactory()).andReturn(
            queueConnectionFactory).anyTimes();
        EasyMock.expect(joramQueue.setupQueue()).andReturn(queue).anyTimes();
        joramQueue.disconnectAdminModule();

        EasyMock.replay(joramQueue);

        joramQueue.init();

        assertEquals(joramQueue.qcf, queueConnectionFactory);
        assertEquals(joramQueue.queue, queue);
    }

    public void testInitException() throws SecurityException,
        NoSuchMethodException, ConnectException, UnknownHostException,
        AdminException, NamingException {
        JoramQueue joramQueue = EasyMock.createMock(JoramQueue.class,
            new Method[] {
                JoramQueue.class.getDeclaredMethod("connectAdminModule",
                    new Class[] {}),
                JoramQueue.class.getDeclaredMethod("disconnectAdminModule",
                    new Class[] {}),
                JoramQueue.class.getDeclaredMethod(
                    "setupQueueConnectionFactory", new Class[] {}),
                JoramQueue.class.getDeclaredMethod("setupConnectionFactory",
                    new Class[] {}),
                JoramQueue.class
                    .getDeclaredMethod("setupQueue", new Class[] {})});

        joramQueue.connectAdminModule();
        EasyMock.expect(joramQueue.setupConnectionFactory()).andThrow(
            new NamingException(
                "problem while the setup of the Connection factory"))
            .anyTimes();

        EasyMock.replay(joramQueue);

        try {
            joramQueue.init();
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    @SuppressWarnings("unchecked")
    public void testSetupConnectionFactory() throws ConnectException,
        NamingException, SecurityException, NoSuchMethodException {
        InitialContext initialContext = EasyMock
            .createMock(InitialContext.class);
        JoramQueue joramQueue = EasyMock.createMock(JoramQueue.class,
            new Method[] {JoramQueue.class.getDeclaredMethod(
                "createConnectionFactory", new Class[] {})});
        MockNamingEnumeration<NameClassPair> list1 = new MockNamingEnumeration<NameClassPair>();
        ConnectionFactory connectionFactory = EasyMock
            .createMock(ConnectionFactory.class);

        EasyMock.expect(initialContext.list("/")).andReturn(list1).anyTimes();
        EasyMock.expect(joramQueue.createConnectionFactory()).andReturn(
            connectionFactory).anyTimes();
        initialContext.bind("JoramConnectionFactory", connectionFactory);

        EasyMock.replay(initialContext);
        EasyMock.replay(joramQueue);

        joramQueue.ictx = initialContext;

        assertEquals(joramQueue.setupConnectionFactory(), connectionFactory);
    }

    @SuppressWarnings("unchecked")
    public void testSetupConnectionFactoryBound() throws ConnectException,
        NamingException, SecurityException, NoSuchMethodException {
        InitialContext initialContext = EasyMock
            .createMock(InitialContext.class);
        JoramQueue joramQueue = EasyMock.createMock(JoramQueue.class,
            new Method[] {JoramQueue.class.getDeclaredMethod(
                "createConnectionFactory", new Class[] {})});
        MockNamingEnumeration<NameClassPair> list1 = new MockNamingEnumeration<NameClassPair>();
        List<NameClassPair> names = new ArrayList<NameClassPair>();
        names.add(new NameClassPair("JoramConnectionFactory",
            ConnectionFactory.class.getName()));
        list1.setElems(names);

        ConnectionFactory connectionFactory = EasyMock
            .createMock(ConnectionFactory.class);

        EasyMock.expect(initialContext.list("/")).andReturn(list1).anyTimes();
        EasyMock.expect(joramQueue.createConnectionFactory()).andReturn(
            connectionFactory).anyTimes();
        EasyMock.expect(initialContext.lookup(JoramQueue.CONNECTION_FACTORY))
            .andReturn(connectionFactory).anyTimes();

        EasyMock.replay(initialContext);
        EasyMock.replay(joramQueue);

        joramQueue.ictx = initialContext;

        assertEquals(joramQueue.setupConnectionFactory(), connectionFactory);
    }

    @SuppressWarnings( {"unchecked", "static-access"})
    public void testSetupQueue() throws Exception {
        InitialContext initialContext = EasyMock
            .createMock(InitialContext.class);
        MockNamingEnumeration<NameClassPair> list1 = new MockNamingEnumeration<NameClassPair>();
        Queue queue = EasyMock.createMock(Queue.class);
        JoramQueue.class.getDeclaredMethods();

        JoramQueue joramQueue = EasyMock.createMock(JoramQueue.class,
            new Method[] {JoramQueue.class.getDeclaredMethod("createQueue",
                new Class[] {String.class})});

        EasyMock.expect(initialContext.list("/")).andReturn(list1).anyTimes();
        EasyMock.expect(joramQueue.createQueue("JoramQueue-0"))
            .andReturn(queue).anyTimes();

        initialContext.bind("JoramQueue-0", queue);
        queue.setFreeReading();
        queue.setFreeWriting();

        EasyMock.replay(initialContext);
        EasyMock.replay(queue);
        EasyMock.replay(joramQueue);

        joramQueue.id = 0;
        joramQueue.ictx = initialContext;

        assertEquals(joramQueue.setupQueue(), queue);
    }

    @SuppressWarnings( {"unchecked", "static-access"})
    public void testSetupQueueBound() throws Exception {
        InitialContext initialContext = EasyMock
            .createMock(InitialContext.class);
        MockNamingEnumeration<NameClassPair> list1 = new MockNamingEnumeration<NameClassPair>();
        Queue queue = EasyMock.createMock(Queue.class);
        JoramQueue.class.getDeclaredMethods();
        List<NameClassPair> names = new ArrayList<NameClassPair>();
        names.add(new NameClassPair("JoramQueue-0", ConnectionFactory.class
            .getName()));
        list1.setElems(names);

        JoramQueue joramQueue = EasyMock.createMock(JoramQueue.class,
            new Method[] {JoramQueue.class.getDeclaredMethod("createQueue",
                new Class[] {String.class})});

        EasyMock.expect(initialContext.list("/")).andReturn(list1).anyTimes();
        EasyMock.expect(joramQueue.createQueue("JoramQueue-0"))
            .andReturn(queue).anyTimes();
        EasyMock.expect(initialContext.lookup("JoramQueue-0")).andReturn(queue)
            .anyTimes();

        initialContext.bind("JoramQueue-0", queue);
        queue.setFreeReading();
        queue.setFreeWriting();

        EasyMock.replay(initialContext);
        EasyMock.replay(queue);
        EasyMock.replay(joramQueue);

        joramQueue.id = 0;
        joramQueue.ictx = initialContext;

        assertEquals(joramQueue.setupQueue(), queue);
    }

    @SuppressWarnings( {"unchecked", "static-access"})
    public void testSetupQueueConnectionFactoryBound() throws Exception {
        InitialContext initialContext = EasyMock
            .createMock(InitialContext.class);
        MockNamingEnumeration<NameClassPair> list1 = new MockNamingEnumeration<NameClassPair>();
        QueueConnectionFactory queueConnectionFactory = EasyMock
            .createMock(QueueConnectionFactory.class);
        List<NameClassPair> names = new ArrayList<NameClassPair>();
        names.add(new NameClassPair(JoramQueue.QUEUE_CONNECTION_FACTORY,
            ConnectionFactory.class.getName()));
        list1.setElems(names);

        JoramQueue joramQueue = EasyMock.createMock(JoramQueue.class,
            new Method[] {JoramQueue.class.getDeclaredMethod(
                "createQueueConnectionFactory", new Class[0])});

        EasyMock.expect(initialContext.list("/")).andReturn(list1).anyTimes();
        EasyMock.expect(
            initialContext.lookup(JoramQueue.QUEUE_CONNECTION_FACTORY))
            .andReturn(queueConnectionFactory).anyTimes();

        EasyMock.replay(initialContext);
        EasyMock.replay(joramQueue);
        EasyMock.replay(queueConnectionFactory);

        joramQueue.id = 0;
        joramQueue.ictx = initialContext;

        assertEquals(joramQueue.setupQueueConnectionFactory(),
            queueConnectionFactory);
    }

    @SuppressWarnings( {"unchecked", "static-access"})
    public void testSetupQueueConnectionFactory() throws Exception {
        InitialContext initialContext = EasyMock
            .createMock(InitialContext.class);
        MockNamingEnumeration<NameClassPair> list1 = new MockNamingEnumeration<NameClassPair>();
        QueueConnectionFactory queueConnectionFactory = EasyMock
            .createMock(QueueConnectionFactory.class);
        JoramQueue.class.getDeclaredMethods();

        JoramQueue joramQueue = EasyMock.createMock(JoramQueue.class,
            new Method[] {JoramQueue.class.getDeclaredMethod(
                "createQueueConnectionFactory", new Class[0])});

        EasyMock.expect(initialContext.list("/")).andReturn(list1).anyTimes();
        EasyMock.expect(joramQueue.createQueueConnectionFactory()).andReturn(
            queueConnectionFactory).anyTimes();
        
        initialContext.bind(JoramQueue.QUEUE_CONNECTION_FACTORY,
            queueConnectionFactory);

        EasyMock.replay(initialContext);
        EasyMock.replay(joramQueue);
        EasyMock.replay(queueConnectionFactory);

        joramQueue.id = 0;
        joramQueue.ictx = initialContext;

        assertEquals(joramQueue.setupQueueConnectionFactory(),
            queueConnectionFactory);
    }

    public void testStartListening() throws JMSException {
        MessageListener messageListener = EasyMock
            .createMock(MessageListener.class);
        QueueConnectionFactory queueConnectionFactory = EasyMock
            .createMock(QueueConnectionFactory.class);
        QueueConnection queueConnection = EasyMock
            .createMock(QueueConnection.class);
        QueueSession queueSession = EasyMock.createMock(QueueSession.class);
        QueueReceiver queueReceiver = EasyMock.createMock(QueueReceiver.class);

        EasyMock.expect(
            queueConnectionFactory.createQueueConnection("petals", "petals"))
            .andReturn(queueConnection).anyTimes();
        EasyMock.expect(queueConnection.createQueueSession(true, 0)).andReturn(
            queueSession).anyTimes();
        EasyMock.expect(queueSession.createReceiver(null)).andReturn(
            queueReceiver).anyTimes();
        queueReceiver.setMessageListener(messageListener);
        queueConnection.start();

        EasyMock.replay(messageListener);
        EasyMock.replay(queueConnectionFactory);
        EasyMock.replay(queueConnection);
        EasyMock.replay(queueSession);
        EasyMock.replay(queueReceiver);

        joramQueue.qcf = queueConnectionFactory;

        joramQueue.startListening(messageListener);
    }

    public void testStopListening() throws JMSException {
        QueueSession queueSession = EasyMock.createMock(QueueSession.class);

        queueSession.commit();
        queueSession.close();

        EasyMock.replay(queueSession);

        joramQueue.qs = queueSession;

        joramQueue.stopListening();
    }

    public void testUnbindQueue() throws JBIException {
        joramQueue.unbindQueue();
        assertTrue(initialContext.isUnbind());
    }

    public void testUnbindQueueException() {
        initialContext.setThrowsException(true);
        try {
            joramQueue.unbindQueue();
            fail();
        } catch (JBIException e) {
            // do nothing
        }
    }
}
