/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JNDIServerTest.java 9:42:06 AM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.kernel.registry;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.petals.kernel.registry.mock.MockConfigurationService;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.petals.util.SystemUtil;
import org.objectweb.util.monolog.api.Logger;

/**
 * Test of the JNDIServer
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class NewMasterElectionTwoServerTest extends TestCase {

    private RegistryServer server1;

    private RegistryServer server2;

    public void testNewMasterElection() {
        try {
            String groupName = "petals"
                + System.nanoTime();

            SystemUtil.setJndiPort("16400");
            server1 = new RegistryServer();
            server1.configurationService = new MockConfigurationService();
            Logger logger = EasyMock.createMock(Logger.class);
            server1.logger = logger;
            server1.log = EasyMock.createMock(LoggingUtil.class);
            server1.setGroupName(groupName);

            server2 = new RegistryServer();
            server2.configurationService = new MockConfigurationService();
            server2.logger = logger;
            server2.log = EasyMock.createMock(LoggingUtil.class);
            server2.setGroupName(groupName);

            server1.startServer();

            // Wait for the server 1 to start
            //Thread.sleep(2000);

            SystemUtil.setJndiPort("16401");
            server2.startServer();

            // Wait for the server 2 the have its master set
            Thread.sleep(2000);

            assertTrue(server2.isMasterSet);
            assertEquals(server2.master, server1.startTime);

            server1.stopServer();

            // Wait for the election of a new master
            Thread.sleep(2000);

            assertEquals(server2.master, 0);

            server2.stopServer();

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}
