/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: LauncherTest.java 11:45:12 AM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.system.launch;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.TreeSet;

import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanServer;
import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.ReflectionException;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXServiceURL;
import javax.naming.Binding;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.LinkRef;
import javax.naming.NameClassPair;
import javax.naming.NamingException;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.AttributeController;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.api.control.NameController;
import org.objectweb.fractal.jmx.agent.Admin;
import org.objectweb.fractal.jmx.agent.AdminAttributes;
import org.objectweb.petals.jbi.management.systemstate.SystemState;
import org.objectweb.petals.kernel.registry.jndi.NamingEnumerationImpl;
import org.objectweb.petals.util.SystemUtil;
import org.objectweb.util.monolog.Monolog;

/**
 * Test of the launcher
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class LauncherTest extends TestCase {

    public void testBrowseJNDI() throws SecurityException,
        NoSuchMethodException, NamingException {
        String[] args = new String[] {"-host", "127.0.0.1", "-port", "16400"};
        Properties properties = new Properties();
        properties.setProperty("java.naming.factory.initial",
            "org.objectweb.petals.kernel.registry.jndi.NamingContextFactory");
        properties.setProperty("java.naming.factory.port", "16400");
        properties.setProperty("java.naming.factory.host", "127.0.0.1");

        NamingEnumerationImpl<Binding> namingEnumerationImpl = new NamingEnumerationImpl<Binding>(
            new Binding[0]);
        Launcher launcher = EasyMock.createMock(Launcher.class,
            new Method[] {Launcher.class.getDeclaredMethod("retrieveContext",
                new Class[] {Properties.class})});
        InitialContext initialContext = EasyMock
            .createMock(InitialContext.class);

        EasyMock.expect(launcher.retrieveContext(properties)).andReturn(
            initialContext).anyTimes();
        EasyMock.expect(initialContext.listBindings("/")).andReturn(
            namingEnumerationImpl).anyTimes();

        EasyMock.replay(launcher);
        EasyMock.replay(initialContext);

        launcher.browseJNDI(args);
    }

    public void testBrowseJNDIException() throws SecurityException,
        NoSuchMethodException, NamingException {
        String[] args = new String[] {"-host", "127.0.0.1", "-port", "16400",
            "-factory", "test"};
        Properties properties = new Properties();
        properties.setProperty("java.naming.factory.initial", "test");
        properties.setProperty("java.naming.factory.port", "16400");
        properties.setProperty("java.naming.factory.host", "127.0.0.1");

        Launcher launcher = EasyMock.createMock(Launcher.class,
            new Method[] {Launcher.class.getDeclaredMethod("retrieveContext",
                new Class[] {Properties.class})});

        EasyMock.expect(launcher.retrieveContext(properties)).andThrow(
            new NamingException("test error"));

        EasyMock.replay(launcher);

        launcher.browseJNDI(args);
    }

    public void testInitializeKernel() throws Exception {
        Launcher launcher = new Launcher();
        launcher.initializeFractal();
        assertNotNull(launcher.petalsComposite);
    }

    public void testInitializeMonolog() throws IOException {
        Launcher launcher = new Launcher();
        String classDir = this.getClass().getResource(".").toString();
        String targetDir = classDir.substring(0, classDir
            .indexOf("test-classes"));
        targetDir = targetDir.substring(targetDir.indexOf(":") + 1);
        File tmpPH = new File(targetDir);
        new File(targetDir + File.separator + "logs").mkdir();
        SystemUtil.setPetalsInstallDirectory(tmpPH);
        launcher.initializeMonolog();
        assertEquals(Monolog.monologFactory.getClass().getName(),
            "org.objectweb.petals.util.monolog.wrapper.javalog.LoggerFactory");
    }

    public void testStop() throws SecurityException, NoSuchMethodException,
        IOException, MalformedObjectNameException, NullPointerException,
        InstanceNotFoundException, MBeanException, ReflectionException {
        Launcher launcher = EasyMock.createMock(Launcher.class, new Method[] {
            Launcher.class.getDeclaredMethod("retrieveJMXConnector",
                new Class[] {JMXServiceURL.class, Map.class}),
            Launcher.class.getDeclaredMethod("invokeStopFc", new Class[] {
                MBeanServerConnection.class, ObjectName.class})});
        String urlStr = "service:jmx:rmi:///jndi/rmi://127.0.0.1:16400/management/rmi-jmx-connector";
        JMXServiceURL url = new JMXServiceURL(urlStr);
        Map<String, Object> args = new HashMap<String, Object>();
        JMXConnector jmxConnector = EasyMock.createMock(JMXConnector.class);
        MBeanServerConnection mBeanServerConnection = EasyMock
            .createMock(MBeanServerConnection.class);
        Set<ObjectName> objectNames = new TreeSet<ObjectName>();
        objectNames
            .add(new ObjectName("FC/Petals@01:itf=lifecycle-controller"));

        EasyMock.expect(launcher.retrieveJMXConnector(url, args)).andReturn(
            jmxConnector).anyTimes();
        EasyMock.expect(jmxConnector.getMBeanServerConnection()).andReturn(
            mBeanServerConnection).anyTimes();
        EasyMock.expect(
            mBeanServerConnection.queryNames(new ObjectName(
                "FC/Petals@*:itf=lifecycle-controller"), null)).andReturn(
            objectNames);
        launcher.invokeStopFc(mBeanServerConnection, new ObjectName(
            "FC/Petals@01:itf=lifecycle-controller"));

        EasyMock.replay(mBeanServerConnection);
        EasyMock.replay(launcher);
        EasyMock.replay(jmxConnector);

        String[] arguments = new String[] {"-host", "127.0.0.1", "-port",
            "16400"};
        launcher.stop(arguments);
    }

    public void testGetComponentByName() throws NoSuchInterfaceException {
        Launcher launcher = new Launcher();
        ContentController contentController = EasyMock
            .createMock(ContentController.class);
        Component component = EasyMock.createMock(Component.class);
        NameController nameController = EasyMock
            .createMock(NameController.class);

        Component[] compos = new Component[] {component};

        EasyMock.expect(contentController.getFcSubComponents()).andReturn(
            compos).anyTimes();
        EasyMock.expect(component.getFcInterface("name-controller")).andReturn(
            nameController).anyTimes();
        EasyMock.expect(nameController.getFcName()).andReturn("test")
            .anyTimes();

        EasyMock.replay(contentController);
        EasyMock.replay(component);
        EasyMock.replay(nameController);

        assertEquals(component, launcher.getComponentByName(contentController,
            "test"));
    }

    public void testGetComponentByNameException()
        throws NoSuchInterfaceException {
        Launcher launcher = new Launcher();
        ContentController contentController = EasyMock
            .createMock(ContentController.class);
        Component component = EasyMock.createMock(Component.class);

        Component[] compos = new Component[] {component};

        EasyMock.expect(contentController.getFcSubComponents()).andReturn(
            compos).anyTimes();
        EasyMock.expect(component.getFcInterface("name-controller")).andThrow(
            new NoSuchInterfaceException("name-controller"));

        EasyMock.replay(contentController);
        EasyMock.replay(component);

        assertNull(launcher.getComponentByName(contentController, "test"));
    }

    public void testRecurseBindings() throws NamingException {
        Launcher launcher = new Launcher();
        Context context = EasyMock.createMock(Context.class);
        Context subContext = EasyMock.createMock(Context.class);
        Map<String, Object> map = new HashMap<String, Object>();
        map.put("key", "value");
        LinkRef linkRef = new LinkRef("link");
        NamingEnumerationImpl<Binding> bindings = new NamingEnumerationImpl<Binding>(
            new Binding[] {new Binding("test", "test"),
                new Binding("cont", subContext), new Binding("map", map),
                new Binding("linkref", linkRef)});

        EasyMock.expect(context.listBindings("cont")).andThrow(
            new RuntimeException()).anyTimes();
        EasyMock.expect(context.list("cont")).andReturn(
            new NamingEnumerationImpl<NameClassPair>(new NameClassPair[0]));

        EasyMock.replay(context);
        EasyMock.replay(subContext);

        launcher.recurseBindings(context, bindings, "");
    }

    public void testRecurseNames() throws NamingException {
        Launcher launcher = new Launcher();
        Context context = EasyMock.createMock(Context.class);
        NamingEnumerationImpl<NameClassPair> names = new NamingEnumerationImpl<NameClassPair>(
            new NameClassPair[] {new NameClassPair("test", "test")});

        EasyMock.replay(context);

        launcher.recurseNames(context, names, "");
    }

    public void testRun() throws Exception {
        Launcher launcher = EasyMock.createMock(Launcher.class, new Method[] {
            Launcher.class.getDeclaredMethod("initializeMonolog",
                new Class[] {}),
            Launcher.class.getDeclaredMethod("getComponentByName", new Class[] {
                ContentController.class, String.class})});
        Component component = EasyMock.createMock(Component.class);
        ContentController contentController = EasyMock
            .createMock(ContentController.class);
        Component[] compos = new Component[] {component};
        NameController nameController = EasyMock
            .createMock(NameController.class);
        Component agent = EasyMock.createMock(Component.class);
        Admin admin = EasyMock.createMock(Admin.class);
        LifeCycleController lifeCycleController = EasyMock
            .createMock(LifeCycleController.class);
        AttributeController attributeController = EasyMock
            .createMock(AttributeController.class);
        AdminAttributes adminAttributes = EasyMock
            .createMock(AdminAttributes.class);
        MBeanServer mBeanServer = EasyMock.createMock(MBeanServer.class);
        Component jbi = EasyMock.createMock(Component.class);
        Component system = EasyMock.createMock(Component.class);
        SystemState systemState = EasyMock.createMock(SystemState.class);

        EasyMock.expect(component.getFcInterface("content-controller"))
            .andReturn(contentController).anyTimes();
        EasyMock.expect(component.getFcInterface("name-controller")).andReturn(
            nameController).anyTimes();
        EasyMock.expect(component.getFcInterface("lifecycle-controller"))
            .andReturn(lifeCycleController).anyTimes();

        EasyMock.expect(contentController.getFcSubComponents()).andReturn(
            compos).anyTimes();
        launcher.initializeMonolog();
        launcher.initializeFractal();
        EasyMock.expect(
            launcher.getComponentByName(contentController, "agent-jmx"))
            .andReturn(agent).anyTimes();
        EasyMock.expect(agent.getFcInterface("admin")).andReturn(admin);
        admin.expose();
        EasyMock.expect(agent.getFcInterface("attribute-controller"))
            .andReturn(adminAttributes);
        EasyMock.expect(adminAttributes.getRawMBeanServer()).andReturn(
            mBeanServer);
        EasyMock.expect(launcher.getComponentByName(contentController, "jbi"))
            .andReturn(jbi);
        EasyMock.expect(jbi.getFcInterface("content-controller")).andReturn(
            contentController);
        EasyMock.expect(
            launcher.getComponentByName(contentController, "systemstate-impl"))
            .andReturn(system);
        EasyMock.expect(system.getFcInterface("service"))
            .andReturn(systemState);
        EasyMock.expect(systemState.recoverAllEntities()).andReturn(
            Boolean.TRUE);
        EasyMock.expect(lifeCycleController.getFcState()).andReturn(
            LifeCycleController.STARTED).anyTimes();
        lifeCycleController.stopFc();

        EasyMock.replay(jbi);
        EasyMock.replay(systemState);
        EasyMock.replay(system);
        EasyMock.replay(adminAttributes);
        EasyMock.replay(lifeCycleController);
        EasyMock.replay(attributeController);
        EasyMock.replay(launcher);
        EasyMock.replay(agent);
        EasyMock.replay(admin);
        EasyMock.replay(nameController);
        EasyMock.replay(contentController);
        EasyMock.replay(component);

        launcher.petalsComposite = component;
        launcher.run(new String[0]);
    }

}
